@@ -90,7 +90,9 @@ extern nl_catd catd;
 
 /* OpenPTS default configulations  */
 
-#define PTSCD_CONFIG_FILE  "/etc/ptsc.conf"
+#define PTSC_CONFIG_FILE  "/etc/ptsc.conf"
+#define PTSV_CONFIG_FILE  "/etc/ptsv.conf"
+
 #define PTSC_GROUP_NAME    "ptsc"
 #define PTSC_IFM_TIMEOUT   5
 #define PTSC_IFM_TIMEOUT_MAX   30
@@ -148,6 +150,11 @@ extern nl_catd catd;
 #define OPENPTS_SELFTEST_FALLBACK  302
 #define OPENPTS_SELFTEST_FAILED    303
 
+// IMV
+#define IMV_ENROLLMENT_NONE          0
+#define IMV_ENROLLMENT_CREDENTIAL    1
+#define IMV_ENROLLMENT_AUTO          2
+
 
 /* structures */
 
@@ -441,10 +448,13 @@ typedef struct {
     TSS_VERSION pts_version;
 
     /* UUID */
-    OPENPTS_UUID * uuid;       /**< Platform(collector) UUID */
-    OPENPTS_UUID * rm_uuid;    /**< RM(now) UUID */
-    OPENPTS_UUID * newrm_uuid; /**< RM(next) UUID */
-    OPENPTS_UUID * oldrm_uuid; /**< RM(old/previous) UUID */
+    OPENPTS_UUID * uuid;         /**< Platform(collector) UUID */
+    OPENPTS_UUID * rm_uuid;      /**< RM(now) UUID */
+    OPENPTS_UUID * newrm_uuid;   /**< RM(next) UUID */
+    OPENPTS_UUID * oldrm_uuid;   /**< RM(old/previous) UUID */
+    OPENPTS_UUID * tmp_uuid;     /**< Platform(collector) UUID - changed */
+    OPENPTS_UUID * tmp_rm_uuid;  /**< RM(now) UUID - changed */
+
 
     /* Daemon UUID */
     PTS_UUID     *daemon_uuid;
@@ -522,6 +532,9 @@ typedef struct {
     char *ssh_port;
     int   ifm_timeout;
 
+    /* IF-M verifier(IMV) */
+    int enrollment;
+
 #ifdef CONFIG_AUTO_RM_UPDATE
     int enable_aru;            /**> Enable update scan */
     int update_exist;          /**> Update exist, used by collector */
@@ -620,6 +633,12 @@ typedef struct {
     UINT32 ifm_errno;  /**<  PTS error code */
     char * ifm_strerror;
 
+    OPENPTS_UUID *collector_uuid;
+    OPENPTS_UUID *rm_uuid;
+
+    /* TNC */
+    int tnc_state;
+
     /* misc ? */
     int cid; /**< */
     int count;  // TODO used by FSM, location is temp
@@ -653,6 +672,8 @@ int readFsmFromPropFile(OPENPTS_CONTEXT *ctx, char * filename);  // fsm.c -> ctx
 
 /* ifm.c */
 int writePtsTlv(OPENPTS_CONTEXT *ctx, int fd, int type);
+// int setTargetCapability(OPENPTS_CONTEXT *ctx, OPENPTS_IF_M_Capability *cap);
+
 
 /* collector.c */
 int collector(
@@ -715,12 +736,12 @@ int addEventToSnapshotBhv(
 int addEventToSnapshotBin(
     OPENPTS_CONTEXT * ctx, OPENPTS_PCR_EVENT_WRAPPER * eventWrapper);
 int getIml(OPENPTS_CONTEXT *ctx, int option);
-int getBiosImlFile(OPENPTS_CONTEXT *ctx, const char *filename, int mode);
-int getImaImlFile(
+int readBiosImlFile(OPENPTS_CONTEXT *ctx, const char *filename, int mode);
+int readImaImlFile(
     OPENPTS_CONTEXT * ctx,
     const char *filename,
     int type,
-    int mode);  // 20100613
+    int mode, int *count);  // 20100613
 int setPcrsToSnapshot(OPENPTS_CONTEXT *ctx, OPENPTS_PCRS *pcrs);
 int getPcrBySysfsFile(OPENPTS_CONTEXT *ctx, const char *filename);
 int validatePcr(OPENPTS_CONTEXT * ctx);
@@ -880,8 +901,6 @@ char * getStringOfUuid(PTS_UUID *uuid);
 PTS_UUID *getUuidFromString(char *str);
 PTS_DateTime * getDateTimeOfUuid(PTS_UUID *uuid);
 PTS_DateTime * getDateTime();
-int writeUuidFile(char *str_uuid, char *filename, int overwrite);
-int readUuidFile(char *filename, char **str_uuid, PTS_UUID **uuid);
 int getRmList(OPENPTS_CONFIG *conf, char * config_dir);
 int purgeRenewedRm(OPENPTS_CONFIG *conf);
 void printRmList(OPENPTS_CONFIG *conf, char *indent);
@@ -891,6 +910,7 @@ char *getTargetConfDir(OPENPTS_CONFIG *conf);
 OPENPTS_TARGET *getTargetCollector(OPENPTS_CONFIG *conf);
 /* OPENPTS_UUID */
 OPENPTS_UUID *newOpenptsUuid();
+OPENPTS_UUID *newOpenptsUuid2(PTS_UUID *pts_uuid);
 OPENPTS_UUID *newOpenptsUuidFromFile(char * filename);
 void freeOpenptsUuid(OPENPTS_UUID *uuid);
 int genOpenptsUuid(OPENPTS_UUID *uuid);
