@@ -36,6 +36,9 @@
 #include <openssl/dh.h>
 #include <arpa/inet.h>  // htonl ntohl
 
+#include <tnc/tncifmpts.h>
+
+#if 0
 typedef struct {
         PTS_Byte   flags;    //
         PTS_Byte   vid[3];   //
@@ -43,6 +46,7 @@ typedef struct {
         PTS_UInt32 length;   //  Network Byte Order (Big Endian)
         PTS_Byte*  value;    //
 } PTS_IF_M_Attribute;
+#endif
 
 /* SMI PEN */
 // ref,
@@ -50,8 +54,8 @@ typedef struct {
 //   http://www.iana.org/assignments/enterprise-numbers
 //     0 - 37250
 //
-#define PEN_RESERVED  0x000000  //     0
-#define PEN_IBM       0x000002  //     2  Kristine Adamson
+#define TNC_VENDORID_RESERVED  0x000000  //     0
+#define TNC_VENDORID_IBM       0x000002  //     2  Kristine Adamson
 //  4769 IBM
 //  5766 IBM Global Service
 //  5949 IBM AIX tools
@@ -59,8 +63,18 @@ typedef struct {
 // 19771 IBM Tivoli
 // 20301 IBM eServer X
 // 32715 IBM MSS
-#define PEN_TCG       0x005597  // 21911  Ned Smith admin&trustedcomputinggroup.org
-#define PEN_OPENPTS   0x000000  // TODO get new PEN or use TCG
+//       OpenPTS
+
+// /usr/include/tncif.h has been define TNC_VENDORID_TCG as 0
+#define TNC_VENDORID_TCG_PEN    0x005597  // 21911  Ned Smith admin&trustedcomputinggroup.org
+#define TNC_VENDORID_TNCFHH     0x0080AB  // 32939  Fachhochschule Hannover
+#define TNC_VENDORID_OSC        0x002358  //  9048  Open System Consultants (libtnc)
+#define TNC_VENDORID_OPENPTS    0x00950E  // 38158  OpenPTS
+
+#define TNC_SUBTYPE_TCG_PTS 0x01
+
+#define TNC_SUBTYPE_OPENPTS 0x01   // OpenPTS v0.2.x IF-M
+
 
 /* PTS IF-M Attribute Enumeration (OpenPTS) */
 // TODO TBD
@@ -92,6 +106,34 @@ typedef struct {
 #define AIDE_DATABASE                            0x00030000  // TBD
 #endif  // CONFIG_AIDE
 
+
+/* TNC stste */
+enum TNC_STATE {
+    /* normal attstation */
+    TNC_STATE_START,        // C->V
+    TNC_STATE_CAP,          // C<-V
+    TNC_STATE_NONCE,        // C<-V
+    TNC_STATE_IR,           // C->V
+
+    /* enrollemnt (auto) */
+    // START
+    TNC_STATE_KEY_REQ,       // C<-V
+    TNC_STATE_KEY,           // C->V
+    TNC_STATE_RM_REQ,        // C<-V
+    TNC_STATE_RM,            // C->V
+    TNC_STATE_NONCE_ENROLL,  // C<-V
+    TNC_STATE_IR_ENROLL,     // C->V
+    // NONCE
+    // IR
+
+//    TNC_STATE_CRED_REQ,
+//    TNC_STATE_CRED,
+//    TNC_STATE_DHN_REQ,
+//    TNC_STATE_DHN_PARM,
+//    TNC_STATE_DHN_FIN,
+    TNC_STATE_DONE
+};
+
 /* Capability flag[0] */
 #define OPENPTS_FLAG0_TPM_ERROR      0x01
 #define OPENPTS_FLAG0_TSS_ERROR      0x02
@@ -122,6 +164,7 @@ typedef struct {
 #define DH_HASH_SHA256  0x0002
 #define DH_HASH_SHA384  0x0004
 
+
 /* DH Nonce Parameters Request (4-bytes) */
 typedef struct {
         PTS_Byte   reserved;       //
@@ -209,6 +252,7 @@ int write_ir_message(int sock, int flag, int vid, char *ir_filename);
 PTS_IF_M_Attribute *readPtsTlv(int fd);
 // int writePtsTlv(OPENPTS_CONTEXT *ctx, int fd, int type);
 
+
 void freePtsTlv(PTS_IF_M_Attribute *tlv);
 
 /* nonce.c */
