@@ -26,7 +26,7 @@
  * \brief FSM action
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-06-19
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-20 SM
  *
  * FSM Action (UML2 doActivity)
  *
@@ -433,7 +433,7 @@ int setLinuxKernelCmdlineAssertion(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPP
 int validateKernelCmdline(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
     TODO("validateKernelCmdline - NA\n");
     updateProperty(ctx, "kernel.commandline", "TBD");
-    return PTS_SUCCESS; // -1;
+    return PTS_SUCCESS;
 }
 
 /* Linux - IMA */
@@ -704,7 +704,7 @@ int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *even
         }
         // TODO free md
         // freeAideMetadata(md);
-        free(name);
+        // free(name);
     } else if (ctx->conf->ima_validation_mode == OPENPTS_VALIDATION_MODE_IIDB) {
         ERROR("validateImaMeasurementNG w/ IIDB - NA\n");
     }
@@ -769,12 +769,23 @@ int startCollector(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper
     TSS_PCR_EVENT *event;
     OPENPTS_EVENT_COLLECTOR_START *start = NULL;
 
+    /* check */
+    if (ctx == NULL) {
+        ERROR("startCollector() - ctx is null\n");
+        return PTS_INTERNAL_ERROR;
+    }
+
     if (ctx->target_conf == NULL) {
         /* collector */
-        DEBUG("startCollector() - collector side - skip\n");
+        ERROR("startCollector() - target_conf is NULL, collector side - skip\n");
         return PTS_SUCCESS;
     }
 
+    if (ctx->target_conf->uuid == NULL) {
+        /* collector */
+        ERROR("startCollector() - uuid is NULL\n");
+        return PTS_SUCCESS;
+    }
 
     /* check */
     if (eventWrapper == NULL) {
@@ -833,15 +844,15 @@ int startCollector(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper
 
   error:
     /* Error */
-    // gen IR data to create the test
+    // printout the example IR data to create the test case
     {
         unsigned char b64[256];
 
         if (start == NULL) {
             start = malloc(sizeof(OPENPTS_EVENT_COLLECTOR_START));
         }
-
-        printHex("OPENPTS_EVENT_COLLECTOR_START", (unsigned char*)start, sizeof(OPENPTS_EVENT_COLLECTOR_START), "\n");
+        printHex("OPENPTS_EVENT_COLLECTOR_START",
+            (unsigned char*)start, sizeof(OPENPTS_EVENT_COLLECTOR_START), "\n");
         encodeBase64(b64, (unsigned char *)start, sizeof(OPENPTS_EVENT_COLLECTOR_START));
         ERROR("EventData: %s\n", b64);
 
@@ -849,9 +860,11 @@ int startCollector(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper
         memcpy(&start->collector_uuid, ctx->target_conf->uuid->uuid, 16);
         memcpy(&start->manifest_uuid, ctx->target_conf->rm_uuid->uuid, 16);
 
-        printHex("OPENPTS_EVENT_COLLECTOR_START", (unsigned char*)start, sizeof(OPENPTS_EVENT_COLLECTOR_START), "\n");
+        printHex("OPENPTS_EVENT_COLLECTOR_START",
+            (unsigned char*)start, sizeof(OPENPTS_EVENT_COLLECTOR_START), "\n");
         encodeBase64(b64, (unsigned char *)start, sizeof(OPENPTS_EVENT_COLLECTOR_START));
         ERROR("EventData: %s\n", b64);
+        free(start);
     }
 
     return rc;  // TODO
@@ -912,6 +925,7 @@ int addIntelTxtTbootProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *ev
                 data = (OPENPTS_EVENT_TBOOT_POLCTL *) event->rgbEvent;
                 buf = getHexString(data->pol_control, 4);
                 updateProperty(ctx, "intel.txt.tboot.pol.control.hex", buf);
+                free(buf);
                 buf = getHexString(data->pol_hash, 20);
                 updateProperty(ctx, "intel.txt.tboot.pol.hash.hex", buf);
                 free(buf);
@@ -1194,8 +1208,8 @@ int doActivity(
     int rc = 0;
     char *name = NULL;
     char *value = NULL;
-    char buf[BUF_SIZE];
-    int len;
+    char *buf = NULL;  // char buf[BUF_SIZE];
+    // int len;
     char *saveptr;
     int i;
 
@@ -1217,15 +1231,10 @@ int doActivity(
     }
 
     /* copy */
-    len = strlen(action);
-
-    if (len < BUF_SIZE) {
-        memset(buf, 0, BUF_SIZE);
-        memcpy(buf, action, len);
-        buf[len] = 0;
-    } else {
-        memcpy(buf, action, BUF_SIZE);
-        buf[BUF_SIZE-1] = 0;
+    buf = smalloc(action);
+    if (buf == NULL) {
+        ERROR("doActivity - no memory\n");
+        return PTS_FATAL;  // -1;
     }
 
     /* no action */
@@ -1301,6 +1310,7 @@ int doActivity(
     rc = OPENPTS_FSM_ERROR;
 
   end:
+    if (buf != NULL) free(buf);
     /* check the RC */
     if (rc == OPENPTS_FSM_ERROR) {
         DEBUG("doActivity rc = %d\n", rc);
