@@ -26,7 +26,7 @@
  * \brief AIDE I/F APIs
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-06-13
- * cleanup 2011-01-22 SM
+ * cleanup 2011-07-06 SM
  *
  * 1) Integrity check with AIDE
  *
@@ -175,7 +175,7 @@ AIDE_CONTEXT * newAideContext() {
     rc = hcreate_r(AIDE_HASH_TABLE_SIZE, ctx->aide_md_table);  // hash table for metadata
     if (rc == 0) {
         ERROR("hcreate faild, errno=%x\n", errno);
-        return NULL;
+        goto error;
     }
     ctx->aide_md_table_size = 0;
 
@@ -186,12 +186,16 @@ AIDE_CONTEXT * newAideContext() {
     rc = hcreate_r(AIDE_HASH_TABLE_SIZE, ctx->aide_in_table);  // hash table for ignore name
     if (rc == 0) {
         ERROR("hcreate faild\n");
-        return NULL;
+        goto error;
     }
     ctx->aide_in_table_size = 0;
 
     DEBUG("newAideContext %p\n", ctx);
     return ctx;
+
+  error:
+    if (ctx != NULL) free(ctx);
+    return NULL;
 }
 
 /**
@@ -375,8 +379,13 @@ int loadAideDatabaseFile(AIDE_CONTEXT *ctx, char *filename) {
 
                 /* find sep */
                 sep = strstr(ptr, " ");
-                *sep = 0;
-
+                if (sep == NULL) {
+                    ERROR("bad data, %s\n", buf);
+                    return -1;
+                } else {
+                    // terminate at " "
+                    *sep = 0;
+                }
                 /* get item code */
                 items[item_num] = getAideItemIndex(ptr);
 
@@ -415,7 +424,12 @@ int loadAideDatabaseFile(AIDE_CONTEXT *ctx, char *filename) {
                 if (i != item_num - 1) {
                     // printf("SEP %d %d\n",i, item_num);
                     sep = strstr(ptr, " ");
-                    *sep = 0;  // set \0
+                    if (sep == NULL) {
+                        ERROR("bad data, %s\n", buf);
+                        return -1;
+                    } else {
+                        *sep = 0;  // set \0
+                    }
                 }
 
                 /* check the null string*/
@@ -1093,12 +1107,12 @@ int escapeFilename(char **out, char *in) {
  */
 int convertImlToAideDbFile(OPENPTS_CONTEXT *ctx, char *filename) {
     gzFile fp;
-    int i;
+    int i = 0;
     OPENPTS_SNAPSHOT *ss;
     OPENPTS_PCR_EVENT_WRAPPER *eventWrapper;
     TSS_PCR_EVENT *event;
     unsigned char buf[128];  // TODO(munetoh)
-    char *aide_filename;
+    char *aide_filename = NULL;
     int len;
 
     DEBUG("convertImlToAideDbFile %s\n", filename);
@@ -1117,11 +1131,14 @@ int convertImlToAideDbFile(OPENPTS_CONTEXT *ctx, char *filename) {
 
     /* IMLs */
     ss = getSnapshotFromTable(ctx->ss_table, 10, 1);  // TODO def or conf
+    if (ss == NULL) {
+        ERROR("events is missing\n");
+        goto close;
+    }
     eventWrapper = ss->start;
-
     if (eventWrapper == NULL) {
         ERROR("events is missing\n");
-        return -1;
+        goto close;
     }
 
     event = eventWrapper->event;
@@ -1136,12 +1153,12 @@ int convertImlToAideDbFile(OPENPTS_CONTEXT *ctx, char *filename) {
 
         if (event == NULL) {
             ERROR("event is NULL\n");
-            return -1;
+            goto close;
         }
 
         if (event->rgbEvent == NULL) {
             ERROR("event->rgbEvent is NULL\n");
-            return -1;
+            goto close;
         }
 
         // TODO 2010-10-05 SM
@@ -1158,7 +1175,7 @@ int convertImlToAideDbFile(OPENPTS_CONTEXT *ctx, char *filename) {
 
         // gzprintf(fp, "%s ",&eventWrapper->event->rgbEvent[20]);
 
-        /* filename */
+        /* filename (allocated) */
         len = escapeFilename(&aide_filename, (char *) &eventWrapper->event->rgbEvent[20]);
         if (len < 0) {
             ERROR("convertImlToAideDbFile - no mem?\n");
@@ -1166,6 +1183,7 @@ int convertImlToAideDbFile(OPENPTS_CONTEXT *ctx, char *filename) {
         } else {
             gzprintf(fp, "%s ", aide_filename);
             free(aide_filename);
+            aide_filename = NULL;
         }
 
         /* digest */
@@ -1184,7 +1202,9 @@ int convertImlToAideDbFile(OPENPTS_CONTEXT *ctx, char *filename) {
 
     /* file close */
     gzseek(fp, 1L, SEEK_CUR);  // add one \n
+  close:
     gzclose(fp);
+    if (aide_filename != NULL) free(aide_filename);
 
     DEBUG("convertImlToAideDbFile - done\n");
 
