@@ -26,7 +26,7 @@
  * \brief TCG IF-M collector functions
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2011-01-06
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-20 SM
  *
  * move from ptscd.c 
  *
@@ -299,6 +299,9 @@ int init(
 
     /* save to rm_uuid file */
     rc = writeOpenptsUuidFile(conf->rm_uuid, 0);  // do not overwrite
+    if (rc != PTS_SUCCESS) {
+        ERROR("writeOpenptsUuidFile fail\n");
+    }
     // TODO check rc
 
     /* RM set DIR */
@@ -325,7 +328,7 @@ int init(
         // TODO change to generic name?  conf->iml_filename[0]  conf->iml_filename[1]
         /* from  securityfs */
         /* BIOS IML */
-        rc = getBiosImlFile(
+        rc = readBiosImlFile(
                 ctx,
                 conf->bios_iml_filename, conf->iml_endian);
         if (rc != PTS_SUCCESS) {
@@ -337,11 +340,12 @@ int init(
 
         /* RUNTIME IML (Linux-IMA) */
         if (ctx->conf->runtime_iml_filename != NULL) {
-            rc = getImaImlFile(
+            int count;
+            rc = readImaImlFile(
                     ctx,
                     conf->runtime_iml_filename,
-                    conf->runtime_iml_type, 0);  // TODO endian?
-            if (rc < 0) {
+                    conf->runtime_iml_type, 0, &count);  // TODO endian?
+            if (rc != PTS_SUCCESS) {
                 fprintf(stderr, "read IMA IML, %s was failed\n", conf->runtime_iml_filename);
                 rc = PTS_INTERNAL_ERROR;
                 goto free;
@@ -359,8 +363,12 @@ int init(
         if (conf->rm_filename[i] != NULL) {
             rc = writeRm(ctx, conf->rm_filename[i], i);
             if (rc != PTS_SUCCESS) {
-                fprintf(stderr, "write RM, %s was failed\n", conf->rm_filename[i]);
-                rc = PTS_INTERNAL_ERROR;
+                fprintf(stderr, "ERROR, initialization was failed\n");
+                addReason(ctx,
+                    "[INIT] Failed to create the manifest file, %s",
+                    conf->rm_filename[i]);
+                printReason(ctx);
+                rc = PTS_FATAL;
                 goto free;
             }
             printf("level %d Reference Manifest  : %s\n", i, conf->rm_filename[i]);
@@ -540,6 +548,10 @@ int selftest(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start,
                 DEBUG("use UUID=%s\n", conf->rm_uuid->str);
                 /* update rm_uuid */
                 rc = writeOpenptsUuidFile(conf->rm_uuid, 1);
+                if (rc != PTS_SUCCESS) {
+                    ERROR("writeOpenptsUuidFile fail\n");
+                }
+
                 // TODO check rc
                 /* delete newrm_uuid */
                 rc = remove(conf->newrm_uuid->filename);
@@ -637,6 +649,9 @@ int newrm(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start, OP
 
     /* save/update rm_uuid file */
     rc = writeOpenptsUuidFile(conf->rm_uuid, 1);  // TODO overwite?
+    if (rc != PTS_SUCCESS) {
+        ERROR("writeOpenptsUuidFile fail\n");
+    }
 
     /* RM set DIR */
     rc = makeRmSetDir(conf);
@@ -662,7 +677,7 @@ int newrm(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start, OP
         // TODO change to generic name?  conf->iml_filename[0]  conf->iml_filename[1]
         /* from  securityfs */
         /* BIOS IML */
-        rc = getBiosImlFile(
+        rc = readBiosImlFile(
                 ctx,
                 conf->bios_iml_filename, conf->iml_endian);
         if (rc != PTS_SUCCESS) {
@@ -674,11 +689,12 @@ int newrm(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start, OP
 
         /* RUNTIME IML (Linux-IMA) */
         if (ctx->conf->runtime_iml_filename != NULL) {
-            rc = getImaImlFile(
+            int count;
+            rc = readImaImlFile(
                     ctx,
                     conf->runtime_iml_filename,
-                    conf->runtime_iml_type, 0);  // TODO endian?
-            if (rc < 0) {
+                    conf->runtime_iml_type, 0, &count);  // TODO endian?
+            if (rc != PTS_SUCCESS) {
                 fprintf(stderr, "read IMA IML, %s was failed\n", conf->runtime_iml_filename);
                 rc = PTS_INTERNAL_ERROR;
                 goto free;
