@@ -27,7 +27,7 @@
  *
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-08-13
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-06 SM
  *
  *
  * grep strncmp src/conf.c | gawk '{print $3}'
@@ -346,7 +346,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
     char *eq;
     char *name;
     char *value = NULL;
-    char *config_path = NULL;
+    // char *config_path = NULL;
     int cnt = 1;
     int len;
     char *path;
@@ -359,41 +359,44 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
         return PTS_INTERNAL_ERROR;
     }
 
-    /* config filename -> fullpath */
+    /* config filename -> fullpath -> filename2 */
     if (filename[0] != '/') {
-        /* get fullpath */
+        /* => get fullpath */
         path = getenv("PWD");
         if (path[0] != '/') {
             ERROR("readPtsConfig() - path, '%s' is not a full path", path);
         }
         filename2 = getFullpathName(path, filename);
     } else {
+        /* started by /, seems full path */
         filename2 = smalloc(filename);
     }
+    if (filename2 == NULL) {
+        ERROR("no memory?\n");
+        return PTS_INTERNAL_ERROR;
+    }
 
     /* set config filename (fullpath) to conf*/
     if (conf->config_file != NULL) {
-        // DEBUG("realloc conf->config_file \n");  // TODO realloc happen
+        /* replace, free old conf path */
         free(conf->config_file);
     }
     conf->config_file = smalloc(filename2);
 
     /* dir where config file -> config_dir */
     if (conf->config_dir != NULL) {
-        config_path = getFullpathDir(filename2);
+        // free old one
         free(conf->config_dir);
-        conf->config_dir = config_path;
-    } else {
-        conf->config_dir = getFullpathDir(filename2);
-        config_path  = conf->config_dir;
     }
+    conf->config_dir = getFullpathDir(filename2);
+    // config_path  = conf->config_dir;
 
 
     /* open */
-
     if ((fp = fopen(filename2, "r")) == NULL) {
         ERROR("readPtsConfig - File %s open was failed\n", filename2);
-        return PTS_INTERNAL_ERROR;
+        rc = PTS_INTERNAL_ERROR;
+        goto free;
     }
 
     /* parse */
@@ -415,14 +418,22 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
 
             //  DEBUG("%4d [%s]=[%s]\n",cnt, name, value);
 
-            /* config dir */
+            /* config dir
+               replace the curent setting  based on the location of config file
+               to path set by config file.
+            */
             if (!strncmp(name, "config.dir", 10)) {
                 DEBUG("conf dir                   : %s\n", value);
-                if (conf->config_dir != NULL) {  // True
+                if (value[0] != '/') {
+                    /* => get fullpath */
+                    path = getFullpathName(conf->config_dir, value);
                     free(conf->config_dir);
+                    conf->config_dir = path;
+                } else {
+                    /* started by /, seems full path, just replace */
+                    free(conf->config_dir);
+                    conf->config_dir = smalloc(value);
                 }
-                // conf->config_dir = getFullpathDir(value);
-                conf->config_dir = getFullpathName(config_path, value);
             }
 
             /* openpts_pcr_index */
@@ -438,7 +449,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 } else if (!strncmp(value, "tss", 3)) {
                     conf->iml_mode = 0;
                 } else {
-                    ERROR("\n");  // TODO
+                    ERROR("TBD\n");  // TODO
                 }
             }
 
@@ -513,12 +524,14 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
 
             /* BIOS IML */
             if (!strncmp(name, "bios.iml.file", 13)) {
-                conf->bios_iml_filename = getFullpathName(config_path, value);
+                // conf->bios_iml_filename = getFullpathName(config_path, value);
+                conf->bios_iml_filename = getFullpathName(conf->config_dir, value);
                 DEBUG("conf->bios_iml_filename    : %s\n", conf->bios_iml_filename);
             }
             /* RUNTIME IML */
             if (!strncmp(name, "runtime.iml.file", 16)) {
-                conf->runtime_iml_filename = getFullpathName(config_path, value);
+                // conf->runtime_iml_filename = getFullpathName(config_path, value);
+                conf->runtime_iml_filename = getFullpathName(conf->config_dir, value);
                 DEBUG("conf->runtime_iml_filename : %s\n", conf->runtime_iml_filename);
             }
             if (!strncmp(name, "runtime.iml.type", 16)) {
@@ -534,7 +547,8 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
             }
             /* PCR */
             if (!strncmp(name, "pcrs.file", 9)) {
-                conf->pcrs_filename = getFullpathName(config_path, value);
+                // conf->pcrs_filename = getFullpathName(config_path, value);
+                conf->pcrs_filename = getFullpathName(conf->config_dir, value);
                 DEBUG("conf->pcrs_filename        : %s\n", conf->pcrs_filename);
             }
 
@@ -544,7 +558,8 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->rm_basedir");  // TODO realloc happen
                     free(conf->rm_basedir);
                 }
-                conf->rm_basedir = getFullpathName(config_path, value);
+                // conf->rm_basedir = getFullpathName(config_path, value);
+                conf->rm_basedir = getFullpathName(conf->config_dir, value);
             }
             if (!strncmp(name, "rm.num", 6)) {
                 conf->rm_num = atoi(value);
@@ -560,7 +575,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->ir_filename");  // TODO realloc happen
                     free(conf->ir_filename);
                 }
-                conf->ir_filename = getFullpathName(config_path, value);
+                conf->ir_filename = getFullpathName(conf->config_dir, value);
                 DEBUG("conf->ir_filename          : %s\n", conf->ir_filename);
             }
             /* IR dir (collector side) */
@@ -569,7 +584,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->ir_filename");  // TODO realloc happen
                     free(conf->ir_dir);
                 }
-                conf->ir_dir = getFullpathName(config_path, value);
+                conf->ir_dir = getFullpathName(conf->config_dir, value);
                 DEBUG("conf->ir_filename          : %s\n", conf->ir_dir);
             }
 
@@ -578,7 +593,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->prop_filename");  // TODO realloc happen
                     free(conf->prop_filename);
                 }
-                conf->prop_filename = getFullpathName(config_path, value);
+                conf->prop_filename = getFullpathName(conf->config_dir, value);
             }
 
             // 20100908 Munetoh -> ifm.c
@@ -591,7 +606,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
 
             /* models */
             if (!strncmp(name, "model.dir", 10)) {
-                conf->model_dir = getFullpathName(config_path, value);
+                conf->model_dir = getFullpathName(conf->config_dir, value);
             }
 
             /* Verifier */
@@ -599,7 +614,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 if (conf->verifier_logging_dir != NULL) {
                     free(conf->verifier_logging_dir);
                 }
-                conf->verifier_logging_dir = getFullpathName(config_path, value);
+                conf->verifier_logging_dir = getFullpathName(conf->config_dir, value);
             }
 
 
@@ -608,7 +623,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->policy_filename\n");  // TODO realloc happen
                     free(conf->policy_filename);
                 }
-                conf->policy_filename = getFullpathName(config_path, value);
+                conf->policy_filename = getFullpathName(conf->config_dir, value);
             }
 
 #if 0
@@ -638,11 +653,11 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->aide_database_filename\n");   // TODO realloc happen
                     free(conf->aide_database_filename);
                 }
-                conf->aide_database_filename = getFullpathName(config_path, value);
+                conf->aide_database_filename = getFullpathName(conf->config_dir, value);
             }
 #ifdef CONFIG_SQLITE
             if (!strncmp(name, "aide.sqlite.file", 18)) {
-                conf->aide_sqlite_filename = getFullpathName(config_path, value);
+                conf->aide_sqlite_filename = getFullpathName(conf->config_dir, value);
             }
 #endif
             if (!strncmp(name, "aide.ignorelist.file", 20)) {
@@ -650,7 +665,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->aide_ignorelist_filename\n");   // TODO realloc happen
                     free(conf->aide_ignorelist_filename);
                 }
-                conf->aide_ignorelist_filename = getFullpathName(config_path, value);
+                conf->aide_ignorelist_filename = getFullpathName(conf->config_dir, value);
             }
 #endif  // CONFIG_AIDE
 
@@ -659,7 +674,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 if (conf->uuid == NULL) {
                     conf->uuid = newOpenptsUuid();
                 }
-                conf->uuid->filename = getFullpathName(config_path, value);
+                conf->uuid->filename = getFullpathName(conf->config_dir, value);
                 conf->uuid->status = OPENPTS_UUID_FILENAME_ONLY;
                 rc = readOpenptsUuidFile(conf->uuid);
                 if (rc != PTS_SUCCESS) {
@@ -667,10 +682,10 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // TODO gen UUID?
                     //  DEBUG("no UUID file %s\n", conf->uuid->filename);
                     conf->uuid->status = OPENPTS_UUID_FILENAME_ONLY;
+                    DEBUG("conf->uuid                 : not initialized\n");
                 } else {
-                    //  DEBUG("read UUID from file %s, UUID=%s\n", conf->uuid->filename, conf->uuid->str);
+                    DEBUG("conf->uuid->str            : %s\n", conf->uuid->str);
                 }
-                DEBUG("conf->uuid->str            : %s\n", conf->uuid->str);
             } else if (!strncmp(name, "uuid", 4)) {
                 ERROR("uuid=XXX is deprecated, in %s\n", filename);
                 if (conf->uuid == NULL) {
@@ -700,7 +715,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->rm_uuid->filename");  // TODO realloc happen
                     free(conf->rm_uuid->filename);
                 }
-                conf->rm_uuid->filename = getFullpathName(config_path, value);
+                conf->rm_uuid->filename = getFullpathName(conf->config_dir, value);
                 conf->rm_uuid->status = OPENPTS_UUID_FILENAME_ONLY;
                 rc = readOpenptsUuidFile(conf->rm_uuid);
                 if (rc != PTS_SUCCESS) {
@@ -724,7 +739,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->rm_uuid->filename");  // TODO realloc happen
                     free(conf->newrm_uuid->filename);
                 }
-                conf->newrm_uuid->filename = getFullpathName(config_path, value);
+                conf->newrm_uuid->filename = getFullpathName(conf->config_dir, value);
                 conf->newrm_uuid->status = OPENPTS_UUID_FILENAME_ONLY;
                 rc = readOpenptsUuidFile(conf->newrm_uuid);
                 if (rc != PTS_SUCCESS) {
@@ -735,6 +750,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 } else {
                     // DEBUG("read UUID from file %s, UUID=%s\n", conf->uuid->filename, conf->uuid->str);
                 }
+                DEBUG("conf->newrm_uuid->str      : %s\n", conf->newrm_uuid->str);
             }
             /* OLDRM UUID for RM set */
             if (!strncmp(name, "oldrm.uuid.file", 15)) {
@@ -745,7 +761,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     // DEBUG("realloc conf->oldrm_uuid->filename");  // TODO realloc happen
                     free(conf->oldrm_uuid->filename);
                 }
-                conf->oldrm_uuid->filename = getFullpathName(config_path, value);
+                conf->oldrm_uuid->filename = getFullpathName(conf->config_dir, value);
                 conf->oldrm_uuid->status = OPENPTS_UUID_FILENAME_ONLY;
                 rc = readOpenptsUuidFile(conf->oldrm_uuid);
                 if (rc != PTS_SUCCESS) {
@@ -756,6 +772,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 } else {
                     // DEBUG("read UUID from file %s, UUID=%s\n", conf->uuid->filename, conf->uuid->str);
                 }
+                DEBUG("conf->oldrm_uuid->str      : %s\n", conf->oldrm_uuid->str);
             }
 
             /* */
@@ -766,19 +783,19 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 }
                 conf->target_uuid = getUuidFromString(value);
                 if (conf->target_uuid == NULL) {
-                    ERROR("read UUID fail\n");
-                }
-
-                if (conf->str_target_uuid != NULL) {
-                    // DEBUG("realloc conf->str_target_uuid\n");  // TODO realloc happen
-                    free(conf->str_target_uuid);
-                }
-                conf->str_target_uuid = getStringOfUuid(conf->target_uuid);
-                if (conf->str_target_uuid == NULL) {
-                    ERROR("read UUID fail\n");
+                    ERROR("bad UUID ? %s\n", value);
+                } else {
+                    // add string too
+                    if (conf->str_target_uuid != NULL) {
+                        // DEBUG("realloc conf->str_target_uuid\n");  // TODO realloc happen
+                        free(conf->str_target_uuid);
+                    }
+                    conf->str_target_uuid = getStringOfUuid(conf->target_uuid);
+                    if (conf->str_target_uuid == NULL) {
+                        ERROR("bad UUID ? %s\n", value);
+                    }
                 }
             }
-
             /* PUBKEY */
             if (!strncmp(name, "target.pubkey", 13)) {
                 // TODO used by two
@@ -858,31 +875,56 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 }
             }
 
+            /* PTSV Enrollment */
+            if (!strncmp(name, "enrollment", 10)) {
+                if (!strncmp(value, "none", 4)) {
+                    conf->enrollment = IMV_ENROLLMENT_NONE;
+                    DEBUG("conf->enrollment           : none\n");
+                } else if (!strncmp(value, "credential", 10)) {
+                    conf->enrollment = IMV_ENROLLMENT_CREDENTIAL;
+                    DEBUG("conf->enrollment           : credential\n");
+                } else if (!strncmp(value, "auto", 4)) {
+                    conf->enrollment = IMV_ENROLLMENT_AUTO;
+                    DEBUG("conf->enrollment           : auto\n");
+                } else {
+                    ERROR("unknown enrollment %s\n", value);  // TODO
+                    conf->enrollment = 0;
+                }
+            }
+
             cnt++;
         } else {
             // TODO
         }
     }
 
-    fclose(fp);
-
-    if (filename2 != NULL) free(filename2);
-
     if (conf->verifier_logging_dir == NULL) {
+        /* set default logging dir */
         conf->verifier_logging_dir = smalloc("~/.openpts");
     }
 
+#if 0
     if (conf->uuid != NULL) {
         DEBUG("conf->uuid->filename       : %s\n", conf->uuid->filename);
     } else {
-        DEBUG("conf->uuid->filename       : uuid is not initialized\n");
+        // DEBUG("conf->uuid->filename       : uuid is not initialized\n");
     }
     if (conf->rm_uuid != NULL) {
         DEBUG("conf->rm_uuid->filename    : %s\n", conf->rm_uuid->filename);
     } else {
-        DEBUG("conf->rm_uuid->filename    : rm_uuid is not initialized\n");
+        // DEBUG("conf->rm_uuid->filename    : rm_uuid is not initialized\n");
     }
-    return PTS_SUCCESS;
+#endif
+
+    rc =  PTS_SUCCESS;
+
+  // close:
+    fclose(fp);
+
+  free:
+    if (filename2 != NULL) free(filename2);
+
+    return rc;
 }
 
 
