@@ -26,7 +26,7 @@
  * \brief PTS context
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-01
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-06 SM
  *
  * OpenPTS main context
  *
@@ -205,7 +205,7 @@ int readFsmFromPropFile(OPENPTS_CONTEXT *ctx, char * filename) {
 
     char buf[FSM_BUF_SIZE];
     char buf2[FSM_BUF_SIZE];
-    char *np = NULL;
+    // char *np = NULL;
     char *eqp = NULL;
     int pcr_index;
     int level;
@@ -244,14 +244,14 @@ int readFsmFromPropFile(OPENPTS_CONTEXT *ctx, char * filename) {
             model_filename = NULL;
 
 #if 1
-            // Using config file <= version 0.2.3 
-            if ((np = strstr(buf, "platform.model.")) != NULL) {
+            // Using config file <= version 0.2.3
+            if (strstr(buf, "platform.model.") != NULL) {
                 ERROR("ptsc.conf has old format <=v0.2.3 %s\n", filename);
                 ERROR("change platform.model to rm.model.0\n");
                 goto error;
             }
 
-            if ((np = strstr(buf, "runtime.model.")) != NULL) {
+            if (strstr(buf, "runtime.model.") != NULL) {
                 ERROR("ptsc.conf has old format <=v0.2.3 %s\n", filename);
                 ERROR("change runtime.model to rm.model.1\n");
                 goto error;
@@ -262,7 +262,7 @@ int readFsmFromPropFile(OPENPTS_CONTEXT *ctx, char * filename) {
             // 01234567890123456
             // rm.model.0.pcr.7
             // if ((np = strstr(buf, "rm.model.")) != NULL) {  // 11
-            np = 0;
+            // np = 0;
             if (!strncmp(buf, "rm.model.", 9)) {
                 level = (int) strtol(&buf[9], NULL, 10);
                 pcr_index = (int) strtol(&buf[15], NULL, 10);
