@@ -27,7 +27,7 @@
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-01
  * cleanup 2011-01-21 SM
- * refactoring 2011-02-14 SM
+ * refactoring 2011-07-20 SM
  * 
  * Input
  *   FSM Model
@@ -285,7 +285,7 @@ int getTypeFlag(char * cond, UINT32 *eventtype) {
     char * loc;
     int len;
     int rc = 0;
-    long int val;  // TODO uint64_t
+    long int val;  // TODO uint64_t but fail to build on i386 platform
 
     len = strlen(cond);
 
@@ -466,6 +466,8 @@ int getDigestFlag(char * cond, BYTE **digest, int *digest_size) {
  *   COUNTER_FLAG_LT   1:  < name
  *   COUNTER_FLAG_GTE  2:  >= name 
  *
+ *   set counter name to name 
+ *
  * Unit Test : check_fsm.c / test_getCounterFlag
  */
 int getCounterFlag(char * cond, char **name) {
@@ -476,6 +478,12 @@ int getCounterFlag(char * cond, char **name) {
 
     // DEBUG_CAL("getCounterFlag\n");
 
+    /* check */
+    if (cond == NULL) {
+        ERROR("getCounterFlag()");
+        return COUNTER_FLAG_SKIP;
+    }
+
     len = strlen(cond);
     loc = strstr(cond, "count");
 
@@ -527,14 +535,11 @@ int getCounterFlag(char * cond, char **name) {
             }
         }
 
-        // TODO check the end, this code only support if conter is the last
+        // TODO check the end, this code only support if counter is the last equation
 
         loc2 = loc;
         len = strlen(loc2);
 
-        // DEBUG("[%d][%s]\n",len, loc2);
-
-
         *name = malloc(len + 1);
         if (*name == NULL) {
             ERROR("no memory\n");
@@ -543,9 +548,7 @@ int getCounterFlag(char * cond, char **name) {
         memset(*name, 0, len + 1);
         memcpy(*name, loc2, len);
     }
-
-    DEBUG_FSM("getCounerFlag  %s #=> %d %s\n", cond, rc, *name);
-
+    DEBUG_FSM("getCounterFlag cond=[%s],  prop=[%s] rc=%d (0:1:2=skip:<:>=),\n", cond,  *name, rc);
     return rc;
 }
 
@@ -689,7 +692,7 @@ int addFsmTransition(
             memset(ptr, 0, sizeof(OPENPTS_FSM_Transition));
             memcpy(ptr->source, source, FSM_BUF_SIZE);
             memcpy(ptr->target, target, FSM_BUF_SIZE);
-            memcpy(ptr->cond, cond, FSM_BUF_SIZE);
+            // memcpy(ptr->cond, cond, FSM_BUF_SIZE);
             ptr->num = ctx->transition_num;
             if (cond == NULL) {
                 ptr->eventTypeFlag = 0;
@@ -697,6 +700,7 @@ int addFsmTransition(
             } else if  (cond[0] == 0) {
                 ptr->eventTypeFlag = 0;
                 ptr->digestFlag = 0;
+                memcpy(ptr->cond, cond, FSM_BUF_SIZE);
             } else {
                 // 0:don't care, 1:care
                 ptr->eventTypeFlag = getTypeFlag(cond, &ptr->eventType);
@@ -706,6 +710,7 @@ int addFsmTransition(
                 ptr->counter_flag = getCounterFlag(cond, &ptr->counter_name);
                 // 0:don't care 1: ==last 2: != last
                 ptr->last_flag = getLastFlag(cond);
+                memcpy(ptr->cond, cond, FSM_BUF_SIZE);
             }
             /* subvertex link (ptr) */
             ptr->source_subvertex = getSubvertex(ctx, ptr->source);
@@ -830,7 +835,8 @@ int updateFsm(
     DEBUG_CAL("updateFsm - start\n");
 
     if (curr_state == NULL) {
-        DEBUG_FSM("updateFsm - set FSM state to 'Start'\n");
+        DEBUG_FSM("[RM%02d-PCR%02d] updateFsm() - curr_state == NULL => set the FSM state to 'Start'\n",
+            fsm->level, fsm->pcr_index);
         curr_state = getSubvertex(fsm, "Start");
     }
 
@@ -886,7 +892,7 @@ int updateFsm(
         fsm->level, fsm->pcr_index,
         curr_state->name, curr_state->action);
 
-    if ((event != NULL) && (verbose | DEBUG_FSM_FLAG)) {
+    if ((event != NULL) && (verbose & DEBUG_FSM_FLAG)) {
         hex = getHexString(event->rgbPcrValue, event->ulPcrValueLength);
         DEBUG_FSM("[RM%02d-PCR%02d] eventtype=0x%x, digest=0x%s\n",
             fsm->level, fsm->pcr_index,
@@ -1091,10 +1097,12 @@ int updateFsm(
                     count = getCountFromProperty(ctx, trans->counter_name);
 
                     if (ctx->count < count) {
-                        DEBUG_FSM("COUNTER %d < %d - HIT\n", ctx->count, count);
+                        DEBUG_FSM("COUNTER[%s] %d < %d - HIT\n",
+                            trans->counter_name, ctx->count, count);
                         counter_check = 1;  // HIT
                     } else {
-                        DEBUG_FSM("COUNTER %d < %d - MISS\n", ctx->count, count);
+                        DEBUG_FSM("COUNTER[%s] %d < %d - MISS\n",
+                            trans->counter_name, ctx->count, count);
                         counter_check = -1;  // MISS
                     }
 
@@ -1105,12 +1113,14 @@ int updateFsm(
 
                     // TODO at this moment we ignore >= condition,
                     if (ctx->count >= count) {
-                        DEBUG_FSM("[RM%02d-PCR%02d] COUNTER %d >=  %d - HIT\n",
+                        DEBUG_FSM("[RM%02d-PCR%02d] COUNTER[%s] %d >=  %d - HIT\n",
                             fsm->level, fsm->pcr_index,
+                            trans->counter_name,
                             ctx->count, count);
                         counter_check = 1;  // HIT
                     } else {
-                        DEBUG_FSM("COUNTER %d >=  %d - MISS\n", ctx->count, count);
+                        DEBUG_FSM("COUNTER[%s] %d >=  %d - MISS\n",
+                            trans->counter_name, ctx->count, count);
                         counter_check = -1;  // MISS
                     }
                 } else {
@@ -1194,9 +1204,10 @@ int updateFsm(
                         break;
                     } else {
                         /* Trans */
-                        DEBUG_FSM("[RM%02d-PCR%02d] %s -> %s - HIT (type=%d, digest=%d)\n",
+                        DEBUG_FSM("[RM%02d-PCR%02d] %s -> %s - HIT (type=%d, digest=%d) <== new curr_state\n",
                             fsm->level, fsm->pcr_index,
                             trans->source, trans->target, type_check, digest_check);
+
                         fsm->curr_state = getSubvertex(fsm, trans->target);
 
                         if (fsm->curr_state != NULL) {
@@ -1240,6 +1251,10 @@ int updateFsm(
                                 // DEBUG("rc = %d -> 0\n");  // fsm.c:1070 rc = 6 -> 0
                                 rc = OPENPTS_FSM_SUCCESS;
                             }
+                        } else {
+                            ERROR("curr_state is NULL\n");
+                            rc = OPENPTS_FSM_ERROR;
+                            return rc;
                         }
                         break;
                     }
@@ -1273,7 +1288,7 @@ int updateFsm(
  *   called from rm.c
  */
 OPENPTS_FSM_CONTEXT *copyFsm(OPENPTS_FSM_CONTEXT *src_fsm) {
-    OPENPTS_FSM_CONTEXT * dst_fsm;
+    OPENPTS_FSM_CONTEXT * dst_fsm = NULL;
 
     OPENPTS_FSM_Subvertex  *src_fsm_sub;
     OPENPTS_FSM_Subvertex  *dst_fsm_sub = NULL;
@@ -1308,7 +1323,7 @@ OPENPTS_FSM_CONTEXT *copyFsm(OPENPTS_FSM_CONTEXT *src_fsm) {
     src_fsm_sub = src_fsm->fsm_sub;
     if (src_fsm_sub == NULL) {
         ERROR("ERROR No FSM SUB\n");
-        return NULL;
+        goto error;
     }
 
     while (src_fsm_sub != NULL) {
@@ -1346,7 +1361,7 @@ OPENPTS_FSM_CONTEXT *copyFsm(OPENPTS_FSM_CONTEXT *src_fsm) {
 
     if (src_fsm_trans == NULL) {
         ERROR("ERROR No FSM TRANS\n");
-        return NULL;
+        goto error;
     }
 
     while (src_fsm_trans != NULL) {
@@ -1395,6 +1410,12 @@ OPENPTS_FSM_CONTEXT *copyFsm(OPENPTS_FSM_CONTEXT *src_fsm) {
     DEBUG_FSM("copyFsm - done\n");
 
     return dst_fsm;
+
+  error:
+    if (dst_fsm != NULL) {
+        free(dst_fsm);
+    }
+    return NULL;
 }
 
 /**
