@@ -26,7 +26,7 @@
  * \brief TCG TNC IF-IMC v1.2 R8
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-05-07
- * cleanup 2011-01-22 SM
+ * cleanup 2011-07-06 SM
  *
  * http://www.trustedcomputinggroup.org/resources/tnc_ifimc_specification
  * http://www.trustedcomputinggroup.org/files/resource_files/8CB977E1-1D09-3519-AD48484530EF6639/TNC_IFIMC_v1_2_r8.pdf
@@ -57,22 +57,10 @@
 #include <openpts.h>
 
 #include <tncifimc.h>
-#include <libtnc.h>  // TODO(munetoh) remove this
 
-// TODO(munetoh) Fix Makefile.am
-#include "ifm.c"
-#include "iml.c"
-#include "ir.c"
-#include "base64.c"
-#include "tpm.c"
-#include "ctx.c"
-#include "uml.c"
-#include "fsm.c"
-#include "rm.c"
-#include "log.c"
+// ifm.c
+char* getPtsTlvMessage(OPENPTS_CONTEXT *ctx, int type, int *len);
 
-// TODO(munetoh) for TEST
-#define VENDORID 9999  // TODO(munetoh)
 
 /* global variables */
 static int initialized = 0;
@@ -88,72 +76,135 @@ static OPENPTS_CONTEXT *ctx = NULL;
 
 // static char *config_filename = NULL;
 
-// int verbose = 0;
+int verbose = 0;
 // int verbose = DEBUG_IFM_FLAG;
-int verbose = DEBUG_FLAG | DEBUG_IFM_FLAG;
+// int verbose = DEBUG_FLAG | DEBUG_IFM_FLAG;
 
+#if 1
 static TNC_Result sendMessage(
     /*in*/ TNC_IMCID imcID,
     /*in*/ TNC_ConnectionID connectionID,
     /*in*/ TNC_BufferReference message,
     /*in*/ TNC_UInt32 messageLength,
     /*in*/ TNC_MessageType messageType);
+#endif
 
 /* List of receive message types */
 static TNC_MessageType messageTypes[] = {
-    TNCMESSAGENUM(TNC_VENDORID_TCG, TNC_SUBTYPE_ANY),  // generic
-    TNCMESSAGENUM(VENDORID,  1),
-    TNCMESSAGENUM(VENDORID,  3),
-    TNCMESSAGENUM(VENDORID,  4),
-    TNCMESSAGENUM(VENDORID,  5),
-    TNCMESSAGENUM(VENDORID,  7),
-    TNCMESSAGENUM(VENDORID,  9),
+    ((TNC_VENDORID_TCG_PEN << 8) | TNC_SUBTYPE_TCG_PTS),  // generic
+    ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS)   // OpenPTS
 };
 
-/* IMC Functions */
 
+/******************************************************************************/
+/*  IMC Functions                                                             */
+/******************************************************************************/
 /**
  * TNC_IMC_Initialize (MANDATORY) 
  * 
+ * TODO share the code with ptsc.c
  */
 TNC_IMC_API TNC_Result TNC_IMC_Initialize(
-/*in*/  TNC_IMCID imcID,
-/*in*/  TNC_Version minVersion,
-/*in*/  TNC_Version maxVersion,
-/*out*/ TNC_Version *pOutActualVersion) {
-    DEBUG("TNC_IMC_Initialize\n");
+    /*in*/  TNC_IMCID imcID,
+    /*in*/  TNC_Version minVersion,
+    /*in*/  TNC_Version maxVersion,
+    /*out*/ TNC_Version *pOutActualVersion) {
+    int rc;
 
-    if (initialized) return TNC_RESULT_ALREADY_INITIALIZED;
+    DEBUG("TNC_IMC_Initialize() - imcID=%d, minVersion=%d maxVersion=%d\n",
+        imcID, minVersion, maxVersion);
 
-    /* check version */
-    /* Only support version 1 */
+    if (initialized) {
+        return TNC_RESULT_ALREADY_INITIALIZED;
+    }
+
+    /* check version - Only support version 1 */
     if ((minVersion < TNC_IFIMC_VERSION_1) ||
         (maxVersion > TNC_IFIMC_VERSION_1)) {
         return TNC_RESULT_NO_COMMON_VERSION;
     }
 
     /* OK */
-
     *pOutActualVersion = TNC_IFIMC_VERSION_1;
     id = imcID;
 
-    /* initialize PTS */
+    /* initialize PTS Collector */
     conf = newPtsConfig();
     if (conf == NULL) {
         ERROR("no memory\n");
-        return TNC_RESULT_FATAL;
+        rc = TNC_RESULT_FATAL;
+        goto error;
     }
     ctx =  newPtsContext(conf);
     if (ctx == NULL) {
-        ERROR("no memory\n");
-        return TNC_RESULT_FATAL;
+        rc = TNC_RESULT_FATAL;
+        goto error;
     }
 
-    /* setup */
-    ctx->conf->ir_filename = "./data/ThinkpadX200_Fedora12/ir_file.xml";
+    DEBUG_IFM("config file  : %s\n", PTSC_CONFIG_FILE);
+
+    /* configure PTS Collector */
+    rc = readPtsConfig(conf, PTSC_CONFIG_FILE);
+    if (rc != PTS_SUCCESS) {
+        ERROR("read config file, '%s' was failed - abort\n", PTSC_CONFIG_FILE);
+        rc = TNC_RESULT_FATAL;
+        goto error;
+    }
+
+    /* check IR dir */
+    if (checkDir(conf->ir_dir) != PTS_SUCCESS) {
+        ERROR("Initialize the IMC. e.g. ptsc -i\n");
+        rc = TNC_RESULT_FATAL;
+        goto error;
+    }
+
+    /* RM UUID */
+    rc = readOpenptsUuidFile(conf->rm_uuid);
+    if (rc != PTS_SUCCESS) {
+        ERROR("read RM UUID file %s was failed, initialize ptscd first\n", conf->rm_uuid->filename);
+        rc = TNC_RESULT_FATAL;
+        goto error;
+    } else {
+        DEBUG("conf->str_rm_uuid         : %s\n", conf->rm_uuid->str);
+    }
+
+    /* NEWRM UUID */
+    rc = readOpenptsUuidFile(conf->newrm_uuid);
+    if (rc != PTS_SUCCESS) {
+        DEBUG("conf->str_newrm_uuid      : missing (file:%s)\n", conf->newrm_uuid->filename);
+        // goto free;
+    } else {
+        DEBUG("conf->str_newrm_uuid      : %s (for next boot)\n", conf->newrm_uuid->str);
+    }
+
+    /* load RSA PUB key */
+    // TODO single key => multiple keys?
+    /* get PUBKEY */
+    rc = getTssPubKey(
+            conf->uuid->uuid,
+            TSS_PS_TYPE_SYSTEM,
+            conf->srk_password_mode,
+            conf->tpm_resetdalock,
+            NULL,
+            &conf->pubkey_length,
+            &conf->pubkey);
+    if (rc != TSS_SUCCESS) {
+        ERROR("getTssPubKey() fail rc=0x%x srk password mode=%d, key =%s\n",
+            rc, conf->srk_password_mode, conf->uuid->str);
+        rc = TNC_RESULT_FATAL;
+        goto error;
+    }
+
+    /* PUBKEY ? */
 
     initialized++;
     return TNC_RESULT_SUCCESS;
+
+  error:
+    if (ctx != NULL) freePtsContext(ctx);
+    ctx = NULL;
+
+    return rc;
 }
 
 
@@ -189,9 +240,10 @@ TNC_IMC_API TNC_Result TNC_IMC_BeginHandshake(
 /*in*/  TNC_IMCID imcID,
 /*in*/  TNC_ConnectionID connectionID) {
     int rc = 0;
-    char* msg = "Hello";  // TBD
+    char* msg;
+    int len;
 
-    DEBUG("TNC_IMC_BeginHandshake %d %d\n",
+    DEBUG("TNC_IMC_BeginHandshake - imcID=%d, connectionID=%d\n",
             (int)imcID, (int)connectionID);
 
     /* check internal status */
@@ -207,14 +259,19 @@ TNC_IMC_API TNC_Result TNC_IMC_BeginHandshake(
 
     /* just send hello to verifier */
 
-    DEBUG_IFM("C    imcID=%d, connectionID=%d - TNC_IMC_BeginHandshake\n", (int)imcID, (int)connectionID);
+    // DEBUG_IFM("C    imcID=%d, connectionID=%d - TNC_IMC_BeginHandshake\n", (int)imcID, (int)connectionID);
+
+
+    msg = getPtsTlvMessage(ctx, OPENPTS_CAPABILITIES, &len);
+
+    DEBUG_IFM("[C->V] OPENPTS_CAPABILITIES[%d]\n", len);
 
     rc = sendMessage(
-                imcID,
-                connectionID,
-                (TNC_BufferReference) msg,
-                strlen(msg),
-                TNCMESSAGENUM(VENDORID, 0));
+        imcID,
+        connectionID,
+        (TNC_BufferReference) msg,
+        len,
+        ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
 
     return rc;
 }
@@ -223,17 +280,20 @@ TNC_IMC_API TNC_Result TNC_IMC_BeginHandshake(
  * TNC_IMC_ReceiveMessage (OPTIONAL)
  */
 TNC_IMC_API TNC_Result TNC_IMC_ReceiveMessage(
-/*in*/  TNC_IMCID imcID,
-/*in*/  TNC_ConnectionID connectionID,
-/*in*/  TNC_BufferReference messageBuffer,
-/*in*/  TNC_UInt32 messageLength,
-/*in*/  TNC_MessageType messageType) {
-    int len = 0;
+    /*in*/  TNC_IMCID imcID,
+    /*in*/  TNC_ConnectionID connectionID,
+    /*in*/  TNC_BufferReference messageBuffer,
+    /*in*/  TNC_UInt32 messageLength,
+    /*in*/  TNC_MessageType messageType) {
     PTS_IF_M_Attribute *read_tlv;
-    int rc;
-    // char* msg = "this is a test message";
+    UINT32 type;
+    int length;
+    int rc = 0;
+    BYTE *value;
+    int len = 0;
+    char* msg;
 
-    DEBUG("TNC_IMC_ReceiveMessage msg=%s\n", messageBuffer);
+    // DEBUG("TNC_IMC_ReceiveMessage msg=%s\n", messageBuffer);
 
     /* check internal status */
     if (!initialized)
@@ -248,11 +308,120 @@ TNC_IMC_API TNC_Result TNC_IMC_ReceiveMessage(
         return TNC_RESULT_INVALID_PARAMETER;
 
     /* */
-    DEBUG_IFM("V->C imcID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
+    DEBUG_IFM("[C<-V] imcID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
         (int)imcID, (int)connectionID, (int)messageType, (int)messageLength);
 
     /* handshake */
+    if (messageType == ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS)) {
+        /* OPENPTS */
+        read_tlv = (PTS_IF_M_Attribute*)messageBuffer;
 
+        /* check VID */
+        // TODO read_tlv->
+        type = ntohl(read_tlv->type);
+        length = ntohl(read_tlv->length);
+        value = (BYTE*)&messageBuffer[12];
+
+        // DEBUG_IFM("[C->V]  type=%08X, length=%d\n", type, length);
+
+        switch (type) {
+        case OPENPTS_CAPABILITIES:
+            /* Check Verifier */
+            DEBUG_IFM("[C<-V] OPENPTS_CAPABILITIES[%d]\n", 12 + length);
+            // TODO check the verifier's UUID?
+            break;
+
+        case REQUEST_TPM_PUBKEY:
+            DEBUG_IFM("[C<-V] REQUEST_TPM_PUBKEY[%d]\n", 12 + length);
+
+            /* send TPM_PUBKEY */
+            msg = getPtsTlvMessage(ctx, TPM_PUBKEY, &len);
+            if (msg == NULL) {
+                ERROR("return  OPENPTS_ERROR");
+                msg = getPtsTlvMessage(ctx, OPENPTS_ERROR, &len);
+            }
+
+            rc = sendMessage(
+                imcID,
+                connectionID,
+                (TNC_BufferReference) msg,
+                len,
+                ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+            DEBUG_IFM("[C->V] TPM_PUBKEY[%d]\n", len);
+            break;
+
+        case REQUEST_RIMM_SET:
+            DEBUG_IFM("[C<-V]  REQUEST_RIMM_SET[%d]\n", 12 + length);
+
+            /* set RM filename */
+            rc = getRmSetDir(conf);
+            if (rc != PTS_SUCCESS) {
+                ERROR("collector() - getRmSetDir() was failed\n");
+                return PTS_INTERNAL_ERROR;
+            }
+
+            /* send RIMM_SET */
+            msg = getPtsTlvMessage(ctx, RIMM_SET, &len);
+            if (msg == NULL) {
+                ERROR("Get RIMM_SET message was faild, return  OPENPTS_ERROR");
+                msg = getPtsTlvMessage(ctx, OPENPTS_ERROR, &len);
+            }
+
+            rc = sendMessage(
+                imcID,
+                connectionID,
+                (TNC_BufferReference) msg,
+                len,
+                ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+            DEBUG_IFM("[C->V] RIMM_SET[%d]\n", len);
+            break;
+
+        case NONCE:
+            DEBUG_IFM("[C<-V]  NONCE[%d]\n", 12 + length);
+            ctx->nonce->nonce_length = length;
+            ctx->nonce->nonce = malloc(length);
+            memcpy(ctx->nonce->nonce, value, length);
+            break;
+
+        case REQUEST_INTEGRITY_REPORT:
+            DEBUG_IFM("[C<-V]  REQUEST_INTEGRITY_REPORT[%d]\n", 12 + length);
+
+            /* send INTEGRITY_REPORT */
+            msg = getPtsTlvMessage(ctx, INTEGRITY_REPORT, &len);
+            if (msg == NULL) {
+                ERROR("return  OPENPTS_ERROR");
+                msg = getPtsTlvMessage(ctx, OPENPTS_ERROR, &len);
+            }
+
+            rc = sendMessage(
+                imcID,
+                connectionID,
+                (TNC_BufferReference) msg,
+                len,
+                ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+            DEBUG_IFM("[C->V] INTEGRITY_REPORT[%d]\n", len);
+
+            break;
+
+        default:
+            ERROR("Unknown type %08X", type);
+            break;
+        }
+        return rc;
+    } else if (messageType == ((TNC_VENDORID_TCG_PEN << 8) | TNC_SUBTYPE_TCG_PTS)) {
+        /* TCG */
+        ERROR("TBD\n");
+        return TNC_RESULT_FATAL;
+    } else {
+        ERROR("bad msg from collector");
+        return TNC_RESULT_FATAL;
+    }
+
+
+
+
+
+#if 0
     if (messageType == TNCMESSAGENUM(VENDORID, 1)) {
         /* capability from client  */
         read_tlv = (PTS_IF_M_Attribute *) messageBuffer;
@@ -346,6 +515,7 @@ TNC_IMC_API TNC_Result TNC_IMC_ReceiveMessage(
         ERROR("bad msg from verifier\n");
         return TNC_RESULT_FATAL;
     }
+#endif
 
     return TNC_RESULT_SUCCESS;
 }
@@ -412,16 +582,19 @@ static TNC_Result reportMessageTypes(
     /*in*/ TNC_IMCID imcID,
     /*in*/ TNC_MessageTypeList supportedTypes,
     /*in*/ TNC_UInt32 typeCount) {
-    DEBUG("TNC_TNCC_ReportMessageTypes\n");
+    DEBUG("TNC_TNCC_ReportMessageTypes() - imcID=%d, supportedTypes=0x%X, typeCount=%d\n",
+        imcID, supportedTypes, typeCount);
 
     if (!reportMessageTypesPtr)
         return TNC_RESULT_FATAL;
 
-    return (*reportMessageTypesPtr)(imcID,
-                                     supportedTypes,
-                                     typeCount);
+    return (*reportMessageTypesPtr)(
+        imcID,
+        supportedTypes,
+        typeCount);
 }
 
+#if 1
 /**
  * Call TNC_TNCC_SendMessage (MANDATORY) in the TNCC
  */
@@ -431,13 +604,13 @@ static TNC_Result sendMessage(
     /*in*/ TNC_BufferReference message,
     /*in*/ TNC_UInt32 messageLength,
     /*in*/ TNC_MessageType messageType) {
-    DEBUG("TNC_TNCC_SendMessage msg='%s' type=%d(0x%x)\n",
-            message, (int)messageType, (int)messageType);
+    DEBUG("TNC_TNCC_SendMessage msg='%s' type=0x%x\n",
+            message, (int)messageType);
 
     if (!sendMessagePtr)
         return TNC_RESULT_FATAL;
 
-    DEBUG_IFM("C->V imcID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
+    DEBUG_IFM("[C->V] imcID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
         (int)imcID, (int)connectionID, (int)messageType, (int)messageLength);
 
     return (*sendMessagePtr)(imcID,
@@ -446,6 +619,7 @@ static TNC_Result sendMessage(
                               messageLength,
                               messageType);
 }
+#endif
 
 #if 0
 // F12 imc.c:277: error: ‘requestHandshakeRetry’ defined but not used
@@ -465,6 +639,7 @@ static TNC_Result requestHandshakeRetry(
 }
 #endif
 
+
 /* Platform-Specific IMC Functions */
 
 /**
@@ -484,11 +659,11 @@ static TNC_Result requestHandshakeRetry(
  * @param bindFunction - Pointer to TNC_TNCC_BindFunction
  */
 TNC_IMC_API TNC_Result TNC_IMC_ProvideBindFunction(
-/*in*/  TNC_IMCID imcID,
-/*in*/  TNC_TNCC_BindFunctionPointer bindFunction) {
+    /*in*/  TNC_IMCID imcID,
+    /*in*/  TNC_TNCC_BindFunctionPointer bindFunction) {
     TNC_Result rc = TNC_RESULT_SUCCESS;
 
-    DEBUG("TNC_IMC_ProvideBindFunction\n");
+    DEBUG("TNC_IMC_ProvideBindFunction() - imcID=%d\n", imcID);
 
     /* check internal status */
     if (!initialized)
@@ -498,6 +673,7 @@ TNC_IMC_API TNC_Result TNC_IMC_ProvideBindFunction(
     if (imcID != id)
         return TNC_RESULT_INVALID_PARAMETER;
 
+
     /* Bind  */
     if (bindFunction) {
         if ((*bindFunction)(imcID,
@@ -526,10 +702,10 @@ TNC_IMC_API TNC_Result TNC_IMC_ProvideBindFunction(
         }
     }
 
-    // TODO(munetoh) rc
-    rc = reportMessageTypes(imcID,
-                            messageTypes,
-                            sizeof(messageTypes) / sizeof(TNC_MessageType));
+    rc = reportMessageTypes(
+            imcID,
+            messageTypes,
+            sizeof(messageTypes) / sizeof(TNC_MessageType));
 
     return rc;
 }
