@@ -26,7 +26,7 @@
  * \brief Load TCG Integrity Measurement Log (IML)
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-01
- * cleanup 2011-01-22 SM
+ * cleanup 2011-07-06 SM
  *
  * get IML/PCRS from filesystem
  * get IML/PCRS vis TSS
@@ -252,7 +252,8 @@ int addEventToSnapshotBhv(
             if (ss == NULL) {
                 /* SS is missing */
                 addReason(ctx,
-                    "[PCR%02d] Snapshot is missing for PCR%d for Level 0 and 1. Please check the configuration file '%s'",
+                    "[PCR%02d] Snapshot is missing for PCR%d for Level 0 and 1. "
+                    "Please check the configuration file '%s'",
                     index,
                     index,
                     ctx->conf->config_file);
@@ -462,7 +463,7 @@ int addEventToSnapshotBin(
                 addReason(ctx,  "[RM%02d-PCR%02d-MissingState] IR validation by RM was faild",
                     active_level,
                     index);
-            } else if (ss->fsm_binary->curr_state->name == NULL) {
+            } else if (ss->fsm_binary->curr_state->name[0] == 0) {  // TODO malloc the name
                 ERROR("ss->fsm_binary->curr_state->name == NULL\n");
                 addReason(ctx,  "[RM%02d-PCR%02d-MissingStateName] IR validation by RM was faild",
                     active_level,
@@ -497,18 +498,22 @@ int addEventToSnapshotBin(
                 DEBUG_FSM("move to level %d\n", getActiveSnapshotLevel(ctx->ss_table, index));
                 /* Update with new SS */
                 ss = getSnapshotFromTable(ctx->ss_table, index, 1);  // TODO new func for next
-                eventWrapper->snapshot = ss;
-
-                rc = updateFsm(ctx, ss->fsm_binary, eventWrapper);
-                if (rc == OPENPTS_FSM_ERROR) {
-                    DEBUG_FSM("No trans, return PTS_INVALID_SNAPSHOT at %s\n", ss->fsm_binary->curr_state->name);
-                    DEBUG("updateFsm fail\n");
-                    addReason(ctx, "[RM%02d-PCR%02d-%s] IR validation by RM was faild",
-                        active_level + 1,
-                        index,
-                        ss->fsm_binary->curr_state->name);
-                    ctx->ss_table->error[index] = PTS_INVALID_SNAPSHOT;
-                    return PTS_INVALID_SNAPSHOT;
+                if (ss == NULL) {
+                    ERROR("getSnapshotFromTable(%d,%d) is NULL\n", index, 1);
+                    return PTS_INTERNAL_ERROR;
+                } else {
+                    eventWrapper->snapshot = ss;
+                    rc = updateFsm(ctx, ss->fsm_binary, eventWrapper);
+                    if (rc == OPENPTS_FSM_ERROR) {
+                        DEBUG_FSM("No trans, return PTS_INVALID_SNAPSHOT at %s\n", ss->fsm_binary->curr_state->name);
+                        DEBUG("updateFsm fail\n");
+                        addReason(ctx, "[RM%02d-PCR%02d-%s] IR validation by RM was faild",
+                            active_level + 1,
+                            index,
+                            ss->fsm_binary->curr_state->name);
+                        ctx->ss_table->error[index] = PTS_INVALID_SNAPSHOT;
+                        return PTS_INVALID_SNAPSHOT;
+                    }
                 }
             } else {
                 ERROR("no BIN-FSM at level 0,  no BIN-FSM at level 1\n");
@@ -851,6 +856,8 @@ int getIml(OPENPTS_CONTEXT * ctx, int option) {
 
 /**
  * fread + endian conv
+ * Return
+ *   0xFFFFFFFF Error
  */
 UINT32 freadUint32(FILE * stream, int endian) {
     size_t size;
@@ -862,7 +869,7 @@ UINT32 freadUint32(FILE * stream, int endian) {
 
     if (size != 4) {
         // This is EOF ERROR("\n");
-        return -1;  // TODO
+        return 0xFFFFFFFF;  // TODO
     }
 
     if (endian == 0) {
@@ -907,19 +914,20 @@ UINT32 freadUint32(FILE * stream, int endian) {
  *   PTS_INTERNAL_ERROR
  *
  */
-
-int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
-    int rc = 0;
+// TODO rename readBiosImlFile()
+int readBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
+    int rc = PTS_SUCCESS;
+    int result;
     int i = 0;
     size_t size;
-    FILE *fp;
+    FILE *fp = NULL;
     UINT32 pcrIndex;
     UINT32 eventType;
     UINT32 eventLength;
     int endian = 0;
     int aligned = 0;
 
-    TSS_PCR_EVENT *event;
+    TSS_PCR_EVENT *event = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew_new = NULL;
     // OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
     int error = 0;
@@ -927,6 +935,7 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
     DEBUG_CAL("getBiosImlFile - start\n");
     // DEBUG("read BIOS IML, file %s\n", filename);
 
+    /* check */
     if (ctx == NULL) {
         ERROR("ERROR\n");  // TODO(munetoh)
         return PTS_INTERNAL_ERROR;
@@ -936,6 +945,12 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
         return PTS_INTERNAL_ERROR;
     }
 
+    /* open file */
+    if ((fp = fopen(filename, "rb")) == NULL) {
+        ERROR("%s missing", filename);
+        return PTS_INTERNAL_ERROR;
+    }
+
     // TODO
     if (mode == USE_BHV_FSM_EC) {
         mode = USE_BHV_FSM;
@@ -943,27 +958,28 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
         aligned = 4;  // TODO conf->iml_aligned?
     }
 
-    /* open file */
-    if ((fp = fopen(filename, "rb")) == NULL) {
-        ERROR("%s missing", filename);
-        return PTS_INTERNAL_ERROR;
-    }
-
     /* Read IML, add to Snapshot */
     while (1) {
         /* PCR index */
         pcrIndex = freadUint32(fp, endian);
-        if (pcrIndex > 24) {  // TODO -1
+        if (pcrIndex == 0xFFFFFFFF) {
+            /* end of data */
             break;
         }
+        if (pcrIndex > MAX_PCRNUM) {
+            ERROR("BIOS IML File %s, bad pcr index value %d at %d event\n",
+                filename, pcrIndex, i);
+            rc = PTS_INTERNAL_ERROR;
+            goto close;
+        }
 
         /* Event type */
         eventType = freadUint32(fp, endian);
-
         event = (TSS_PCR_EVENT *) malloc(sizeof(TSS_PCR_EVENT));
         if (event == NULL) {
-            printf("no memory\n");
-            return PTS_INTERNAL_ERROR;
+            ERROR("no memory\n");
+            rc = PTS_FATAL;
+            goto close;
         }
         memset(event, 0, sizeof(TSS_PCR_EVENT));
 
@@ -974,17 +990,22 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
         /* Digest */
         event->ulPcrValueLength = SHA1_DIGEST_SIZE;
         event->rgbPcrValue = (BYTE *) malloc(SHA1_DIGEST_SIZE);  // leaked
-
+        if (event->rgbPcrValue == NULL) {
+            ERROR("no memory\n");
+            rc = PTS_FATAL;
+            goto close;
+        }
         size = fread(event->rgbPcrValue, 1, SHA1_DIGEST_SIZE, fp);
         if (size != SHA1_DIGEST_SIZE) {  // TODO(munetoh) SHA1 only
-            ERROR("size = %d != 20\n", (int)size);
-            break;
+            ERROR("BIOS IML File %s, bad pcr size %d at %d event\n",
+                filename, (int)size, i);
+            rc = PTS_INTERNAL_ERROR;
+            goto close;
         }
 
         /* EventData len */
         eventLength = freadUint32(fp, endian);
         event->ulEventLength = eventLength;
-
         /* adjust read data length */
         if (aligned == 4) {
             if ((eventLength & 0x03) != 0) {
@@ -992,17 +1013,19 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
                 eventLength = (eventLength & 0xFFFFFFFC) + 0x04;
             }
         }
-
-        /* EventData  */
+        /* malloc EventData */
         if ((event->rgbEvent = malloc(eventLength)) == NULL) {
-            return PTS_INTERNAL_ERROR;
+            ERROR("no memory\n");
+            rc = PTS_FATAL;
+            goto close;
         }
-
         // TODO if rgbevent is huge 0x4000000 #=> check the endian
         size = fread(event->rgbEvent, 1, eventLength, fp);
         if (size != eventLength) {
-            ERROR("getBiosImlFile - size =  0x%x != 0x%x\n", (int)size, (int)eventLength);
-            return PTS_INTERNAL_ERROR;
+            ERROR("BIOS IML File %s, bad eventdata size 0x%x != 0x%xat %d event\n",
+                filename, (int)size, (int)eventLength, i);
+            rc = PTS_INTERNAL_ERROR;
+            goto close;
         }
 
 #if 0
@@ -1013,38 +1036,36 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
         }
 #endif
 
-        /* create wrapper */
-        // ew_last = ew_new;
+        /* create event wrapper */
         ew_new = (OPENPTS_PCR_EVENT_WRAPPER *)
             malloc(sizeof(OPENPTS_PCR_EVENT_WRAPPER));
         if (ew_new == NULL) {
             ERROR("no memory\n");
-            return PTS_INTERNAL_ERROR;
+            rc = PTS_FATAL;
+            goto close;
         }
         memset(ew_new, 0, sizeof(OPENPTS_PCR_EVENT_WRAPPER));
         ew_new->event = event;
 
-        // DEBUG
-        // printEventWrapper(ew_new);
-
+        /* add to the snapshot */
         if (mode == USE_BHV_FSM) {
             /* BHV-FSM - map to the snapshot */
-            rc = addEventToSnapshotBhv(ctx, ew_new);  // iml.c
+            result = addEventToSnapshotBhv(ctx, ew_new);  // iml.c
 
-            if (rc == PTS_SUCCESS) {
+            if (result == PTS_SUCCESS) {
                 /* OK */
-            } else if (rc == PTS_INVALID_SNAPSHOT) {  // OPENPTS_FSM_ERROR) {
+            } else if (result == PTS_INVALID_SNAPSHOT) {  // OPENPTS_FSM_ERROR) {
                 /* ERROR but continue the verification of rest of IML */
                 error++;
-            } else if (rc == PTS_INTERNAL_ERROR) {  // 58
+            } else if (result == PTS_INTERNAL_ERROR) {  // 58
                 /* SKIP */
-            } else if (rc == OPENPTS_FSM_TRANSIT) {
+            } else if (result == OPENPTS_FSM_TRANSIT) {
                 DEBUG_FSM("\tTransit to next FSM ======================================\n");
                 rc = addEventToSnapshotBhv(ctx, ew_new);  // iml.c
-            } else if (rc == OPENPTS_FSM_FINISH_WO_HIT) {
+            } else if (result == OPENPTS_FSM_FINISH_WO_HIT) {
                 DEBUG_FSM("\tTransit to next FSM ======================================\n");
-                rc = addEventToSnapshotBhv(ctx, ew_new);  // iml.c
-            } else if (rc == OPENPTS_FSM_MIGRATE_EVENT) {
+                result = addEventToSnapshotBhv(ctx, ew_new);  // iml.c
+            } else if (result == OPENPTS_FSM_MIGRATE_EVENT) {
                 /* SKIP */
             } else {
                 /* Unknwon error */
@@ -1052,22 +1073,22 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
             }
         } else {  // USE_BIN_FSM
             /* BIN-FSM - map to the snapshot */
-            rc = addEventToSnapshotBin(ctx, ew_new);  // iml.c
+            result = addEventToSnapshotBin(ctx, ew_new);  // iml.c
 
-            if (rc == OPENPTS_FSM_SUCCESS) {
+            if (result == OPENPTS_FSM_SUCCESS) {
                 /* OK */
-            } else if (rc == PTS_INVALID_SNAPSHOT) {
+            } else if (result == PTS_INVALID_SNAPSHOT) {
                 /* Keep */
-            } else if (rc == OPENPTS_FSM_TRANSIT) {
+            } else if (result == OPENPTS_FSM_TRANSIT) {
                 DEBUG_FSM("\tTransit to next FSM ======================================\n");
-                rc = addEventToSnapshotBin(ctx, ew_new);  // iml.c
-                if (rc < 0) {  // TODO
+                result = addEventToSnapshotBin(ctx, ew_new);  // iml.c
+                if (result < 0) {  // TODO
                     TODO("getBiosImlFile - addEventToSnapshotBin rc = %d\n", rc);
                 }
-            } else if (rc == OPENPTS_FSM_FINISH_WO_HIT) {
+            } else if (result == OPENPTS_FSM_FINISH_WO_HIT) {
                 DEBUG_FSM("\tTransit to next FSM ======================================\n");
-                rc = addEventToSnapshotBin(ctx, ew_new);  // iml.c
-                if (rc < 0) {  // TODO
+                result = addEventToSnapshotBin(ctx, ew_new);  // iml.c
+                if (result < 0) {  // TODO
                     TODO("getBiosImlFile - addEventToSnapshotBin rc = %d\n", rc);
                 }
             } else {
@@ -1077,34 +1098,53 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
         }
 
         /* TPM Extend */
-        rc = extendTpm(&ctx->tpm, ew_new->event);
-        if (rc !=0) {
+        result = extendTpm(&ctx->tpm, ew_new->event);
+        if (result !=0) {
             ERROR("extend TPM fail\n");
-            return PTS_INTERNAL_ERROR;
+            rc = PTS_INTERNAL_ERROR;
+            goto close;
         }
 
+        /* moved to the snapshot */
+        event = NULL;
+        ew_new = NULL;
+
         /* inc */
         ctx->ss_table->event_num++;
         i++;
-    }
+    }  // while loop
 
+
+  close:
     if (fclose(fp) == EOF) {
-        // TODO(munetoh) SYSLOG ERROR?
-        return PTS_INTERNAL_ERROR;
+        ERROR("BIOS IML File %s, read fail\n", filename);
+        rc = PTS_INTERNAL_ERROR;
     }
     DEBUG("read BIOS IML, file %s => %d events\n", filename, ctx->ss_table->event_num);
 
-    DEBUG_CAL("iml.c - getBiosImlFile - done\n");
-
     if (error > 0) {
         addReason(ctx, "[IML] Load IML(file:%s) was faild",
             filename);
-        return PTS_INVALID_SNAPSHOT;
+        rc = PTS_INVALID_SNAPSHOT;
     }
 
-    return PTS_SUCCESS;
+    /* free (for ERROR) */
+    if (event != NULL) {
+        if (event->rgbPcrValue != NULL) free(event->rgbPcrValue);
+        if (event->rgbEvent != NULL) free(event->rgbEvent);
+        free(event);
+    }
+    if (ew_new != NULL) free(ew_new);
+
+    DEBUG_CAL("iml.c - getBiosImlFile - done\n");
+
+    return rc;
 }
 
+
+
+#define TEMPLATE_TYPE_SIZE 16
+
 /**
  * \brief  read Runtime(Linux-IMA) IML file
  *
@@ -1271,15 +1311,12 @@ d2 a1 0c 44 d7 d5 5a 41 48 75 2a 7d 01 cd 8c b3 fe 14 78 06
 
 </PRE>
 */
-
-#define TEMPLATE_TYPE_SIZE 16
-
-// TODO rename to readImaImlFile
-int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mode) {
-    int rc = 0;
+int readImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mode, int *count) {
+    int rc = PTS_SUCCESS;
+    int result;
     int i = 0;
     size_t size;
-    FILE *fp;
+    FILE *fp = NULL;
     UINT32 pcr_index;
     UINT32 event_type;
     UINT32 template_len;
@@ -1290,23 +1327,23 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
 
     TSS_PCR_EVENT *event = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew = NULL;
-    // OPENPTS_PCR_EVENT_WRAPPER *ew_start = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
 
-    DEBUG_CAL("getImaImlFile - start\n");
+    DEBUG_CAL("readImaImlFile - start\n");
 
+    /* check */
     if (ctx == NULL) {
-        ERROR("getImaImlFile - ctx is NULL\n");  // TODO(munetoh)
+        ERROR("readImaImlFile - ctx is NULL\n");  // TODO(munetoh)
         return -1;
     }
     if (filename == NULL) {
-        ERROR("getImaImlFile - no filename\n");  // TODO(munetoh)
+        ERROR("readImaImlFile - no filename\n");  // TODO(munetoh)
         return -1;
     }
 
     /* open file */
     if ((fp = fopen(filename, "rb")) == NULL) {
-        ERROR("getImaImlFile - file open was failed, [%s]\n", filename);
+        ERROR("readImaImlFile - file open was failed, [%s]\n", filename);
         return -1;
     }
 
@@ -1316,38 +1353,29 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
      *   level 1 <= IMA
      */
 
-#if 0
-    ss = &ctx->snapshots[10];
-    ss->event_num = 0;
-    ss->next = newSnapshot();  // PCR[10] IMA => level 1
-    ss = ss->next;
-    ss->level = 1;
-    ctx->snapshots_level[10] = 1;
-#endif
-
     /* pass one - check the IML size */
     while (1) {
         /* read PCR index */
         size = fread(&pcr_index, 1, 4, fp);
         if (size != 4) {
-            // DEBUG("getImaImlFile - end of event\n");
+            /* end of event? => exit */
             break;
         }
-
-        if (pcr_index > 24) {
-            ERROR("getImaImlFile - Bad pcrindex %d %x\n", pcr_index, pcr_index);
-            break;
+        if (pcr_index > MAX_PCRNUM) {
+            ERROR("Linux-IMA IML File %s, bad pcr index value %d at %d event\n",
+                filename, pcr_index, i);
+            rc = PTS_INTERNAL_ERROR;
+            goto close;
         }
 
         /* alloc event structure */
-        // TODO memory leak error by static analysis
         event = (TSS_PCR_EVENT *) malloc(sizeof(TSS_PCR_EVENT));
         if (event == NULL) {
-            ERROR("no mem\n");  // TODO(munetoh)
-            return -1;
+            ERROR("no memory\n");
+            rc = PTS_FATAL;
+            goto close;
         }
         memset(event, 0, sizeof(TSS_PCR_EVENT));
-
         event->ulPcrIndex = pcr_index;
         event->eventType = 0;  // TODO
         event->ulPcrValueLength = 0;
@@ -1356,86 +1384,110 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
 
         /* many formats :-( */
         if (type == BINARY_IML_TYPE_IMA_ORIGINAL) {  // 2.6.29
-            // DEBUG("getImaImlFile - BINARY_IML_TYPE_IMA_ORIGINAL\n");
             /* read type */
             size = fread(&event->eventType, 1, 4, fp);
             if (size != 4) {
-                ERROR("missing event type\n");
-                break;
+                ERROR("Linux-IMA(ORIGINAL) IML File %s, bad eventType at %d event\n",
+                    filename, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             /* read Digest (SHA1) */
             event->ulPcrValueLength = SHA1_DIGEST_SIZE;
             event->rgbPcrValue = (BYTE *) malloc(SHA1_DIGEST_SIZE);
-
+            if (event->rgbPcrValue == NULL) {
+                ERROR("no memory\n");
+                rc = PTS_FATAL;
+                goto close;
+            }
             size = fread(event->rgbPcrValue, 1, SHA1_DIGEST_SIZE, fp);
             if (size != SHA1_DIGEST_SIZE) {
-                ERROR("missing pcr\n");
-                break;
+                ERROR("Linux-IMA(ORIGINAL) IML File %s, bad pcr size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             /* read eventdata length */
             size = fread(&event->ulEventLength, 1, 4, fp);
             if (size != 4) {
-                ERROR("missing\n");
-                break;
+                ERROR("Linux-IMA(ORIGINAL) IML File %s, bad event length size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
-
             /* alloc eventdata */
-            if ((event->rgbEvent = malloc(event->ulEventLength)) == NULL) {
-                ERROR("missing\n");
-                rc = -1;
-                goto free;
+            event->rgbEvent = malloc(event->ulEventLength);
+            if (event->rgbEvent == NULL) {
+                ERROR("no memory\n");
+                rc = PTS_FATAL;
+                goto close;
             }
             // memset(event->rgbEvent,0,event->ulEventLength);
 
             /* read filename */
             size = fread(event->rgbEvent, 1, event->ulEventLength, fp);
             if (size != event->ulEventLength) {
-                ERROR("missing\n");
-                break;
+                ERROR("Linux-IMA(ORIGINAL) IML File %s, bad event size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
         } else if (type == BINARY_IML_TYPE_IMA_31) {  // 2.6.30-32
             // DEBUG("getImaImlFile - BINARY_IML_TYPE_IMA_31\n");
             /* read type */
             size = fread(&event_type, 1, 4, fp);
             if (size != 4) {
-                ERROR("missing event type\n");
-                break;
+                ERROR("Linux-IMA(IMA_31) IML File %s, bad eventType at %d event\n",
+                    filename, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             /* read Digest (SHA1) */
             event->ulPcrValueLength = SHA1_DIGEST_SIZE;
             event->rgbPcrValue = (BYTE *) malloc(SHA1_DIGEST_SIZE);
-
+            if (event->rgbPcrValue == NULL) {
+                ERROR("no memory\n");
+                rc = PTS_FATAL;
+                goto close;
+            }
             size = fread(event->rgbPcrValue, 1, SHA1_DIGEST_SIZE, fp);
             if (size != SHA1_DIGEST_SIZE) {
-                ERROR("missing pcr\n");
-                break;
+                ERROR("Linux-IMA(IMA_31) IML File %s, bad pcr size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             /* read Template length */
             size = fread(&template_len, 1, 4, fp);
             if (size != 4) {
-                ERROR("missing\n");
-                break;
+                ERROR("Linux-IMA(IMA_31) IML File %s, bad template size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
             filename_len = template_len - 20;
 
             /* alloc template (=event data) */
             event->ulEventLength = 20 + 256;  // TODO(munetoh)
-            if ((event->rgbEvent = malloc(event->ulEventLength)) == NULL) {
-                ERROR("missing\n");
-                rc = -1;
-                goto free;
+            event->rgbEvent = malloc(event->ulEventLength);
+            if (event->rgbEvent == NULL) {
+                ERROR("no memory\n");
+                rc = PTS_FATAL;
+                goto close;
             }
             memset(event->rgbEvent, 0, event->ulEventLength);
 
             /* read Template digest */
             size = fread(event->rgbEvent, 1, SHA1_DIGEST_SIZE, fp);
             if (size != SHA1_DIGEST_SIZE) {
-                ERROR("getImaImlFile - missing\n");
-                break;
+                ERROR("Linux-IMA(IMA_31) IML File %s, bad event size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             // DEBUG("getImaImlFile - filename_len %d\n", filename_len);
@@ -1443,9 +1495,10 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
             /* read filename */
             size = fread(&event->rgbEvent[20], 1, filename_len, fp);
             if (size != filename_len) {
-                ERROR("getImaImlFile - rgbEvent is missing, len=%d (!=%d) type=BINARY_IML_TYPE_IMA_31\n",
-                    (int)size, (int)filename_len);
-                break;
+                ERROR("Linux-IMA(IMA_31) IML File %s, bad event size %d != %dat %d event\n",
+                    filename, (int)size, (int)filename_len, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
         } else {
             /* read Digest (SHA1) */
@@ -1454,20 +1507,25 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
 
             size = fread(event->rgbPcrValue, 1, SHA1_DIGEST_SIZE, fp);
             if (size != SHA1_DIGEST_SIZE) {
-                ERROR("missing pcr\n");
-                break;
+                ERROR("Linux-IMA() IML File %s, bad pcr size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             /* read Template type length */
             size = fread(&template_type_len, 1, 4, fp);
             if (size != 4) {
-                ERROR("missing\n");
-                break;
+                ERROR("Linux-IMA() IML File %s, bad template size %d at %d event\n",
+                    filename, size, i);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
 
             if (template_type_len >= TEMPLATE_TYPE_SIZE) {
                 ERROR("template_type_len %d(0x%x) is too big\n", template_type_len, template_type_len);
-                break;
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
             // DEBUG("getImaImlFile - template_type_len %d\n",template_type_len);
 
@@ -1475,7 +1533,8 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
             size = fread(&buf, 1, template_type_len, fp);
             if (size != template_type_len) {
                 ERROR("missing\n");
-                break;
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
             // TODO accessing beyond memory
             buf[template_type_len] = 0;
@@ -1487,9 +1546,11 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
 
                 /* alloc template (=event data) */
                 event->ulEventLength = 20 + 256;  // TODO(munetoh)
-                if ((event->rgbEvent = malloc(event->ulEventLength)) == NULL) {
-                    ERROR("missing\n");
-                    return -1;
+                event->rgbEvent = malloc(event->ulEventLength);
+                if (event->rgbEvent == NULL) {
+                    ERROR("no memory\n");
+                    rc = PTS_FATAL;
+                    goto close;
                 }
                 memset(event->rgbEvent, 0, event->ulEventLength);
 
@@ -1497,33 +1558,38 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
                 size = fread(event->rgbEvent, 1, SHA1_DIGEST_SIZE, fp);
                 if (size != SHA1_DIGEST_SIZE) {
                     ERROR("missing\n");
-                    break;
+                    rc = PTS_INTERNAL_ERROR;
+                    goto close;
                 }
 
                 /* read filename len */
                 size = fread(&filename_len, 1, 4, fp);
                 if (size != 4) {
                     ERROR("missing\n");
-                    break;
+                    rc = PTS_INTERNAL_ERROR;
+                    goto close;
                 }
 
                 if (filename_len > 255) {
                     ERROR("filename_len is too big, %d, 0x%x\n", filename_len, filename_len);
-                    break;
+                    rc = PTS_INTERNAL_ERROR;
+                    goto close;
                 }
 
-                DEBUG_CAL("getImaImlFile - filename_len %d\n", filename_len);
+                DEBUG_CAL("readImaImlFile - filename_len %d\n", filename_len);
 
                 /* read filename */
                 size = fread(&event->rgbEvent[20], 1, filename_len, fp);
                 if (size != filename_len) {
                     ERROR("missing\n");
-                    break;
+                    rc = PTS_INTERNAL_ERROR;
+                    goto close;
                 }
 
             } else {
                 ERROR("Unknown template [%s]\n", buf);
-                break;
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
         }
 
@@ -1533,84 +1599,75 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
             malloc(sizeof(OPENPTS_PCR_EVENT_WRAPPER));
         if (ew == NULL) {
             ERROR("no memory\n");
-            rc = -1;
-            goto free;
+            rc = PTS_FATAL;
+            goto close;
         }
         memset(ew, 0, sizeof(OPENPTS_PCR_EVENT_WRAPPER));
         ew->event = event;
-        event = NULL;
 
         if (event_num == 0) {
-            // ew_start = ew;
             ew_last = ew;
         } else {
             ew->index = event_num;
             ew_last->next_all = ew;
             ew_last = ew;
         }
-        ew = NULL;
-
-        // DEBUG("event %p  EW    %p\n", event, ew_new);
-
-        rc = 0;
 
+        /* to snapshot */
         if (mode == USE_BHV_FSM) {
             /* map to the snapshot */
-            rc = addEventToSnapshotBhv(ctx, ew_last);  // iml.c
-            if (rc != PTS_SUCCESS) {
-                ERROR("getImaImlFile - addEventToSnapshotBhv fail, rc = %d\n", rc);
-                rc = -1;
-                goto free;
+            result = addEventToSnapshotBhv(ctx, ew_last);  // iml.c
+            if (result != PTS_SUCCESS) {
+                ERROR("readImaImlFile - addEventToSnapshotBhv fail, rc = %d\n", rc);
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
         } else {  // USE_BIN_FSM
             /* map to the snapshot */
-            rc = addEventToSnapshotBin(ctx, ew_last);  // iml.c
-            if (rc != PTS_SUCCESS) {
-                ERROR("getImaImlFile - addEventToSnapshotBin fail\n");
-                rc = -1;
-                goto free;
+            result = addEventToSnapshotBin(ctx, ew_last);  // iml.c
+            if (result != PTS_SUCCESS) {
+                ERROR("readImaImlFile - addEventToSnapshotBin fail\n");
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
             }
         }
 
-        DEBUG_CAL("getImaImlFile - TPM_extend\n");
+        DEBUG_CAL("readImaImlFile - TPM_extend\n");
 
         /* TPM Extend */
-        rc = extendTpm(&ctx->tpm, ew_last->event);
-        if (rc !=0) {
+        result = extendTpm(&ctx->tpm, ew_last->event);
+        if (result !=0) {
             ERROR("extend TPM fail\n");
-            rc = -1;
-            goto free;
+            rc = PTS_INTERNAL_ERROR;
+            goto close;
         }
 
+        /* moved to the snapshot */
+        event = NULL;
+        ew    = NULL;
+
         event_num++;
         i++;
     }  // while
 
     ctx->ss_table->event_num += event_num;
 
+  close:
     fclose(fp);
 
     // DEBUG("iml.c - getBiosImlFile - done, %d events\n", event_num);
     // ERROR("SS LEVEL %d  == 1?, ss->event_num =%d\n",ss->level,ss->event_num );
     DEBUG("read IMA IML, file %s => %d events\n", filename, event_num);
-    DEBUG_CAL("getImaImlFile - done, %d events\n", event_num);
+    DEBUG_CAL("readImaImlFile - done, %d events\n", event_num);
 
-    rc = event_num;
-
-  free:
+    *count = event_num;
     /* free (for error) */
     if (event != NULL) {
-        if (event->rgbPcrValue != NULL) {
-            free(event->rgbPcrValue);
-        }
-        if (event->rgbEvent != NULL) {
-            free(event->rgbEvent);
-        }
+        if (event->rgbPcrValue != NULL) free(event->rgbPcrValue);
+        if (event->rgbEvent != NULL) free(event->rgbEvent);
         free(event);
     }
-    if (ew  != NULL) {
-        free(ew);  // TODO memory leak
-    }
+    if (ew  != NULL) free(ew);
 
     return rc;
 }
@@ -2000,6 +2057,8 @@ int printIml(OPENPTS_CONTEXT * ctx) {
     return rc;
 }
 
+#if 0
+// TODO REMOVE
 /**
  * print TSS_PCR_EVENT
  *
@@ -2238,7 +2297,9 @@ void printEvent(TSS_PCR_EVENT *event) {
         ERROR("NULL event\n");  // TODO(munetoh)
     }
 }
+#endif
 
+#if 0
 /**
  * print OPENPTS_SNAPSHOT
  */
@@ -2255,7 +2316,10 @@ void printSnapshot(OPENPTS_SNAPSHOT * ss) {
         eventWrapper = eventWrapper->next_pcr;
     }
 }
+#endif
 
+#if 0
+// TODO REMOVE
 /**
  * print events in each snapshot 
  */ 
@@ -2297,7 +2361,7 @@ void printSnapshots(OPENPTS_CONTEXT * ctx) {
     printf("level 1 total = %d\n", level1_num);
     printf("---------------------------\n");
 }
-
+#endif
 
 /**
  *  print event number of each snapshot
