@@ -24,7 +24,7 @@
  * \brief Convert binary IML file to plaintext
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-08-25
- * cleanup 2011-04-26 SM
+ * cleanup 2011-05-31 SM
  *
  * show eventlog (get though TSS)
  *
@@ -783,9 +783,10 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
     UINT32 eventType;
     UINT32 eventLength;
     UINT32 aligned_length;
+    // BYTE buf[4];
 
     /**/
-    TSS_PCR_EVENT *event = NULL;
+    // TSS_PCR_EVENT *event = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew_start = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
@@ -794,7 +795,7 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
 
     /* check */
     if (filename == NULL) {
-        ERROR("filename is NULL\n");  // TODO(munetoh)
+        ERROR("filename is NULL\n");
         return TSS_E_INTERNAL_ERROR;
     }
 
@@ -804,7 +805,7 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         return TSS_E_INTERNAL_ERROR;
     }
 
-    /* Read IML, add to Snapshot */
+    /* STEP 1, Read IML, add to Snapshot */
     while (1) {
         DEBUG("--- event %d ------------\n", i);
         /* PCR index */
@@ -817,40 +818,44 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         if (size != 4)
             break;
 
+        /* create wrapper */
+        ew = (OPENPTS_PCR_EVENT_WRAPPER *)malloc(sizeof(OPENPTS_PCR_EVENT_WRAPPER));
+        if (ew == NULL) {
+            ERROR("no memory\n");
+            rc =  TSS_E_INTERNAL_ERROR;
+            goto close;
+        }
+        memset(ew, 0, sizeof(OPENPTS_PCR_EVENT_WRAPPER));
+
         /* alloc new event */
-        event = (TSS_PCR_EVENT *) malloc(sizeof(TSS_PCR_EVENT));   // TODO memory leak
-        if (event == NULL) {
+        ew->event = (TSS_PCR_EVENT *) malloc(sizeof(TSS_PCR_EVENT));
+        if (ew->event == NULL) {
             printf("no memory\n");
             rc =  TSS_E_INTERNAL_ERROR;
             goto close;
         }
-        memset((void*)event, 0, sizeof(TSS_PCR_EVENT));
+        memset((void*)ew->event, 0, sizeof(TSS_PCR_EVENT));
 
-        // event->versionInfo = 0;  // TODO(munetoh)
         if (endian == 0) {
-            event->ulPcrIndex = pcrIndex;
-            event->eventType = eventType;
+            ew->event->ulPcrIndex = pcrIndex;
+            ew->event->eventType = eventType;
         } else {
             /* Big endian */
-            event->ulPcrIndex = b2l(pcrIndex);
-            event->eventType = b2l(eventType);
+            ew->event->ulPcrIndex = b2l(pcrIndex);
+            ew->event->eventType = b2l(eventType);
         }
 
-        DEBUG("\tpcr index = 0x%x\n", event->ulPcrIndex);
-        DEBUG("\tevent type = 0x%x\n", event->eventType);
-
-        // DEBUG("%x %x\n", event->ulPcrIndex, event->eventType);
-
+        DEBUG("\tpcr index = 0x%x\n", ew->event->ulPcrIndex);
+        DEBUG("\tevent type = 0x%x\n", ew->event->eventType);
 
         /* Digest */
-        event->ulPcrValueLength = SHA1_DIGEST_SIZE;
-        if ((event->rgbPcrValue = (BYTE *) malloc(SHA1_DIGEST_SIZE)) == NULL) {
+        ew->event->ulPcrValueLength = SHA1_DIGEST_SIZE;
+        if ((ew->event->rgbPcrValue = (BYTE *) malloc(SHA1_DIGEST_SIZE)) == NULL) {
             ERROR("no memory");
             rc =  TSS_E_INTERNAL_ERROR;
             goto close;
         }
-
-        size = fread(event->rgbPcrValue, 1, SHA1_DIGEST_SIZE, fp);
+        size = fread(ew->event->rgbPcrValue, 1, SHA1_DIGEST_SIZE, fp);
         if (size != SHA1_DIGEST_SIZE) {  // TODO(munetoh) SHA1 only
             ERROR("SHA1 only");
             rc =  TSS_E_INTERNAL_ERROR;
@@ -859,7 +864,7 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
 
         if (verbose == DEBUG_FLAG) {
             DEBUG("digest");
-            printHex("\t\t\t\t", event->rgbPcrValue, 20, "\n");
+            printHex("\t\t\t\t", ew->event->rgbPcrValue, 20, "\n");
         }
 
         /* EventData len */
@@ -871,33 +876,32 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         }
 
         if (endian == 0) {
-            event->ulEventLength = eventLength;
+            ew->event->ulEventLength = eventLength;
         } else {
-            event->ulEventLength = b2l(eventLength);
+            ew->event->ulEventLength = b2l(eventLength);
         }
 
         /* adjust read data length */
-        aligned_length = event->ulEventLength;
+        aligned_length = ew->event->ulEventLength;
         if (aligned == 4) {
-            if ((event->ulEventLength & 0x03) != 0) {
-                aligned_length = (event->ulEventLength & 0xFFFFFFFC) + 0x04;
+            if ((ew->event->ulEventLength & 0x03) != 0) {
+                aligned_length = (ew->event->ulEventLength & 0xFFFFFFFC) + 0x04;
             }
         }
 
-        DEBUG("\tevent size = 0x%x (%d)\n", event->ulEventLength, event->ulEventLength);
+        DEBUG("\tevent size = 0x%x (%d)\n", ew->event->ulEventLength, ew->event->ulEventLength);
 
         /* EventData  */
-        if ((event->rgbEvent = malloc(aligned_length)) == NULL) {
+        if ((ew->event->rgbEvent = malloc(aligned_length)) == NULL) {
             ERROR("no memory");
             rc =  TSS_E_INTERNAL_ERROR;
             goto close;
         }
-        // DEBUG("\n");
-        size = fread(event->rgbEvent, 1, aligned_length, fp);
+        size = fread(ew->event->rgbEvent, 1, aligned_length, fp);
         if (size != aligned_length) {
             ERROR("fread NG, size = %d != %d (@PCR[%d])\n",
                 (unsigned int) size,
-                (unsigned int) event->ulEventLength,
+                (unsigned int) ew->event->ulEventLength,
                 pcrIndex);
             rc =  TSS_E_INTERNAL_ERROR;
             goto close;
@@ -905,24 +909,10 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
 
         if (verbose == DEBUG_FLAG) {
             DEBUG("");
-            printHex("\tevent data", event->rgbEvent, event->ulEventLength, "\n");
+            printHex("\tevent data", ew->event->rgbEvent, ew->event->ulEventLength, "\n");
         }
 
-
-        // DEBUG("\n");
-        /* create wrapper */
-        ew = (OPENPTS_PCR_EVENT_WRAPPER *)malloc(sizeof(OPENPTS_PCR_EVENT_WRAPPER));
-        if (ew == NULL) {
-            ERROR("no memory\n");
-            rc =  TSS_E_INTERNAL_ERROR;
-            goto close;
-        }
-        memset(ew, 0, sizeof(OPENPTS_PCR_EVENT_WRAPPER));
-
-        // DEBUG("\n");
-        ew->event = event;
-        event = NULL;
-
+        /* move to EW chain */
         if (i == 0) {
             /* 1st EW */
             ew_start = ew;
@@ -931,18 +921,17 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
             ew_last->next_all = ew;
             ew_last = ew;
         }
-        ew = NULL;
 
-        // DEBUG("Next");
+
         i++;
     }
+    /* done, clear last event & ew */
+    ew = NULL;
 
-
-
+    /* event num */
     *event_num = i;
 
-    /* generate TSS_PCR_EVENT** */
-
+    /* STEP 2, generate TSS_PCR_EVENT**,  ew chain -> TSS_PCR_EVENT** */
     /* maloc */
     *pcr_events = calloc(*event_num, sizeof(TSS_PCR_EVENT));
     if ((*pcr_events) == NULL) {
@@ -950,21 +939,23 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         rc = TSS_E_INTERNAL_ERROR;
         goto close;
     }
-
-    ew = ew_start;
+    /* copy events */
+    ew = ew_start;  // first EW
     for (i = 0; i < (int)(*event_num); i++) {
-        event = ew->event;
-        memcpy(&((*pcr_events)[i]), event, sizeof(TSS_PCR_EVENT));
+        memcpy(&((*pcr_events)[i]), ew->event, sizeof(TSS_PCR_EVENT));
 
         ew_last = ew;
-        ew = ew->next_all;
+        ew = ew->next_all;  // next
+
+        /* free event, ew (PCR.EventData are linked to pcr_events) */
+        ew_last->event->rgbPcrValue = NULL;
+        ew_last->event->rgbEvent = NULL;
+        free(ew_last->event);
         free(ew_last);
     }
     ew = NULL;
-    event = NULL;
-
-    // TODO free ew
 
+    /* OK */
     rc = TSS_SUCCESS;
 
   close:
@@ -972,6 +963,15 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         // TODO(munetoh) SYSLOG ERROR?
         rc = TSS_E_INTERNAL_ERROR;
     }
+    /* free for error */
+    if (ew != NULL) {
+        if (ew->event != NULL) {
+            if (ew->event->rgbPcrValue != NULL) free(ew->event->rgbPcrValue);
+            if (ew->event->rgbEvent != NULL) free(ew->event->rgbEvent);
+            free(ew->event);
+        }
+        free(ew);
+    }
 
     return rc;  // TSS_E_INTERNAL_ERROR;
 }
@@ -999,19 +999,14 @@ void usage(void) {
 int main(int argc, char *argv[]) {
     TSS_RESULT result = 0;
     TSS_HCONTEXT hContext;
-
     TSS_HTPM hTPM;
-
-    UINT32 ulEventNumber = 0;
     TSS_PCR_EVENT * PcrEvents;
+    UINT32 ulEventNumber = 0;
     BYTE digest[20];
     BYTE zero[20];
     BYTE fox[20];
     BYTE boot_aggregate[20];
     int i, j;
-    char *outfilename = NULL;
-    int firmware = 1;
-    int kernel = 1;
     int pcrindex = -1;
     int gmode = 0;  // 0:TSS,1,file
     char *filename = NULL;
@@ -1021,23 +1016,20 @@ int main(int argc, char *argv[]) {
     int aligned = 0;
     int ima_exist = 0;
     int drtm = 0;
-
-    /* init */
     BYTE *blob;
     UINT32 blobLength;
+
+    /* init */
     memset(zero, 0, 20);
     memset(fox, 0xff, 20);
     memset(boot_aggregate, 0xff, 20);
 
-
-
     /* Args */
     while ((c = getopt(argc, argv, "i:p:I:EAvVDh")) != EOF) {
         switch (c) {
         case 'i':
             filename = optarg;
             gmode = 1;  // file
-            // DEBUG("%s\n", filename);
             break;
         case 'p':
             pcrindex = atoi(optarg);
@@ -1086,14 +1078,13 @@ int main(int argc, char *argv[]) {
         }
     }
 
-
     if (verbose_cnt > 1) {
        verbose = DEBUG_FLAG;
     }
 
+    /* TSS and Verify */
     if (gmode == 0 || verify) {
         /* in both cases, we have to connect to TCSD */
-
         result = Tspi_Context_Create(&hContext);
         if (result != TSS_SUCCESS) {
             printf("ERROR: Tspi_Context_Create failed rc=0x%x\n",
@@ -1117,11 +1108,9 @@ int main(int argc, char *argv[]) {
         }
     }
 
+    /* get IML */
     if (gmode == 0) {
-        /* TSS */
-
-        /* Get Log */
-
+        /* Get EventLog via TSS */
         result = Tspi_TPM_GetEventLog(
                     hTPM,
                     &ulEventNumber,
@@ -1132,7 +1121,7 @@ int main(int argc, char *argv[]) {
             goto close;
         }
     } else {
-        // FILE
+        /* Get EventLog File */
         result = getEventLog(filename, endian, aligned, &ulEventNumber, &PcrEvents);
         if (result != TSS_SUCCESS) {  // ERROR
             ERROR("getEventLog failed rc=0x%x\n",
@@ -1141,72 +1130,59 @@ int main(int argc, char *argv[]) {
         }
     }
 
-
-    /* print all */
-    if (outfilename != NULL) {
-        if ((fp = fopen(outfilename, "w")) == NULL) {
-            ERROR("File open failed, %s \n",
-                   outfilename);
-            result = -1;  // TODO
-            goto free;
-        }
-    }
-
+    /* Dump */
     fprintf(fp, " Idx PCR       Type    Digest                                EventData\n");
     fprintf(fp, "-----------------------------------------------------------------------\n");
     for (i = 0; i < (int)ulEventNumber; i++) {
-        if ((firmware == 0) && (PcrEvents[i].ulPcrIndex <= 8)) {
-            // SKIP
-        } else if ((kernel == 0) && (PcrEvents[i].ulPcrIndex == 10)) {
-            // SKIP
-        } else {
-            if ((pcrindex < 0) || (pcrindex == (int)PcrEvents[i].ulPcrIndex)) {
-                fprintf(fp, "%4d ", i);
-                fprintf(fp, "%3d ", PcrEvents[i].ulPcrIndex);
-                fprintf(fp, "0x%08x ", PcrEvents[i].eventType);
-                for (j = 0; j < (int)PcrEvents[i].ulPcrValueLength; j++)
-                    fprintf(fp, "%02x", PcrEvents[i].rgbPcrValue[j]);
-                fprintf(fp, " ");
+        if ((pcrindex < 0) || (pcrindex == (int)PcrEvents[i].ulPcrIndex)) {
+            fprintf(fp, "%4d ", i);
+            fprintf(fp, "%3d ", PcrEvents[i].ulPcrIndex);
+            fprintf(fp, "0x%08x ", PcrEvents[i].eventType);
+            for (j = 0; j < (int)PcrEvents[i].ulPcrValueLength; j++)
+                fprintf(fp, "%02x", PcrEvents[i].rgbPcrValue[j]);
+            fprintf(fp, " ");
 
-                /* event Data */
-                fprintEventData(
-                    fp,
-                    PcrEvents[i].rgbEvent,
-                    PcrEvents[i].ulEventLength,
-                    PcrEvents[i].ulPcrIndex,
-                    PcrEvents[i].eventType,
-                    PcrEvents[i].rgbPcrValue);
+            /* event Data */
+            fprintEventData(
+                fp,
+                PcrEvents[i].rgbEvent,
+                PcrEvents[i].ulEventLength,
+                PcrEvents[i].ulPcrIndex,
+                PcrEvents[i].eventType,
+                PcrEvents[i].rgbPcrValue);
 
-                fprintf(fp, "\n");
-
-                if (verify) {
-                    if (PcrEvents[i].ulPcrIndex == 10) {  // IMA log
-                        if (memcmp(PcrEvents[i].rgbPcrValue, zero, 20) == 0) {  // zero
-                            extend(PcrEvents[i].ulPcrIndex, fox);
-                        } else {
-                            extend(PcrEvents[i].ulPcrIndex, PcrEvents[i].rgbPcrValue);
-                        }
-                        // TODO get boot aggregate
-                        if (memcmp(PcrEvents[i].rgbEvent, "boot_aggregate", 14) == 0) {
-                            memcpy(boot_aggregate, PcrEvents[i].rgbPcrValue, 20);
-                        }
+            fprintf(fp, "\n");
 
+            if (verify) {
+                if (PcrEvents[i].ulPcrIndex == 10) {  // IMA log
+                    if (memcmp(PcrEvents[i].rgbPcrValue, zero, 20) == 0) {  // zero
+                        extend(PcrEvents[i].ulPcrIndex, fox);
                     } else {
                         extend(PcrEvents[i].ulPcrIndex, PcrEvents[i].rgbPcrValue);
                     }
+                    // TODO get boot aggregate
+                    if (memcmp(PcrEvents[i].rgbEvent, "boot_aggregate", 14) == 0) {
+                        memcpy(boot_aggregate, PcrEvents[i].rgbPcrValue, 20);
+                    }
+
+                } else {
+                    extend(PcrEvents[i].ulPcrIndex, PcrEvents[i].rgbPcrValue);
                 }
-                if (PcrEvents[i].ulPcrIndex == 10) {
-                    ima_exist = 1;
-                }
+            }
+            if (PcrEvents[i].ulPcrIndex == 10) {
+                ima_exist = 1;
             }
         }
     }
+
+    /* Verify */
     if (verify) {
         fprintf(fp, "\n");
         fprintf(fp, "Verify IML :-)\n");
         fprintf(fp, "\n");
         fprintf(fp, "\tcalculated pcr values\t\t\t\tactual pcr values\n");
-        fprintf(fp, "------------------------------------------------------------------------------------------------------\n");
+        fprintf(fp, "---------------------------------------------------"
+            "---------------------------------------------------\n");
         for (i = 0; i < 24; i++) {
             fprintf(fp, "pcr.%d=\t", i);
             // my calc
@@ -1239,7 +1215,8 @@ int main(int argc, char *argv[]) {
             ima_boot_aggregate(digest);
             fprintf(fp, "IMA boot aggregate:\n");
             fprintf(fp, "\tcalculated boot_aggregate\t\t\t\tactual boot_aggregate\n");
-            fprintf(fp, "------------------------------------------------------------------------------------------------------\n");
+            fprintf(fp, "---------------------------------------------------"
+                "---------------------------------------------------\n");
             for (j = 0; j < 20; j++) {
                 fprintf(fp, "%02x", digest[j]);
             }
@@ -1256,6 +1233,12 @@ int main(int argc, char *argv[]) {
         result = Tspi_Context_FreeMemory(hContext, (BYTE *)PcrEvents);
         Tspi_Context_FreeMemory(hContext, NULL);
     } else {
+        // TODO free PcrEvents
+        for (i = 0; i < (int)ulEventNumber; i++) {
+            if (PcrEvents[i].rgbPcrValue != NULL) free(PcrEvents[i].rgbPcrValue);
+            if (PcrEvents[i].rgbEvent != NULL) free(PcrEvents[i].rgbEvent);
+        }
+        free(PcrEvents);
     }
     /* Close TSS/TPM */
 
