@@ -26,7 +26,7 @@
  * \brief TCG TNC IF-IMV v1.2 R8
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-05-07
- * cleanup 2011-03-06 SM
+ * cleanup 2011-07-06 SM
  *
  * http://www.trustedcomputinggroup.org/resources/tnc_ifimv_specification
  * http://www.trustedcomputinggroup.org/files/static_page_files/646808C3-1D09-3519-AD2E60765779A42A/TNC_IFIMV_v1_2_r8.pdf
@@ -50,23 +50,14 @@
 #include <string.h>
 
 #include <tncifimv.h>
-#include <libtnc.h>
+// #include <libtnc.h>
 
 #include <openpts.h>
 
-// TODO(munetoh) Fix Makefile.am
-#include "ifm.c"
-#include "iml.c"
-#include "ir.c"
-#include "base64.c"
-#include "tpm.c"
-#include "ctx.c"
-#include "uml.c"
-#include "fsm.c"
-#include "rm.c"
-#include "log.c"
 
-#define VENDORID 9999  // TODO(munetoh) Get ID for OpenPTS
+// ifm.c
+BYTE* getPtsTlvMessage(OPENPTS_CONTEXT *ctx, int type, int *len);
+
 
 /* global */
 
@@ -75,17 +66,20 @@ static int initialized = 0;
 
 static OPENPTS_CONFIG *conf = NULL;
 static OPENPTS_CONTEXT *ctx = NULL;
+static int result = OPENPTS_RESULT_UNKNOWN;
 
-// int verbose = 0;
+int verbose = 0;
 // int verbose = DEBUG_IFM_FLAG;
-int verbose = DEBUG_FLAG | DEBUG_IFM_FLAG;
+// int verbose = DEBUG_FLAG | DEBUG_IFM_FLAG;
 
+#if 1
 static TNC_Result sendMessage(
     /*in*/ TNC_IMVID imvID,
     /*in*/ TNC_ConnectionID connectionID,
     /*in*/ TNC_BufferReference message,
     /*in*/ TNC_UInt32 messageLength,
     /*in*/ TNC_MessageType messageType);
+#endif
 static TNC_Result provideRecommendation(
     /*in*/ TNC_IMVID imvID,
     /*in*/ TNC_ConnectionID connectionID,
@@ -109,13 +103,8 @@ static TNC_TNCS_SendMessagePointer           sendMessagePtr;
 
 /* List of receive message types */
 static TNC_MessageType messageTypes[] = {
-    TNCMESSAGENUM(TNC_VENDORID_TCG, TNC_SUBTYPE_ANY),  // generic
-    TNCMESSAGENUM(VENDORID,  0),
-    TNCMESSAGENUM(VENDORID,  2),
-    TNCMESSAGENUM(VENDORID,  4),
-    TNCMESSAGENUM(VENDORID,  6),
-    TNCMESSAGENUM(VENDORID,  8),
-    TNCMESSAGENUM(VENDORID, 10),
+    ((TNC_VENDORID_TCG_PEN << 8) | TNC_SUBTYPE_TCG_PTS),  // generic
+    ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS)   // OpenPTS
 };
 
 /* IMV Functions */
@@ -158,11 +147,12 @@ static TNC_MessageType messageTypes[] = {
  * @praram  pOutActualVersion - Mutually supported API version number
  */
 TNC_IMV_API TNC_Result TNC_IMV_Initialize(
-/*in*/  TNC_IMVID imvID,
-/*in*/  TNC_Version minVersion,
-/*in*/  TNC_Version maxVersion,
-/*in*/  TNC_Version *pOutActualVersion) {
-    DEBUG("TNC_IMV_Initialize - %d %d %d\n",
+    /*in*/  TNC_IMVID imvID,
+    /*in*/  TNC_Version minVersion,
+    /*in*/  TNC_Version maxVersion,
+    /*in*/  TNC_Version *pOutActualVersion) {
+    int rc;
+    DEBUG("TNC_IMV_Initialize() - imvID=%d, minVersion=%d, maxVersion=%d\n",
         (int)imvID, (int)minVersion, (int)maxVersion);
 
     /* */
@@ -185,26 +175,79 @@ TNC_IMV_API TNC_Result TNC_IMV_Initialize(
     imv_id = imvID;
 
 
-#if 1
     /* initialize PTS */
     conf =  newPtsConfig();
     if (conf == NULL) {
         ERROR("no memory\n");
-        return TNC_RESULT_FATAL;
+        rc = TNC_RESULT_FATAL;
+        goto error;
     }
 
     ctx =  newPtsContext(conf);
     if (ctx == NULL) {
         ERROR("no memory\n");
-        return TNC_RESULT_FATAL;
+        rc = TNC_RESULT_FATAL;
+        goto error;
     }
-#endif
+
+    /* configure PTS Verifier (System wide) */
+    rc = readPtsConfig(conf, PTSV_CONFIG_FILE);
+    if (rc != PTS_SUCCESS) {
+        ERROR("read config file, '%s' was failed - abort\n",
+            PTSV_CONFIG_FILE);
+        rc = TNC_RESULT_FATAL;
+        goto error;
+    }
+    DEBUG_IFM("config file                 : %s\n", PTSV_CONFIG_FILE);
+
+    /* check the IMV settings */
+    // UUID
+    if (conf->uuid->status == OPENPTS_UUID_FILENAME_ONLY) {
+        /* 1st use?,  create new UUID */
+        rc = genOpenptsUuid(conf->uuid);
+        if (rc != PTS_SUCCESS) {
+            ERROR("generation of UUID was failed\n");
+            rc = TNC_RESULT_FATAL;
+            goto error;
+        }
+        /* save to the file */
+        rc = writeOpenptsUuidFile(conf->uuid, 1);
+        if (rc != PTS_SUCCESS) {
+            ERROR("Creation of UUID file, %s was failed\n",
+                conf->uuid->filename);
+            rc = TNC_RESULT_FATAL;
+            goto error;
+        }
+        DEBUG_IFM("conf->uuid->filename        : %s (new UUID)\n", conf->uuid->filename);
+        DEBUG_IFM("conf->uuid->str             : %s (new UUID)\n", conf->uuid->str);
+    } else {
+        DEBUG_IFM("conf->uuid->filename        : %s\n", conf->uuid->filename);
+        DEBUG_IFM("conf->uuid->str             : %s\n", conf->uuid->str);
+    }
+
+    // Targets
+    DEBUG_IFM("conf->enrollment            : 0x%x (none:%x, cred:%x, auto:%x)\n",
+        conf->enrollment,
+        IMV_ENROLLMENT_NONE, IMV_ENROLLMENT_CREDENTIAL, IMV_ENROLLMENT_AUTO);
+
+    DEBUG_IFM("conf->config_dir            : %s\n",
+        conf->config_dir);
+
+
+    // IIDB  -- TODO
 
     initialized++;
 
     DEBUG_IFM("V    imvID=%d - TNC_IMV_Initialize\n", (int) imvID);
 
     return TNC_RESULT_SUCCESS;
+
+  error:
+    if (ctx != NULL) freePtsContext(ctx);
+    ctx = NULL;
+    // TODO conf = NULL;
+
+    return rc;
 }
 
 /**
@@ -229,6 +272,7 @@ TNC_IMV_API TNC_Result TNC_IMV_NotifyConnectionChange(
     return TNC_RESULT_SUCCESS;
 }
 
+
 /**
  * TNC_IMV_ReceiveMessage (OPTIONAL)
  *
@@ -274,40 +318,373 @@ TNC_IMV_API TNC_Result TNC_IMV_NotifyConnectionChange(
  * @praram  messageLength  - Number of octets in message
  * @praram  messageType    - Message type of message
  *
+ *
+ *  TODO add AIDE
+ *
  */
 TNC_IMV_API TNC_Result TNC_IMV_ReceiveMessage(
-/*in*/  TNC_IMVID imvID,
-/*in*/  TNC_ConnectionID connectionID,
-/*in*/  TNC_BufferReference messageBuffer,
-/*in*/  TNC_UInt32 messageLength,
-/*in*/  TNC_MessageType messageType) {
-    int rc;
-    int len;
+    /*in*/  TNC_IMVID imvID,
+    /*in*/  TNC_ConnectionID connectionID,
+    /*in*/  TNC_BufferReference messageBuffer,
+    /*in*/  TNC_UInt32 messageLength,
+    /*in*/  TNC_MessageType messageType) {
     PTS_IF_M_Attribute *read_tlv;
+    UINT32 type;
+    int length;
+    BYTE * value;
+    int rc = 0;
+    BYTE* msg;
+    int len;
+    OPENPTS_IF_M_Capability *cap;
+    UINT32 vid;
+    int enrollment = 0;
+    OPENPTS_CONFIG *target_conf;
+    int mode = 0;
 
-    DEBUG("TNC_IMV_ReceiveMessage msg='%s' type=%d(0x%x)\n",
-        (char*)messageBuffer, (int)messageType, (int)messageType);
+
+/* verifier.c */
+    int verifierHandleCapability(
+        OPENPTS_CONTEXT *ctx,
+        char *conf_dir,
+        char *host,
+        OPENPTS_IF_M_Capability *cap);
+
+    int verifierHandleRimmSet(
+        OPENPTS_CONTEXT *ctx,
+        BYTE *buf);
+
+    int verifierHandleIR(
+        OPENPTS_CONTEXT *ctx,
+        int length,
+        BYTE *value,
+        int mode,
+        int *result);
+
+    DEBUG("TNC_IMV_ReceiveMessage  msg[%d] type=0x%x\n",
+        messageLength, (int)messageType);
 
     if (!initialized)
         return TNC_RESULT_NOT_INITIALIZED;
 
 
-    DEBUG_IFM("C->V imvID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
+    DEBUG_IFM("[C->V] imvID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
         (int)imvID, (int)connectionID, (int)messageType, (int)messageLength);
 
+
     /* handshake */
-    if (messageType == TNCMESSAGENUM(VENDORID, 0)) {
-        /* Hello from Client, send capability */
-        char* msg = getPtsTlvMessage(ctx, OPENPTS_CAPABILITIES, &len);
-        rc = sendMessage(imvID,
-                            connectionID,
-                            (TNC_BufferReference)msg,
-                            len,
-                            TNCMESSAGENUM(VENDORID, 1));
-        free(msg);
-        DEBUG_IFM("Verifier send PTS_CAPABILITIES len=%d\n", len);
+    if (messageType == ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS)) {
+        /* OPENPTS */
+        read_tlv = (PTS_IF_M_Attribute*)messageBuffer;  // NBO
+
+        /* check VID */
+        vid = read_tlv->vid[0] << 16;
+        vid += read_tlv->vid[1] << 8;
+        vid += read_tlv->vid[2];
+        if (vid != TNC_VENDORID_OPENPTS) {
+            ERROR("read_tlv->vid = 0x%X (!= 0x%X)",
+                vid, TNC_VENDORID_OPENPTS);
+            return TNC_RESULT_FATAL;
+        }
+
+        /* type and length */
+        type = ntohl(read_tlv->type);
+        length = ntohl(read_tlv->length);
+        value = &messageBuffer[12];
+
+        /* check length */
+        if (messageLength != (TNC_UInt32) (12 + length)) {
+            ERROR("Bad message %d != %d\n",
+                messageLength, 12 + length);
+            return TNC_RESULT_FATAL;
+        }
+
+
+        DEBUG_IFM("[C->V] vid=%X, type=%08X, length=%d\n", vid, type, length);
+
+        /* message type */
+        switch (type) {
+        case OPENPTS_CAPABILITIES:
+            /* Check Collector */
+            DEBUG_IFM("[C->V] OPENPTS_CAPABILITIES[%d]\n", 12 + length);
+            if (ctx->tnc_state != TNC_STATE_START) {
+                /* Bad message order */
+                ERROR("Bad message order state=%d != %d, type=%08x",
+                    ctx->tnc_state, TNC_STATE_START, type);
+                return TNC_RESULT_FATAL;
+            }
+
+            /* Capability */
+            cap =  (OPENPTS_IF_M_Capability *) value;
+
+
+            rc = verifierHandleCapability(ctx, conf->config_dir, NULL, cap);
+
+            if (rc == PTS_NOT_INITIALIZED) {
+                if (conf->enrollment ==  IMV_ENROLLMENT_AUTO) {
+                    /* enroll with this collector */
+                    DEBUG_IFM("Auto Mode, Trust 1st connection -> start enrollment!!!"
+                        " #######################################\n");
+
+                    if (ctx->target_conf == NULL) {
+                        ctx->target_conf = newPtsConfig();
+                    }
+                    target_conf = ctx->target_conf;
+
+                    /* intialize the target_conf */
+                    target_conf->uuid = ctx->collector_uuid;
+                    ctx->collector_uuid = NULL;
+                    target_conf->rm_uuid = ctx->rm_uuid;
+                    ctx->rm_uuid = NULL;
+
+                    target_conf->config_dir =
+                        getFullpathName(conf->config_dir, target_conf->uuid->str);
+                    target_conf->config_file =
+                        getFullpathName(target_conf->config_dir, "target.conf");
+                    target_conf->uuid->filename =
+                        getFullpathName(target_conf->config_dir, "uuid");
+                    target_conf->rm_basedir =
+                        getFullpathName(target_conf->config_dir, target_conf->rm_uuid->str);
+
+                    target_conf->ir_filename =
+                        getFullpathName(target_conf->config_dir, "./ir.xml");
+                    target_conf->prop_filename =
+                        getFullpathName(target_conf->config_dir, "./vr.properties");
+                    target_conf->policy_filename =
+                        getFullpathName(target_conf->config_dir, "./policy.conf");
+
+#ifdef CONFIG_AIDE
+                    target_conf->aide_database_filename =
+                        getFullpathName(target_conf->config_dir, "aide.db.gz");
+                    target_conf->aide_ignorelist_filename =
+                        getFullpathName(target_conf->config_dir, "aide.ignore");
+#ifdef CONFIG_SQLITE
+                    target_conf->aide_sqlite_filename =
+                        getFullpathName(target_conf->config_dir, "aide.sqlite.db");
+#endif
+#endif
+
+                    /* create */
+                    rc = makeDir(target_conf->config_dir);
+                    // TODO check rc
+
+                    DEBUG("conf dir               : %s\n", target_conf->config_dir);
+                    DEBUG("rm dir                 : %s\n", target_conf->rm_basedir);
+                    DEBUG("AIDE DB                : %s\n", target_conf->aide_database_filename);
+#ifdef CONFIG_SQLITE
+                    DEBUG("AIDE SQLite DB         : %s\n", target_conf->aide_sqlite_filename);
+#endif
+
+                    /* get Key */
+                    /* get RM */
+                    /* verify and create policy */
+                    /* then allow the 1st connection */
+                    enrollment = 1;
+                } else if (conf->enrollment == IMV_ENROLLMENT_CREDENTIAL) {
+                    TODO("TBD\n");
+                    return rc;
+                } else {
+                    ERROR("Collector is not initialized yet\n");
+                    return rc;
+                }
+            } else if (rc != PTS_SUCCESS) {
+                return rc;
+            } else {
+                enrollment = 0;
+                ctx->tnc_state = TNC_STATE_CAP;
+                // TODO load target
+            }
+
+            /* send IMV's capability  to IMC */
+            msg = getPtsTlvMessage(ctx, OPENPTS_CAPABILITIES, &len);
+            rc = sendMessage(
+                    imvID,
+                    connectionID,
+                    (TNC_BufferReference)msg,
+                    len,
+                    ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+            free(msg);
+            DEBUG_IFM("[C<-V] OPENPTS_CAPABILITIES[%d]\n", len);
+
+
+            if (enrollment == 1) {
+                /* start enrollment */
+                ctx->tnc_state = TNC_STATE_KEY_REQ;
+                msg = getPtsTlvMessage(ctx, REQUEST_TPM_PUBKEY, &len);
+                rc = sendMessage(
+                        imvID,
+                        connectionID,
+                        (TNC_BufferReference)msg,
+                        len,
+                        ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+                free(msg);
+                DEBUG_IFM("[C<-V] REQUEST_TPM_PUBKEY[%d]\n", len);
+            } else {
+                /*start verify, send NONCE and IR REQ*/
+                /* Next : Send NONCE */
+                ctx->nonce->nonce_length = 20;
+                ctx->nonce->nonce = malloc(20);
+                rc = getRandom(ctx->nonce->nonce, 20);
+                if (rc != PTS_SUCCESS) {
+                    ERROR("getRandom() fail\n");
+                }
+
+                ctx->tnc_state = TNC_STATE_NONCE;
+                msg = getPtsTlvMessage(ctx, NONCE, &len);
+                rc = sendMessage(
+                        imvID,
+                        connectionID,
+                        (TNC_BufferReference)msg,
+                        len,
+                        ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+                free(msg);
+                DEBUG_IFM("[C<-V] NONCE[%d]\n", len);
+
+                /* Next : REQ IR */
+                ctx->tnc_state = TNC_STATE_IR;
+                msg = getPtsTlvMessage(ctx, REQUEST_INTEGRITY_REPORT, &len);
+                rc = sendMessage(
+                        imvID,
+                        connectionID,
+                        (TNC_BufferReference)msg,
+                        len,
+                        ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+                free(msg);
+                DEBUG_IFM("[C<-V] REQUEST_INTEGRITY_REPORT[%d]\n", len);
+            }
+            break;
+
+        case TPM_PUBKEY:
+            DEBUG_IFM("[C->V] TPM_PUBKEY[%d]\n", 12 + length);
+            // TODO check the state
+
+            if (ctx->target_conf == NULL) {
+                ERROR("Bad sequence\n");
+            } else {
+                /* PUBKEY -> target_conf */
+                ctx->target_conf->pubkey_length = length;
+                ctx->target_conf->pubkey = malloc(ctx->target_conf->pubkey_length);
+                if (ctx->target_conf->pubkey == NULL) {
+                    ERROR("no memory");
+                    return TNC_RESULT_FATAL;
+                }
+
+                memcpy(
+                    ctx->target_conf->pubkey,
+                    value,  // NG read_tlv->value
+                    ctx->target_conf->pubkey_length);
+                ctx->tnc_state = TNC_STATE_KEY;
+
+
+                /* Next: send RM REQ - continue enrollment */
+                ctx->tnc_state = TNC_STATE_RM_REQ;
+                msg = getPtsTlvMessage(ctx, REQUEST_RIMM_SET, &len);
+                rc = sendMessage(
+                        imvID,
+                        connectionID,
+                        (TNC_BufferReference)msg,
+                        len,
+                        ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+                free(msg);
+                DEBUG_IFM("[C<-V] REQUEST_RIMM_SET[%d]\n", len);
+            }
+            break;
+
+        case RIMM_SET:
+            DEBUG_IFM("[C->V] RIMM_SET[%d]\n", 12 + length);
+
+            /* save to the file, UUID/UUID/rmN.xml*/
+            rc = verifierHandleRimmSet(ctx, value);
+            if (rc != PTS_SUCCESS) {
+                ERROR("verifierHandleRimmSet() fail\n");
+                return TNC_RESULT_FATAL;
+            }
+
+            /* save target conf */
+            writeTargetConf(
+                ctx->target_conf,
+                ctx->target_conf->uuid->uuid,
+                ctx->target_conf->config_file);  // ctx.c
+
+            /* Next : Send NONCE */
+            ctx->nonce->nonce_length = 20;
+            ctx->nonce->nonce = malloc(20);
+            rc = getRandom(ctx->nonce->nonce, 20);
+            if (rc != PTS_SUCCESS) {
+                ERROR("getRandom() fail\n");
+            }
+
+            ctx->tnc_state = TNC_STATE_NONCE_ENROLL;
+            msg = getPtsTlvMessage(ctx, NONCE, &len);
+            rc = sendMessage(
+                    imvID,
+                    connectionID,
+                    (TNC_BufferReference)msg,
+                    len,
+                    ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+            free(msg);
+            DEBUG_IFM("[C<-V] NONCE[%d]\n", len);
+
+            /* Next : REQ IR */
+            ctx->tnc_state = TNC_STATE_IR_ENROLL;
+            msg = getPtsTlvMessage(ctx, REQUEST_INTEGRITY_REPORT, &len);
+            rc = sendMessage(
+                    imvID,
+                    connectionID,
+                    (TNC_BufferReference)msg,
+                    len,
+                    ((TNC_VENDORID_OPENPTS << 8) | TNC_SUBTYPE_OPENPTS));
+            free(msg);
+            DEBUG_IFM("[C<-V] REQUEST_INTEGRITY_REPORT[%d]\n", len);
+
+            break;
+
+        case INTEGRITY_REPORT:
+            if (ctx->tnc_state == TNC_STATE_IR_ENROLL) {
+                /* Enrollment, create default policy */
+                DEBUG_IFM("[C->V] INTEGRITY_REPORT[%d] (Enrollment)\n", 12 + length);
+                mode = OPENPTS_UPDATE_MODE;
+            } else if (ctx->tnc_state == TNC_STATE_IR) {
+                /* Just verify */
+                DEBUG_IFM("[C->V] INTEGRITY_REPORT[%d]\n", 12 + length);
+                mode = OPENPTS_VERIFY_MODE;
+            } else {
+                /* BAD STATE */
+                ERROR("bad state");
+            }
+
+
+            /* verify */
+            rc = verifierHandleIR(ctx, length, value, mode, &result);
+            if (rc != PTS_SUCCESS) {
+                ERROR("verifierHandleIR() fail rc = %d\n", rc);
+                // 25 PTS_INVALID_SNAPSHOT?
+                // return TNC_RESULT_FATAL;
+            }
+            // DEBUG("result = %d => recomandation\n", result);
+            // TODO create
+            break;
+        case OPENPTS_ERROR:
+            ERROR("The corrector returns ERROR message");
+            // TODO invalid
+            result = OPENPTS_RESULT_UNKNOWN;
+            // break;
+            return TNC_RESULT_FATAL;
+        default:
+            ERROR("Unknown type %08X", type);
+            result = OPENPTS_RESULT_UNKNOWN;
+            break;
+        }
         return rc;
-    } else if (messageType == TNCMESSAGENUM(VENDORID, 2)) {
+    } else if (messageType == ((TNC_VENDORID_TCG_PEN << 8) | TNC_SUBTYPE_TCG_PTS)) {
+        /* TCG */
+        ERROR("TBD\n");
+        return TNC_RESULT_FATAL;
+    } else {
+        ERROR("bad msg from collector");
+        return TNC_RESULT_FATAL;
+    }
+
+#if 0
         /* capability from client  */
         read_tlv = (PTS_IF_M_Attribute *) messageBuffer;
         if (read_tlv->type != OPENPTS_CAPABILITIES) {
@@ -408,10 +785,8 @@ TNC_IMV_API TNC_Result TNC_IMV_ReceiveMessage(
                     TNC_IMV_ACTION_RECOMMENDATION_ISOLATE,
                     TNC_IMV_EVALUATION_RESULT_NONCOMPLIANT_MAJOR);
         return rc;
-    } else {
-        ERROR("bad msg from collector");
-        return TNC_RESULT_FATAL;
-    }
+#endif
+
 
     return TNC_RESULT_SUCCESS;
 }
@@ -452,8 +827,14 @@ TNC_IMV_API TNC_Result TNC_IMV_ReceiveMessage(
  *
  */
 TNC_IMV_API TNC_Result TNC_IMV_SolicitRecommendation(
-/*in*/  TNC_IMVID imvID,
-/*in*/  TNC_ConnectionID connectionID) {
+    /*in*/  TNC_IMVID imvID,
+    /*in*/  TNC_ConnectionID connectionID) {
+    TNC_BufferReference lang = (TNC_BufferReference) "en";   // BYTE*
+    TNC_BufferReference str;
+    TNC_IMV_Action_Recommendation recommendation;
+    TNC_IMV_Evaluation_Result evaluation;
+    int len;
+
     DEBUG("TNC_IMV_SolicitRecommendation\n");
 
     if (!initialized)
@@ -464,29 +845,66 @@ TNC_IMV_API TNC_Result TNC_IMV_SolicitRecommendation(
         return TNC_RESULT_INVALID_PARAMETER;
     }
 
+
+    if (result == OPENPTS_RESULT_VALID) {
+        TODO("verifier() result      : VALID");
+        str            = (TNC_BufferReference)"valid";
+        recommendation = TNC_IMV_ACTION_RECOMMENDATION_ALLOW;
+        evaluation     = TNC_IMV_EVALUATION_RESULT_COMPLIANT;
+    } else if (result == OPENPTS_RESULT_UNVERIFIED) {
+        DEBUG("verifier() result      : UNVERIFIED");
+        str            = (TNC_BufferReference)"unverified";
+        recommendation = TNC_IMV_ACTION_RECOMMENDATION_ISOLATE;
+        evaluation     = TNC_IMV_EVALUATION_RESULT_NONCOMPLIANT_MAJOR;
+    } else if (result == OPENPTS_RESULT_INVALID) {
+        TODO("verifier() result      : INVALID");
+        str            = (TNC_BufferReference)"invalid";
+        recommendation = TNC_IMV_ACTION_RECOMMENDATION_ISOLATE;
+        evaluation     = TNC_IMV_EVALUATION_RESULT_NONCOMPLIANT_MAJOR;
+    } else if (result == OPENPTS_RESULT_UNKNOWN) {
+        DEBUG("verifier() result      : UNKNOWN");
+        str            = (TNC_BufferReference)"unknown";
+        recommendation = TNC_IMV_ACTION_RECOMMENDATION_ISOLATE;
+        evaluation     = TNC_IMV_EVALUATION_RESULT_DONT_KNOW;
+    } else if (result == OPENPTS_RESULT_IGNORE) {
+        DEBUG("verifier() result      : IGNORE");
+        str            = (TNC_BufferReference)"ignore";
+        recommendation = TNC_IMV_ACTION_RECOMMENDATION_ISOLATE;
+        evaluation     = TNC_IMV_EVALUATION_RESULT_DONT_KNOW;
+    } else {
+        DEBUG("verifier() result      : ERROR");
+        str            = (TNC_BufferReference)"error";
+        recommendation = TNC_IMV_ACTION_RECOMMENDATION_ISOLATE;
+        evaluation     = TNC_IMV_EVALUATION_RESULT_ERROR;
+    }
+
+
     // Just for testing, provide a recommendation:
     // IMVs may tell the TNCS about languages and resons
+    len = strlen((char*)lang) + 1;
     setAttribute(
             imvID,
             connectionID,
             TNC_ATTRIBUTEID_REASON_LANGUAGE,
-            3,
-            (TNC_BufferReference)"en");
+            len,
+            lang);
+
+    len = strlen((char*)str) + 1;
     setAttribute(
             imvID,
             connectionID,
             TNC_ATTRIBUTEID_REASON_STRING,
-            8,
-            (TNC_BufferReference)"testing");
+            len,
+            str);
 
-    DEBUG_IFM("V->C imvID=%d, connectionID=%d - TNC_IMV_SolicitRecommendation\n",
+    DEBUG_IFM("[C<-V] imvID=%d, connectionID=%d - TNC_IMV_SolicitRecommendation\n",
         (int)imvID, (int)connectionID);
 
     return provideRecommendation(
                 imvID,
                 connectionID,
-                TNC_IMV_ACTION_RECOMMENDATION_ALLOW,
-                TNC_IMV_EVALUATION_RESULT_COMPLIANT);
+                recommendation,
+                evaluation);
 }
 
 /**
@@ -515,8 +933,8 @@ TNC_IMV_API TNC_Result TNC_IMV_SolicitRecommendation(
  *
  */
 TNC_IMV_API TNC_Result TNC_IMV_BatchEnding(
-/*in*/  TNC_IMVID imvID,
-/*in*/  TNC_ConnectionID connectionID) {
+    /*in*/  TNC_IMVID imvID,
+    /*in*/  TNC_ConnectionID connectionID) {
     DEBUG("TNC_IMV_BatchEnding\n");
 
     if (!initialized)
@@ -537,7 +955,7 @@ TNC_IMV_API TNC_Result TNC_IMV_BatchEnding(
  * TNC_IMV_Terminate (OPTIONAL)
  */
 TNC_IMV_API TNC_Result TNC_IMV_Terminate(
-/*in*/  TNC_IMVID imvID) {
+    /*in*/  TNC_IMVID imvID) {
     DEBUG("TNC_IMV_Terminate\n");
 
     if (!initialized)
@@ -573,13 +991,14 @@ static TNC_Result reportMessageTypes(
     if (!reportMessageTypesPtr)
         return TNC_RESULT_FATAL;
 
-    DEBUG_IFM("V->C imvID=%d - reportMessageTypes\n",
+    DEBUG_IFM("[C<-V] imvID=%d - reportMessageTypes\n",
         (int)imvID);
 
     // Call the function in the TMCC
     return (*reportMessageTypesPtr)(imvID, supportedTypes, typeCount);
 }
 
+#if 1
 /**
  * Call TNC_TNCS_SendMessage (MANDATORY) in the TNCS
  */
@@ -596,7 +1015,7 @@ static TNC_Result sendMessage(
         return TNC_RESULT_FATAL;
     }
 
-    DEBUG_IFM("V->C imvID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
+    DEBUG_IFM("[C<-V] imvID=%d, connectionID=%d, type=0x%x, msg[%d]\n",
         (int)imvID, (int)connectionID, (int)messageType, (int)messageLength);
 
     // Call the function in the TMCC
@@ -607,6 +1026,7 @@ static TNC_Result sendMessage(
                 messageLength,
                 messageType);
 }
+#endif
 
 #if 0
 // imv.c:343: error: ‘requestHandshakeRetry’ defined but not used
@@ -641,7 +1061,7 @@ static TNC_Result provideRecommendation(
     if (!provideRecommendationPtr)
         return TNC_RESULT_FATAL;
 
-    DEBUG_IFM("V->C imvID=%d, connectionID=%d - provideRecommendation\n",
+    DEBUG_IFM("[C<-V] imvID=%d, connectionID=%d - provideRecommendation\n",
         (int)imvID, (int)connectionID);
 
     return (*provideRecommendationPtr)(
@@ -694,7 +1114,7 @@ static TNC_Result setAttribute(
         return TNC_RESULT_FATAL;
     }
 
-    DEBUG_IFM("V->C imvID=%d, connectionID=%d - setAttribute\n",
+    DEBUG_IFM("[C<-V] imvID=%d, connectionID=%d - setAttribute\n",
         (int)imvID, (int)connectionID);
 
     return (*setAttributePtr)(
@@ -714,8 +1134,8 @@ static TNC_Result setAttribute(
  * TNC_IMV_ProvideBindFunction
  */
 TNC_IMV_API TNC_Result TNC_IMV_ProvideBindFunction(
-/*in*/  TNC_IMVID imvID,
-/*in*/  TNC_TNCS_BindFunctionPointer bindFunction) {
+    /*in*/  TNC_IMVID imvID,
+    /*in*/  TNC_TNCS_BindFunctionPointer bindFunction) {
     DEBUG("TNC_IMV_ProvideBindFunction\n");
 
     if (!initialized)
