@@ -26,23 +26,23 @@
  * \brief Generate Integrity Report from IML
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-01
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-06 SM
  *
  *  TSS Event Struct -> IR
  *  IML and PCR may not match, since the read them is not an atmic operation
  *
- Get PCR
- Get Event(IML)
- Event-> XML, calc PCR(c)
- get TPM PCR(t)
-
- PCR(c) == PCR(t)
-  then, next PCR
-  else, error or
-        try again or
-        if PCR(t) is old, ignore new events
-
-  TOCTOU?
+ *  Get PCR
+ *  Get Event(IML)
+ *  Event-> XML, calc PCR(c)
+ *  get TPM PCR(t)
+ *
+ *  PCR(c) == PCR(t)
+ *   then, next PCR
+ *   else, error or
+ *         try again or
+ *         if PCR(t) is old, ignore new events
+ *
+ *  TOCTOU?
  *
  *
  */
@@ -394,15 +394,15 @@ int writeStuffObjects(
         int eventindex) {
     char id[256];  // TODO(munetoh)
 
-    snprintf(id, sizeof(id), "PCR_%d_LV%d_%d_%d_EVENT",
-        event->ulPcrIndex, ss_level, event->eventType, eventindex);
-
     if (event == NULL) {
-        printf("ERROR: addStuffObjects, event == NULL\n");
+        ERROR("writeStuffObjects, event == NULL\n");
         return PTS_INTERNAL_ERROR;
     }
 
-    DEBUG_XML("addStuffObjects - pcr %d,id %s\n", event->ulPcrIndex, id);
+    snprintf(id, sizeof(id), "PCR_%d_LV%d_%d_%d_EVENT",
+        event->ulPcrIndex, ss_level, event->eventType, eventindex);
+
+    DEBUG_XML("writeStuffObjects - pcr %d,id %s\n", event->ulPcrIndex, id);
 
     /* start "stuff:Objects" */
     if (xmlTextWriterStartElement(
@@ -470,7 +470,7 @@ int writeStuffObjects(
                 (const char *) event->rgbEvent,
                 0,
                 event->ulEventLength) < 0) {
-            printf("SM DEBUG ERROR rgbEvent  len %d \n", event->ulEventLength);
+            ERROR("rgbEvent  len %d \n", event->ulEventLength);
             goto error;
         }
 
@@ -584,7 +584,7 @@ int writePcrHash(
 int writeSnapshot(
         xmlTextWriterPtr writer,
         OPENPTS_TPM_CONTEXT *tpm,
-        PTS_ComponentId cid,
+        PTS_ComponentId *cid,
         int index,
         OPENPTS_SNAPSHOT *ss) {
     int rc = PTS_SUCCESS;
@@ -661,7 +661,7 @@ int writeSnapshot(
         goto free;
     }
 
-    rc = writeComponentID(writer, &cid, index);
+    rc = writeComponentID(writer, cid, index);
 
     rc = writeDigestMethod(writer);
 
@@ -913,6 +913,7 @@ int writeQuote(
                 ERROR("Error at xmlTextWriterWriteAttribute\n");
                 return PTS_INTERNAL_ERROR;
             }
+
             /* Write a text, PCR, base64  */
             rc = xmlTextWriterWriteBase64(
                     writer,
@@ -1336,6 +1337,8 @@ int writeQuote2(
                 ERROR("Error at xmlTextWriterWriteAttribute\n");
                 return PTS_INTERNAL_ERROR;
             }
+
+
             /* Write a text, PCR, base64  */
             rc = xmlTextWriterWriteBase64(
                     writer,
@@ -1555,11 +1558,6 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
         goto freexml;
     }
 
-// #ifdef DEBUG
-//  // indent the XML :-)
-//  rc = xmlTextWriterSetIndent(writer, 1);
-// #endif
-
     /* Start the document */
     rc = xmlTextWriterStartDocument(writer, "1.0", XML_ENCODING, "no");
     if (rc < 0) {
@@ -1680,23 +1678,6 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
                     goto free;
                 }
             }
-#if 0
-#ifdef TSS_V11
-            rc = writeQuote(writer, ctx);
-            if (rc < 0) {
-                ERROR("writeIr - writeQuote() rc = %d\n", rc);
-                rc = PTS_INTERNAL_ERROR;
-                goto free;
-            }
-#else
-            rc = writeQuote2(writer, ctx);
-            if (rc < 0) {
-                ERROR("writeIr - writeQuote2() rc = %d\n", rc);
-                rc = PTS_INTERNAL_ERROR;
-                goto free;
-            }
-#endif
-#endif
         }
     } else {
         // DEBUG("iml.mode!=tss, skip writeQuote()\n");
@@ -1704,7 +1685,6 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
 
     /* add Snapshots */
     DEBUG_CAL("genIr - start snapshot\n");
-    // j = 0;
 
     for (i = 0; i < MAX_PCRNUM; i++) {
         /* level 0, platform */
@@ -1714,7 +1694,7 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
                 // level 0
                 // printf("DEBUG add level %d snapshot for PCR%d\n",ss->level, i);
                 // ERROR("writeIr PCR[%d] LV0 num=%d\n", i,ss->event_num);
-                writeSnapshot(writer, &tpm, cid, i, ss);
+                writeSnapshot(writer, &tpm, &cid, i, ss);
             }
         }
 
@@ -1723,7 +1703,7 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
         if (ss != NULL) {
             if (ss->event_num > 0) {
                 // ERROR("writeIr PCR[%d] LV1 num=%d\n", i,ss->event_num);
-                writeSnapshot(writer, &tpm, cid, i, ss);
+                writeSnapshot(writer, &tpm, &cid, i, ss);
             }
         }
     }  // PCR LOOP
@@ -1745,20 +1725,24 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
 
 
     /* set IR file */
-    // TODO function
-    if (ctx->conf->ir_filename != NULL) {
-        free(ctx->conf->ir_filename);
-    }
-    if (ctx->conf->ir_dir == NULL) {
-        ERROR("Set ir.dir at ptsc.conf. \n");
-        ctx->conf->ir_dir = smalloc("/tmp/.ptsc");
-    }
-    snprintf(filename, sizeof(filename), "%s_%s.xml",
-        ctx->str_uuid,
-        str_ir_uuid);
-
-    ctx->conf->ir_filename = getFullpathName(ctx->conf->ir_dir, filename);
+    if (file != NULL) {
+        /* use given filename */
+        ctx->conf->ir_filename = smalloc((char *)file);
+    } else {
+        /* use default filename */
+        if (ctx->conf->ir_filename != NULL) {
+            free(ctx->conf->ir_filename);
+        }
+        if (ctx->conf->ir_dir == NULL) {
+            ERROR("Set ir.dir at ptsc.conf. \n");
+            ctx->conf->ir_dir = smalloc("/tmp/.ptsc");
+        }
+        snprintf(filename, sizeof(filename), "%s_%s.xml",
+            ctx->str_uuid,
+            str_ir_uuid);
 
+        ctx->conf->ir_filename = getFullpathName(ctx->conf->ir_dir, filename);
+    }
 
     /* write to file */
     xmlFreeTextWriter(writer);
@@ -1857,6 +1841,7 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
     int rc = 0;
 
     ir_ctx->char_size = 0;
+    // memset(buf, 0, IR_SAX_BUFFER_SIZE);  // UNINIT
 
     if (!strcmp((char *)name, "Report")) {
         //
@@ -1946,6 +1931,7 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
     } else if (!strcmp((char *)name, "PcrComposit")) {
         // <PcrComposit>...
     } else if (!strcmp((char *)name, "PcrSelection")) {
+        int attr_cnt = 0;
         // <PcrSelection SizeOfSelect="3" PcrSelect="/6AA"/>
         //   SizeOfSelect => ctx->pcrs->pcr_select_size
         //   PcrSelect    => ctx->pcrs->pcr_select
@@ -1960,6 +1946,7 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
                     if (!strcmp(type, "SizeOfSelect")) {
                         /* TPM1.2 - 24 PCRS -> 3 */
                         pcrs->pcr_select_size = atoi(value);
+                        attr_cnt++;
                         // DEBUG("SizeOfSelect = %d\n", ir_ctx->pcr_select_size);
                     }
                     if (!strcmp(type, "PcrSelect")) {
@@ -1967,13 +1954,15 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
                             buf,  // ir_ctx->pcr_select,
                             (unsigned char *)value,
                             strlen(value));
+                        attr_cnt++;
                         // DEBUG("PcrSelect = 0x%02x %02x %02x \n", buf[0],buf[1],buf[2]);
                     }
                 }
             }
         }
         /* set pcr_select */
-        if (pcrs->pcr_select_size > 0) {
+        // if (pcrs->pcr_select_size > 0) {
+        if (attr_cnt == 2) {
             // ir_ctx->pcrs->pcr_select_byte
             if (pcrs->pcr_select_byte != NULL) {
                 free(pcrs->pcr_select_byte);
@@ -1984,6 +1973,9 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
             } else {
                 memcpy(pcrs->pcr_select_byte, buf, pcrs->pcr_select_size);
             }
+        } else {
+            /* BAD IR */
+            ERROR("BAD IR SizeOfSelect or PcrSelect are missing\n");
         }
     } else if (!strcmp((char *)name, "ValueSize")) {
         // <ValueSize>200</ValueSize>
@@ -2282,8 +2274,10 @@ void irEndElement(void * ctx, const xmlChar * name) {
                     memset(pcrs, 0, sizeof(OPENPTS_PCRS));
                     pctx->pcrs = pcrs;
                 }
-                pcrs->pcr_select[ir_ctx->pcr_index] = 1;
-                memcpy(pcrs->pcr[ir_ctx->pcr_index], ir_ctx->pcr, 20);  // TODO pcr size
+                // TODO PcrValue and PcrHash
+                // pcrs->pcr_select[ir_ctx->pcr_index] = 1;
+                // memcpy(pcrs->pcr[ir_ctx->pcr_index], ir_ctx->pcr, 20);  // TODO pcr size
+
             } else {
                 // DEBUG("iml.mode!=tss, skip pcr copy to PCRS\n");
             }
@@ -2296,9 +2290,9 @@ void irEndElement(void * ctx, const xmlChar * name) {
         rc = decodeBase64(
             buf,
             (unsigned char *)ir_ctx->buf,
-            ir_ctx->char_size);  //strlen(ir_ctx->buf));
+            ir_ctx->char_size);
         // TODO check rc
-        // TODO("CompositeHash %s", ir_ctx->buf);
+        // DEBUG("CompositeHash %s", ir_ctx->buf);
         memcpy(&validation_data->rgbData[32], buf, 20);
     } else if (!strcmp((char *)name, "ValueSize")) {
         // <ValueSize>200</ValueSize>
@@ -2313,14 +2307,18 @@ void irEndElement(void * ctx, const xmlChar * name) {
                 pcrs->pcr[ir_ctx->pcr_index],
                 (unsigned char *)ir_ctx->buf,
                 ir_ctx->char_size);
+        pcrs->pcr_select[ir_ctx->pcr_index] = 1;
+
+        // DEBUG("PCR[%d] base64=%s\n", ir_ctx->pcr_index,ir_ctx->buf);
+        // printHex("", (BYTE *)pcrs->pcr[ir_ctx->pcr_index], 20, "\n");
 
         // DEBUG("set reference PCR values\n");
         {
             // tpm.quote.pcr.0=base64
             // note) Do not use PCR10(IMA) as policy
-            char name[20];
-            snprintf(name, sizeof(name), "tpm.quote.pcr.%d", ir_ctx->pcr_index);
-            addProperty(ctx, name, ir_ctx->buf);
+            char buf2[20];
+            snprintf(buf2, sizeof(buf2), "tpm.quote.pcr.%d", ir_ctx->pcr_index);
+            addProperty(ctx, buf2, ir_ctx->buf);
         }
     } else if (!strcmp((char *)name, "QuoteInfo2")) {
         // set pcr select
@@ -2465,12 +2463,14 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
     }
     conf = ctx->target_conf;
 
+    /* new */
     if (ctx->pcrs == NULL) {
         /* malloc OPENPTS_PCRS */
         ctx->pcrs = malloc(sizeof(OPENPTS_PCRS));
         if (ctx->pcrs == NULL) {
             ERROR("no memory\n");
-            return PTS_FATAL;
+            rc = PTS_FATAL;
+            goto free;
         }
         memset(ctx->pcrs, 0, sizeof(OPENPTS_PCRS));
     }
@@ -2483,7 +2483,8 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
         ctx->ir_ctx = newIrContext();
         if (ctx->ir_ctx == NULL) {
             ERROR("no memory\n");
-            return PTS_FATAL;
+            rc = PTS_FATAL;
+            goto free;
         }
     }
 
@@ -2508,7 +2509,8 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
             rc = loadSQLiteDatabaseFile(ctx->aide_ctx, conf->aide_sqlite_filename);
             if (rc != PTS_SUCCESS) {
                 ERROR("loadSQLiteDatabaseFile fail\n");
-                return PTS_FATAL;
+                rc = PTS_FATAL;
+                goto free;
             }
 #else
             rc = loadAideDatabaseFile(ctx->aide_ctx, conf->aide_database_filename);
@@ -2541,7 +2543,8 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
     if ((rc = xmlSAXUserParseFile(&sax_handler, (void *)ctx, filename)) != 0) {
         // SAX parse error
         DEBUG("validateIr() - SAX parse error rc=%d\n", rc);
-        return rc;
+        rc = PTS_INTERNAL_ERROR;
+        goto free;
     } else {
         rc = ctx->ir_ctx->sax_error;
         DEBUG("validateIr() - SAX parse     : PASS (rc=%d)\n", rc);
@@ -2629,6 +2632,7 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
         rc = OPENPTS_RESULT_INVALID;
     }
 
+  free:
     /* free */
 
 #ifdef CONFIG_AIDE
@@ -2637,12 +2641,15 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
          freeAideContext(ctx->aide_ctx);
     }
 #endif
-
-    if (ctx->ir_ctx->buf != NULL) {
-        free(ctx->ir_ctx->buf);
+    // TODO Keep PCRs?
+    if (ctx->pcrs != NULL) {
+        free(ctx->pcrs);
+        ctx->pcrs = NULL;
     }
 
+
     if (ctx->ir_ctx != NULL) {
+        if (ctx->ir_ctx->buf != NULL) free(ctx->ir_ctx->buf);
         free(ctx->ir_ctx);
         ctx->ir_ctx = NULL;
     }
@@ -2684,16 +2691,18 @@ int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
     }
 
     /* read BIOS IML */
-    rc = getBiosImlFile(ctx, ctx->conf->bios_iml_filename, ctx->conf->iml_endian);
+    rc = readBiosImlFile(ctx, ctx->conf->bios_iml_filename, ctx->conf->iml_endian);
     if (rc != PTS_SUCCESS) {
         ERROR("fail to load BIOS IML, rc = %d\n", rc);
         return PTS_INTERNAL_ERROR;
     }
 
     if (ctx->conf->runtime_iml_filename != NULL) {
+        int count;
         /* read Runtime IML */
-        rc = getImaImlFile(ctx, ctx->conf->runtime_iml_filename, ctx->conf->runtime_iml_type, 0);  // TODO endian?
-        if (rc < 0) {
+        rc = readImaImlFile(ctx, ctx->conf->runtime_iml_filename,
+                ctx->conf->runtime_iml_type, 0, &count);  // TODO endian?
+        if (rc != PTS_SUCCESS) {
             ERROR("fail to load IMA IML, rc = %d\n", rc);
             return PTS_INTERNAL_ERROR;
         }
