@@ -643,7 +643,7 @@ int main(int argc, char *argv[]) {
 
     resetPcr();
 
-    ctx = malloc(sizeof(IR_CONTEXT));
+    // ctx = malloc(sizeof(IR_CONTEXT));
     ctx = (IR_CONTEXT *) malloc(sizeof(IR_CONTEXT));
     if (ctx == NULL) {
         ERROR("no memory\n");
