@@ -46,19 +46,20 @@ void writeLog(int priority, const char *format, ...) {
     int len;
     char *format2 = NULL;
     va_list list;
-    va_start(list, format);
     char buf[SYSLOG_BUF_SIZE];
 
     // TODO \n
     /* remove \n */
     len = strlen(format);
     if (format[len - 1] == '\n') {
-        format2 = malloc(len);
+        format2 = malloc(len + 1);  // +1 space
         memcpy(format2, format, len - 1);
         format2[len - 1] = 0;
         format = format2;
     }
 
+    va_start(list, format);
+
     // if (getenv("PTSCD_DAEMON") != NULL) {
     if (getenv("OPENPTS_SYSLOG") != NULL) {
         /* daemon -> syslog */
@@ -85,6 +86,7 @@ void writeLog(int priority, const char *format, ...) {
         fprintf(stdout, "\n");
     }
 
+    va_end(list);
     if (format2 != NULL) free(format2);
 }
 
