@@ -26,7 +26,7 @@
  * \brief misc functions
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-09-08
- * cleanup 2011-01-20 SM
+ * cleanup 2011-07-06 SM
  *
  */
 
@@ -375,7 +375,7 @@ char *getHexString(BYTE *bin, int size) {
     buf = malloc(size * 2 + 1);
     ptr = buf;
     for (i = 0; i < size; i++) {
-        len = snprintf(ptr, 3, "%02x", bin[i]);
+        len = snprintf(ptr, sizeof(ptr), "%02x", bin[i]);
         ptr += len;
     }
 
@@ -476,23 +476,42 @@ int getUint32(BYTE *buf) {
  *
  */
 int makeDir(char *dirname) {
-    int rc;
+    int rc = PTS_SUCCESS;
     struct stat st;
 
-    if (lstat(dirname, &st) == -1) {
-        /* Missing dir => create */
-        rc = mkdir(dirname, S_IRUSR | S_IWUSR | S_IXUSR |
-                            S_IRGRP | S_IWGRP | S_IXGRP);
-        if (rc != 0) {
-            ERROR("create directory, %s was failed\n", dirname);
-            return PTS_INTERNAL_ERROR;
+    /* create anyway */
+    rc = mkdir(dirname, S_IRUSR | S_IWUSR | S_IXUSR |
+                        S_IRGRP | S_IWGRP | S_IXGRP);
+    if (rc != 0) {
+        switch (errno) {
+        case EACCES:
+            ERROR("mkdir %s failed, EACCES", dirname);
+            rc = PTS_FATAL;
+            break;
+        case EEXIST:
+            /* already exist */
+            lstat(dirname, &st);
+            if ((st.st_mode & S_IFMT) != S_IFDIR) {
+                ERROR("directory, %s is not a directory %x %x\n", dirname, (st.st_mode & S_IFMT), S_IFDIR);
+                rc = PTS_INTERNAL_ERROR;
+            } else {
+                // OK
+                rc = PTS_SUCCESS;
+            }
+            break;
+        case EFAULT:
+            ERROR("mkdir %s failed, EFAULT", dirname);
+            rc = PTS_FATAL;
+            break;
+        // TODO add others :-)
+        default:
+            ERROR("mkdir %s failed, errono = 0x%X", dirname, errno);
+            rc = PTS_FATAL;
+            break;
         }
-    } else if ((st.st_mode & S_IFMT) != S_IFDIR) {
-        ERROR("directory, %s is not a directory %x %x\n", dirname, (st.st_mode & S_IFMT), S_IFDIR);
-        return PTS_INTERNAL_ERROR;
     }
 
-    return PTS_SUCCESS;
+    return rc;
 }
 
 /**
@@ -549,7 +568,3 @@ ssize_t wrapWrite(int fd, const void *buf, size_t count) {
     }
 }
 
-
-
-
-
