@@ -26,7 +26,7 @@
  * \brief main of openpts command
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-07-25
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-20 SM
  *
  * This is verifier and utility to maintain the collector/verifier
  *
@@ -51,7 +51,7 @@
 #include <dirent.h>
 #include <limits.h>
 
-#define USE_SCANDIR
+// #define USE_SCANDIR
 
 #include <openpts.h>
 
@@ -99,77 +99,74 @@ void usage(void) {
 #define NONE    4
 
 /**
- * print target info
+ * get Default Config File
  */
-int printTargetInfo(OPENPTS_CONFIG *conf, char *conf_dir, char *uuid, int cnt) {
-    char * target_conf_dir = NULL;
-    char * target_conf_filename = NULL;
+int getDefaultConfigfile(OPENPTS_CONFIG *conf) {
     int rc;
+    /* use system default config file */
+    char dirbuf[BUF_SIZE];
+    char confbuf[BUF_SIZE];
+    char uuidbuf[BUF_SIZE];
 
-    /* target dir, conf */
-    target_conf_dir = getFullpathName(conf_dir, uuid);
-    target_conf_filename = getFullpathName(target_conf_dir, "target.conf");
+    snprintf(dirbuf, BUF_SIZE, "%s/.openpts", getenv("HOME"));
+    snprintf(confbuf, BUF_SIZE, "%s/.openpts/openpts.conf", getenv("HOME"));
+    snprintf(uuidbuf, BUF_SIZE, "%s/.openpts/uuid", getenv("HOME"));
 
-    /* read target.conf */
-    rc = readTargetConf(conf, target_conf_filename);
-    if (rc != PTS_SUCCESS) {
-        ERROR("printTargetInfo() - readTargetConf failed\n");
-        ERROR("  uuid = %s\n", uuid);
-        ERROR("  conf_dir = %s\n", conf_dir);
-        return PTS_INTERNAL_ERROR;
+    /* check dir */
+    if (checkDir(dirbuf) != PTS_SUCCESS) {
+        char ans;
+        if (isatty(STDIN_FILENO)) {
+            // Console
+            char buf[256];
+            char *s;
+            printf("%s is missing. create [Y/n]:", dirbuf);
+            s = fgets(buf, sizeof(buf), stdin);
+            if (s !=NULL) {
+                ans = buf[0];
+            } else {
+                printf("bad input?");
+                goto error;
+            }
+        } else {
+            ans = 'Y';
+        }
+
+        /* new UUID */
+        conf->uuid = newOpenptsUuid();
+        conf->uuid->filename = smalloc(uuidbuf);
+        conf->uuid->status = OPENPTS_UUID_FILENAME_ONLY;
+        genOpenptsUuid(conf->uuid);
+
+        if ((ans == 'Y') || (ans == 'y')) {
+            rc = mkdir(dirbuf, S_IRUSR | S_IWUSR | S_IXUSR);
+            rc = writeOpenptsUuidFile(conf->uuid, 1);
+            if (rc != PTS_SUCCESS) {
+                ERROR("writeOpenptsUuidFile fail\n");
+            }
+            rc = writeOpenptsConf(conf, confbuf);
+
+        } else {
+            printf("Bad answer %c, exit\n", ans);
+            rc = -1;
+            goto error;
+        }
     }
 
-    printf("%s\n", LINE);
-    printf("target[%d] uuid            : %s\n", cnt, uuid);
-    printf("target[%d] config          : %s\n", cnt, target_conf_filename);
-    printf("target[%d] hostname        : %s\n", cnt, conf->hostname);
-    if (conf->ssh_username != NULL) {
-        printf("target[%d] SSH remote user : %s\n", cnt, conf->ssh_username);
-    }
-    if (conf->ssh_port != NULL) {
-        printf("target[%d] SSH port        : %s\n", cnt, conf->ssh_port);
-    }
+    /* check conf  */
 
+    DEBUG("read conf file          : %s\n", confbuf);
+    rc = readOpenptsConf(conf, confbuf);
+    if (rc != 0) {
+        ERROR("readOpenptsConf() failed\n");
+        goto error;
+    }
     return PTS_SUCCESS;
+
+  error:
+    return PTS_FATAL;
 }
 
 
-#ifdef USE_SCANDIR   // scandir or opendir
-/**
- * selectDir
- *
- * select target dir
- */
-// TODO use selectUuidDir in uuid.c
-static int selectDir(const struct dirent *entry) {
-    int len;
-#ifndef __linux__
-    struct stat buffer;
-#endif
-
-    /* skip . .. dirs */
-    if (0 == strcmp(".", entry->d_name)) return 0;
-    if (0 == strcmp("..", entry->d_name)) return 0;
-
-    /* UUID */
-    /* skip bad dir name - by length */
-    len = strlen(entry->d_name);
-    // TODO ("UUID dirname len = %d, %s\n",len, entry->d_name);
-    if (len != 36) return 0;
-
-    /* Dir HIT */
-#ifndef __linux__
-    if (stat(entry->d_name, &buffer) != 0) return 0;
-    if (S_ISDIR(buffer.st_mode)) return 1;
-#else
-    if (entry->d_type == DT_DIR) return 1;
-#endif
-
-    return 0;
-}
-
-#endif
-
 /**
  * main of "openpts" command 
  *
@@ -184,7 +181,6 @@ int main(int argc, char *argv[]) {
 
     OPENPTS_CONFIG *conf = NULL;  // conf for openpts
     OPENPTS_CONTEXT *ctx = NULL;
-    // char * execpath = NULL;
     char * config_filename = NULL;
     char * target_hostname = NULL;
     int initialized = 0;  // 0 -> 1 -> 2
@@ -206,7 +202,6 @@ int main(int argc, char *argv[]) {
     char *ptsc_conf = NULL;
 
     verbose = 0;
-    // execpath = argv[0];
 
     /* args */
     while ((opt = getopt(argc, argv, "ivuDVc:dfuyl:p:P:C:h")) != -1) {
@@ -277,7 +272,7 @@ int main(int argc, char *argv[]) {
 
     target_hostname = argv[0];
 
-
+    /* check */
     if ((ptsc_path != NULL) && (ptsc_conf != NULL)) {
         int len;
         INFO("ptsc debug mode\n");
@@ -308,53 +303,12 @@ int main(int argc, char *argv[]) {
         return -1;
     }
 
-    /* check/create config file - HOME./openpts/openpts.conf */
+    /* check/create config file */
     if (config_filename == NULL) {
-        /* use system default config file */
-        char dirbuf[BUF_SIZE];
-        char confbuf[BUF_SIZE];
-        char uuidbuf[BUF_SIZE];
-
-        snprintf(dirbuf, BUF_SIZE, "%s/.openpts", getenv("HOME"));
-        snprintf(confbuf, BUF_SIZE, "%s/.openpts/openpts.conf", getenv("HOME"));
-        snprintf(uuidbuf, BUF_SIZE, "%s/.openpts/uuid", getenv("HOME"));
-
-        /* check dir */
-        if (checkDir(dirbuf) != PTS_SUCCESS) {
-            char ans;
-            if (isatty(STDIN_FILENO)) {
-                printf("%s is missing. create [Y/n]:", dirbuf);
-                rc = scanf("%[YyNn]", &ans);
-            } else {
-                rc = 1;
-                ans = 'Y';
-            }
-
-            /* new UUID */
-            conf->uuid = newOpenptsUuid();
-            conf->uuid->filename = smalloc(uuidbuf);
-            conf->uuid->status = OPENPTS_UUID_FILENAME_ONLY;
-            genOpenptsUuid(conf->uuid);
-
-            if ((ans == 'Y') || (ans == 'y')) {
-                rc = mkdir(dirbuf, S_IRUSR | S_IWUSR | S_IXUSR);
-                rc = writeOpenptsUuidFile(conf->uuid, 1);
-                rc = writeOpenptsConf(conf, confbuf);
-
-            } else {
-                printf("Bad answer %c, exit\n", ans);
-                rc = -1;
-                goto error;
-            }
-        }
-
-        /* check conf  */
-
-        DEBUG("read conf file          : %s\n", confbuf);
-        rc = readOpenptsConf(conf, confbuf);
-
-        if (rc != 0) {
-            ERROR("readOpenptsConf() failed\n");
+        /* use default config file, HOME./openpts/openpts.conf  */
+        rc = getDefaultConfigfile(conf);
+        if (rc != PTS_SUCCESS) {
+            ERROR("getDefaultConfigfile() failed\n");
             rc = -1;
             goto error;
         } else {
@@ -373,137 +327,86 @@ int main(int argc, char *argv[]) {
         }
     }
 
-    /* check/create target conf dir */
-    if (target_hostname != NULL) {
-        /* look up the conf of target(hostname) */
+    /* Display (no remote access) */
+    if (command == DISPLAY) {
+        /* target List */
         rc = getTargetList(conf, conf->config_dir);
         if (rc != PTS_SUCCESS) {
             TODO("main() - getTargetList rc =%d\n", rc);
         }
-        if (verbose & DEBUG_FLAG) {
-            DEBUG("target list\n");
-            printTargetList(conf, "");  // uuid.c
-        }
 
-        /* set the target hostname:port and search */
-        if (conf->hostname != NULL) {
-            TODO("realloc conf->hostname\n");
-            free(conf->hostname);
-        }
-        conf->hostname = smalloc(target_hostname);
-        target_collector =  getTargetCollector(conf);
-        if (target_collector == NULL) {
-            /* missing, new target => malloc temp target */
-            new_target = 1;
-            target_conf = newPtsConfig();
+        if (target_hostname != NULL) {
+            /* given target (by hostname) */
+            /* look up */
+            conf->hostname = smalloc(target_hostname);
+            target_collector =  getTargetCollector(conf);
+            if (target_collector != NULL) {
+                target_conf = (OPENPTS_CONFIG*)target_collector->target_conf;
+
+                printf("hostname  : %s\n", target_hostname);
+                printf("UUID      : %s\n", target_collector->str_uuid);
+                printf("State     : %d\n", target_collector->state);
+                printf("Dir       : %s\n", target_collector->dir);
+                printf("Manifests :\n");
+
+                getRmList(target_conf, target_conf->config_dir);
+                printRmList(target_conf, "");
+            } else {
+                printf("hostname  : %s --- unknown host, check the list\n", target_hostname);
+            }
+            goto free;  // exit
         } else {
-            /* HIT exist */
-            target_conf_dir  = getTargetConfDir(conf);  // HOME/.openpts/UUID
-            target_conf_filename = smalloc(target_collector->target_conf_filename);
-            target_conf = (OPENPTS_CONFIG*)target_collector->target_conf;
-        }
-
-    } else {
-        if (command == DISPLAY) {
-#ifdef USE_SCANDIR  // scandir
-            int cnt = 0;
-            int dir_num;
-            struct dirent **dir_list;
-            char * base_conf_dir;
-
-            DEBUG("USE_SCANDIR\n");
-
-            printf("Show openpts config\n");
-            printf("%s\n", LINE);
-            printf("config file               : %s\n", conf->config_file);
-            printf("uuid                      : %s\n", conf->uuid->str);
-
-            /* move to config dir */
-            if ((chdir(conf->config_dir)) != 0) {
-                fprintf(stderr, "Accessing config directory %s\n", conf->config_dir);
-                rc = -1;
-                goto error;
-            }
-
-            /* scan dirs */
-            dir_num = scandir(".", &dir_list, &selectDir, NULL);
-            if ( dir_num == -1 ) {
-                fprintf(stderr, "no target data\n");
-                rc = -1;
-                goto error;
-            }
-
-            base_conf_dir = smalloc(conf->config_dir);
-
-            // TODO read conf modify the conf structure
-            for (cnt = 0; cnt < dir_num; cnt++) {
-                DEBUG("Config %d at %s %s\n", cnt, base_conf_dir, dir_list[cnt]->d_name);
-                printTargetInfo(conf, base_conf_dir, dir_list[cnt]->d_name, cnt);
-                free(dir_list[cnt]);
-             }
-            free(dir_list);
-            free(base_conf_dir);
-
-            printf("%s\n", LINE);
-
-#else  // opndir
-            DIR *dir;
-            struct dirent *de;
-            int cnt = 0;
-            char * base_conf_dir;
-
-            ERROR("DDDD\n");
-
-            printf("Show openpts config\n");
-            printf("%s\n", LINE);
-            printf("config file               : %s\n", conf->config_file);
-            // printf("uuid                      : %s\n", conf->str_uuid);
-
-            if ((dir = opendir(conf->config_dir)) == NULL) {
-                ERROR("TODO\n");
-                rc = -1;
-                goto error;
-            }
-
-            base_conf_dir = smalloc(conf->config_dir);
-
-            // TODO use readdir_r
-            for (de = readdir(dir); de != NULL; de = readdir(dir)) {
-                if (de->d_type == DT_DIR) {
-                    if (!strncmp(de->d_name, "..\0", 3)) {
-                        // skip parents dir
-                    } else if (!strncmp(de->d_name, ".\0", 2)) {
-                        // skip this dir
-                    } else {
-                        printTargetInfo(conf, base_conf_dir, de->d_name, cnt);
-                        cnt++;
-                    }
-                }
-            }
-            closedir(dir);
-            free(base_conf_dir);
-            printf("%s\n", LINE);
-#endif
-            rc = 0;
-            goto free;
-        } else {
-            usage();
-            rc = -1;
-            goto error;
+            /* all target (simple) */
+            printTargetList(conf, "");  // target.c
+            goto free;  // exit anyway
         }
     }
-    if (command == DISPLAY) {
-        printTargetInfo(conf, conf->config_dir, target_hostname, 0);
-        printf("%s\n", LINE);
-        rc = 0;
+
+
+    /* Other commands use Remote Access (SSH) */
+
+    if (target_hostname == NULL) {
+        printf("set the target.\n");
+        usage();
         goto free;
     }
 
+    /* check/create target conf dir */
+    /* look up the conf of target(hostname) */
+    rc = getTargetList(conf, conf->config_dir);
+    if (rc != PTS_SUCCESS) {
+        TODO("main() - getTargetList rc =%d\n", rc);
+    }
+    if (verbose & DEBUG_FLAG) {
+        DEBUG("target list\n");
+        printTargetList(conf, "");  // uuid.c
+    }
+
+    /* set the target hostname:port and search */
+    if (conf->hostname != NULL) {
+        TODO("realloc conf->hostname\n");
+        free(conf->hostname);
+    }
+    conf->hostname = smalloc(target_hostname);
+    target_collector =  getTargetCollector(conf);
+    if (target_collector == NULL) {
+        /* missing, new target => malloc temp target */
+        new_target = 1;
+        target_conf = newPtsConfig();
+    } else {
+        /* HIT exist */
+        target_conf_dir  = getTargetConfDir(conf);  // HOME/.openpts/UUID
+        target_conf_filename = smalloc(target_collector->target_conf_filename);
+        target_conf = (OPENPTS_CONFIG*)target_collector->target_conf;
+    }
+
     /* verify -> read target conf */
     if (command == VERIFY) {
         /* check the target dir  */
         if (checkDir(target_conf_dir) == PTS_SUCCESS) {
             initialized++;  // 1->2
+        } else {
+            ERROR("%s is missing", target_conf_dir);
         }
     }
 
@@ -529,10 +432,8 @@ int main(int argc, char *argv[]) {
         ssh_port = target_conf->ssh_port;
     }
 
-    // TODO reset old settings if exist (previous config)
+    // TODO Wrong?
     if (command == INIT) {
-        // TODO ("reset conf side\n");
-        // target_collector->target_conf->ssh_mode = 0;
         if (target_conf->hostname != NULL) {
             // DEBUG("realloc target_conf->hostname\n"); TODO realloc happen
             free(target_conf->hostname);
@@ -551,7 +452,12 @@ int main(int argc, char *argv[]) {
     ctx->target_conf = target_conf;
 
     /* command */
-    if (command == INIT) {
+    switch (command) {
+    case INIT:
+        /* */
+        ctx->target_conf = NULL;
+
+
         /* get UUID, RMs, AIDE DB */
         DEBUG("enroll with %s  (SSH uses localost)\n", target_hostname);
         DEBUG("conf->config_dir %s\n", conf->config_dir);
@@ -562,14 +468,6 @@ int main(int argc, char *argv[]) {
             goto error;
         }
 
-        /* gen  initial policy  and AIDE-ignorelist */
-        // DEBUG("Generate initial validation policy for %s\n", target);
-        // rc = readTargetConf(ctx->conf, ctx->target_conf_filename);  // TODO BAD use new conf for the target
-        // if (rc != PTS_SUCCESS) {
-        //     ERROR(" readTargetConf failed\n");
-        //     // TODO so?
-        // }
-
         DEBUG("conf->config_dir %s\n", conf->config_dir);
         rc =  verifier(ctx, target_hostname, ssh_username, ssh_port, conf->config_dir, 1);  // init
         if (rc != OPENPTS_RESULT_VALID) {
@@ -580,7 +478,6 @@ int main(int argc, char *argv[]) {
 
         /* message */
         printf("Target            : %s\n", target_hostname);
-        printf("Collector UUID    : %s\n", ctx->target_conf->uuid->str);
 
         if (ctx->target_conf != NULL) {
             if (ctx->target_conf->rm_uuid != NULL) {
@@ -589,17 +486,15 @@ int main(int argc, char *argv[]) {
                     printf("manifest[%d]       : %s\n", i, ctx->target_conf->rm_filename[i]);
                 }
             }
+            printf("Collector UUID    : %s\n", ctx->target_conf->uuid->str);
             printf("configuration     : %s\n", ctx->target_conf->config_file);
             printf("validation policy : %s\n", ctx->target_conf->policy_filename);
         } else {
             // TODO never happen?
             printf("configuration     : new target\n");
         }
-
-        // printf("validation policy : %s\n", ctx->conf->policy_filename);
-        // printf("aide database     : %s\n", ctx->conf->aide_database_filename);
-        // printf("aide ignore list  : %s\n", ctx->conf->aide_ignorelist_filename);
-    } else if (command == VERIFY) {
+        break;
+    case VERIFY:
         /* verify */
         if (initialized < 2) {
             fprintf(stderr, "ERROR: target %s is not initialized yet. please enroll with %s first\n\n",
@@ -666,22 +561,20 @@ int main(int argc, char *argv[]) {
             printReason(ctx);
         }
 
-
 #ifdef CONFIG_AUTO_RM_UPDATE
         // Verify() check the ARU
         // remote : conf->aru_newrm_uuid
-        // local  : target_conf->newrm_uuid 
+        // local  : target_conf->newrm_uuid
         if ((target_conf != NULL) && (conf->newrm_exist > 0)) {
-            //char uuid_str[40];  // 37?
             char *uuid_str;
             PTS_DateTime *uuid_time;
             char ans;
             int same = 0;
 
-            if (verbose) {
+            if (verbose & DEBUG_FLAG) {
                 DEBUG("NEWRM_UUID\n");
                 printHex("NEWRM UUID (remote)  : ", (BYTE*)conf->aru_newrm_uuid, 16, "\n");
-                if (target_conf->newrm_uuid->uuid != NULL) {
+                if (target_conf->newrm_uuid != NULL) {
                     printHex("NEWRM UUID (local)   : ", (BYTE*)target_conf->newrm_uuid->uuid, 16, "\n");
                 } else {
                     printf("NEWRM UUID (local)   : missing\n");
@@ -689,7 +582,7 @@ int main(int argc, char *argv[]) {
             }
 
             /* Check local newrm vs remote newrm */
-            if (target_conf->newrm_uuid->uuid != NULL) {
+            if ((target_conf->newrm_uuid != NULL) && (target_conf->newrm_uuid->uuid != NULL)) {
                 if (memcmp(
                         (BYTE*)conf->aru_newrm_uuid,
                         (BYTE*)target_conf->newrm_uuid->uuid, 16) == 0) {
@@ -706,16 +599,13 @@ int main(int argc, char *argv[]) {
                     goto free;
                 } else {
                     /* local is old? */
-                    //printf("New reference manifest has been received, but update exist\n");
                     same = 1;
                 }
             }
 
             /* msg */
             printf("---------------------------------------------------------\n");
-            //encodeBase64((unsigned char *)uuid_str, (unsigned char *)conf->aru_newrm_uuid, 16);
             uuid_time = getDateTimeOfUuid(conf->aru_newrm_uuid);
-            // base64 printf("New Manifest UUID : %s ", uuid_str);
             uuid_str = getStringOfUuid(conf->aru_newrm_uuid);
             printf("New Manifest UUID : %s ", uuid_str);
             printf("(date: %04d-%02d-%02d-%02d:%02d:%02d)\n",
@@ -728,7 +618,6 @@ int main(int argc, char *argv[]) {
             free(uuid_str);
 
             if (same == 0) {
-                // printf("New reference manifest exist. update? [Y/n]\n");
                 if (isatty(STDIN_FILENO)) {
                     printf("New reference manifest exist. update? [Y/n]\n");
                     rc = scanf("%[YyNn]", &ans);
@@ -769,25 +658,25 @@ int main(int argc, char *argv[]) {
             printf("New reference manifest exist. if this is expected change, update the manifest by openpts -i -f \n");
         }
 #endif
-    } else if (command == UPDATE) {
-       TODO("TBD\n");
-    } else {
+        break;
+    case UPDATE:
+        TODO("TBD\n");
+        break;
+    default:
+        ERROR("Bad command?");
         usage();
+        break;
     }
-
     rc = 0;
 
  error:
  free:
     if (target_conf_dir != NULL) free(target_conf_dir);
-
     if (target_conf_filename != NULL) free(target_conf_filename);
-
     if ((new_target == 1) && (target_conf != NULL)) {
         /* free new target conf */
         freePtsConfig(target_conf);
     }
-
     if (ctx != NULL) {
         ctx->target_conf = NULL;
         freePtsContext(ctx);
