@@ -28,7 +28,7 @@
  * @author Olivier Valentin <olivier.valentin@us.ibm.com>
  * @author Alexandre Ratchov <alexandre.ratchov@bull.net>
  * @date 2010-04-04
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-06 SM
  *
  */
 
@@ -69,6 +69,7 @@ int collector2(OPENPTS_CONFIG *conf) {
     OPENPTS_CONTEXT *ctx = NULL;
     PTS_IF_M_Attribute *read_tlv = NULL;
     int count = 0;
+    int len;
 
     /* Init RMs */
     rc = getRmSetDir(conf);
@@ -146,7 +147,10 @@ int collector2(OPENPTS_CONFIG *conf) {
                     INFO("verifier (UUID=%s)\n", ctx->str_uuid);
 
                     /* send PTS_CAPABILITIES msg. to verifier (=UUID) */
-                    rc = writePtsTlv(ctx, STDOUT, OPENPTS_CAPABILITIES);
+                    len = writePtsTlv(ctx, STDOUT, OPENPTS_CAPABILITIES);
+                    if (len < 0) {
+                        ERROR("send OPENPTS_CAPABILITIES was failed\n");
+                    }
                     TODO("IF-M OPENPTS_CAPABILITIES rc=%d (0x%x)\n", rc, rc);
                 }
                 break;
@@ -166,8 +170,11 @@ int collector2(OPENPTS_CONFIG *conf) {
                     rc = getDhResponce(ctx->nonce);
 
                     /* send responce */
-                    rc = writePtsTlv(
+                    len = writePtsTlv(
                             ctx, STDOUT, DH_NONCE_PARAMETORS_RESPONSE);
+                    if (len < 0) {
+                        ERROR("send DH_NONCE_PARAMETORS_RESPONSE was failed\n");
+                    }
                 }
                 break;
             case DH_NONCE_FINISH:
@@ -207,8 +214,11 @@ int collector2(OPENPTS_CONFIG *conf) {
                 if (read_tlv->length != 0) {
                     ERROR("Bad REQUEST__RIMM_SET, len = %d != 0\n", read_tlv->length);
                 } else {
-                    rc = writePtsTlv(
+                    len = writePtsTlv(
                             ctx, STDOUT, RIMM_SET);
+                    if (len < 0) {
+                        ERROR("send RIMM_SET was failed\n");
+                    }
                 }
                 break;
             case REQUEST_NEW_RIMM_SET:
@@ -217,8 +227,11 @@ int collector2(OPENPTS_CONFIG *conf) {
                 if (read_tlv->length != 0) {
                     ERROR("Bad REQUEST_NEW_RIMM_SET, len = %d != 0\n", read_tlv->length);
                 } else {
-                    rc = writePtsTlv(
+                    len = writePtsTlv(
                             ctx, STDOUT, NEW_RIMM_SET);
+                    if (len < 0) {
+                        ERROR("sendNEW_RIMM_SET was failed\n");
+                    }
                     TODO("IF-M REQUEST_NEW_RIMM_SET, len = %d\n", rc);
                 }
                 break;
@@ -228,7 +241,10 @@ int collector2(OPENPTS_CONFIG *conf) {
                 if (read_tlv->length != 0) {
                     ERROR("Bad REQUEST_INTEGRITY_REPORT, len = %d != 0\n", read_tlv->length);
                 } else {
-                    rc = writePtsTlv(ctx, STDOUT, INTEGRITY_REPORT);
+                    len = writePtsTlv(ctx, STDOUT, INTEGRITY_REPORT);
+                    if (len < 0) {
+                        ERROR("send INTEGRITY_REPORT was failed\n");
+                    }
                     TODO("IF-M INTEGRITY_REPORT len=%d (0x%x)\n", rc, rc);
                 }
                 break;
@@ -245,7 +261,10 @@ int collector2(OPENPTS_CONFIG *conf) {
                 if (read_tlv->length != 0) {
                     ERROR("Bad REQUEST_AIDE_DATABASE, len = %d != 0\n", read_tlv->length);
                 } else {
-                    rc = writePtsTlv(ctx, STDOUT, AIDE_DATABASE);
+                    len = writePtsTlv(ctx, STDOUT, AIDE_DATABASE);
+                    if (len < 0) {
+                        ERROR("send AIDE_DATABASE was failed\n");
+                    }
                 }
                 break;
 #endif
@@ -254,7 +273,10 @@ int collector2(OPENPTS_CONFIG *conf) {
                 if (read_tlv->length != 0) {
                     ERROR("Bad REQUEST_TPM_PUBKEY, len = %d != 0\n", read_tlv->length);
                 } else {
-                    rc = writePtsTlv(ctx, STDOUT, TPM_PUBKEY);  // ifm.c
+                    len = writePtsTlv(ctx, STDOUT, TPM_PUBKEY);  // ifm.c
+                    if (len < 0) {
+                        ERROR("send TPM_PUBKEY was failed\n");
+                    }
                 }
                 break;
             case NONCE:
@@ -282,26 +304,24 @@ int collector2(OPENPTS_CONFIG *conf) {
                 ctx->ifm_errno = PTS_UNRECOGNIZED_COMMAND;
                 if (ctx->ifm_strerror != NULL) free(ctx->ifm_strerror);
                 ctx->ifm_strerror = smalloc("Unknown message type");
-                rc = writePtsTlv(ctx, STDOUT, OPENPTS_ERROR);  // ifm.c
+
+                len = writePtsTlv(ctx, STDOUT, OPENPTS_ERROR);  // ifm.c
+                if (len < 0) {
+                    ERROR("send OPENPTS_ERROR was failed\n");
+                }
                 terminate = 1;
                 break;
             }  // switch case
 
             /* free TLV */
             if (read_tlv != NULL) {
-                if (read_tlv->value != NULL) {
-                    free(read_tlv->value);
-                }
-                free(read_tlv);
+                freePtsTlv(read_tlv);
             }
         }  // GET loop
         /* out */
         /* free TLV for break out */
         if (read_tlv != NULL) {
-            if (read_tlv->value != NULL) {
-                free(read_tlv->value);
-            }
-            free(read_tlv);
+            freePtsTlv(read_tlv);
         }
     } while (terminate == 0);
 
@@ -332,7 +352,7 @@ void usage(void) {
     fprintf(stderr, NLS(1, 13, "  -v                    Verbose mode. Multiple -v options increase the verbosity.\n"));
     fprintf(stderr, "\n");
     fprintf(stderr, NLS(1, 14, "Options:\n"));
-    fprintf(stderr, NLS(1, 15, "  -c configfile         Set configuration file. defalt is %s\n"), PTSCD_CONFIG_FILE);
+    fprintf(stderr, NLS(1, 15, "  -c configfile         Set configuration file. defalt is %s\n"), PTSC_CONFIG_FILE);
     // fprintf(stderr, NLS(1, 14, "  -f                    foreground, run in the foreground."));
     // fprintf(stderr, NLS(1, 15, "                        Logging goes to stderr " "instead of syslog.\n"));
     fprintf(stderr, NLS(1, 16, "  -P name=value         Set properties.\n"));
@@ -495,7 +515,7 @@ int main(int argc, char *argv[]) {
 #endif
 
 
-    // TODO
+    // TODO chgrp
     rc = preparePriv();
     if (rc != PTS_SUCCESS) {
         ERROR("preparePriv fail\n");
@@ -590,10 +610,10 @@ int main(int argc, char *argv[]) {
 
     /* load config */
     if (config_filename == NULL) {
-        DEBUG("config file               : %s\n", PTSCD_CONFIG_FILE);
-        rc = readPtsConfig(conf, PTSCD_CONFIG_FILE);
+        DEBUG("config file               : %s\n", PTSC_CONFIG_FILE);
+        rc = readPtsConfig(conf, PTSC_CONFIG_FILE);
         if (rc != PTS_SUCCESS) {
-            ERROR("read config file, '%s' was failed - abort\n", PTSCD_CONFIG_FILE);
+            ERROR("read config file, '%s' was failed - abort\n", PTSC_CONFIG_FILE);
             goto free;
         }
     } else {
