@@ -26,7 +26,7 @@
  * \brief Reference Manifest (RM)
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-01
- * cleanup 2011-01-22 SM
+ * cleanup 2011-07-20 SM
  *
  * IML + Model -> RM
  *
@@ -200,13 +200,15 @@ int writeAllCoreValues(xmlTextWriterPtr writer, OPENPTS_SNAPSHOT * ss) {
         // link was set by getIml, BHV-FSM
         bhv_trans = eventWrapper->fsm_trans;  // EW keeps the link to BHV
         if (bhv_trans == NULL) {
-            UINT32 i;
-            ERROR("writeAllCoreValues() - BHV Trans is missing\n");
-            printf("\tpcrindex=%d, eventype=%d, digest=",
-                event->ulPcrIndex, event->eventType);
-            for (i = 0;i < event->ulPcrValueLength; i++)
-                printf("%02x", event->rgbPcrValue[i]);
-            printf("\n");
+            DEBUG("writeAllCoreValues() - BHV Trans is missing\n");
+            if (verbose & DEBUG_FLAG) {
+                UINT32 i;
+                printf("\tpcrindex=%d, eventype=%d, digest=",
+                    event->ulPcrIndex, event->eventType);
+                for (i = 0;i < event->ulPcrValueLength; i++)
+                    printf("%02x", event->rgbPcrValue[i]);
+                printf("\n");
+            }
             return -1;
         }
 
@@ -700,35 +702,6 @@ int writeCoreAssertionInfo(xmlTextWriterPtr writer, OPENPTS_CONTEXT * ctx, int l
                 goto error;
             }
         }
-
-#if 0
-        if (level == 0) {
-            if (ss->event_num > 0) {  // TODO(munetoh) check FSM
-                // level 0
-                rc = writeValidationModel(writer, ss);
-                if (rc < 0) {
-                    ERROR("writeCoreAssertionInfo - pcr[%d] level 2\n", i);
-                    rc = -1;
-                    goto error;
-                }
-            }
-        } else if (level == 1) {
-            if (ss->next != NULL) {
-                ss = ss->next;
-                if (ss->event_num > 0) {  // TODO(munetoh) check FSM
-                    // level 1
-                    rc = writeValidationModel(writer, ss);
-                    if (rc < 0) {
-                        ERROR("writeCoreAssertionInfo - pcr[%d], level 1\n", i);
-                        rc = -1;
-                        goto error;
-                    }
-                }
-            }
-        } else {
-            ERROR("level %d is NA\n", level);
-        }
-#endif
     }  // SS(PCR) LOOP
 
     if (xmlTextWriterEndElement(writer) < 0)  // ValidationModels
@@ -919,7 +892,11 @@ int writeRm(OPENPTS_CONTEXT * ctx, const char *file, int level) {
                 /* update Binary FSM using IML */
                 rc = writeAllCoreValues(writer, ss);
                 if (rc < 0) {
-                    ERROR("writeRm() - bad IML or FSM at pcr=%d, level=%d\n", i, level);
+                    addReason(ctx,
+                        "[RM] The manifest generation was failed at pcr=%d, level=%d", i, level);
+                    addReason(ctx,
+                        "[RM] The validation model may not support this platform. "
+                        "Report this to openpts-users@lists.sourceforge.jp.");
                     rc = PTS_INTERNAL_ERROR;
                     goto free;
                 }
@@ -1449,25 +1426,16 @@ int makeRmSetDir(OPENPTS_CONFIG *conf) {
     int i;
 
     if (conf->rm_basedir != NULL) {
-        struct stat st;
+        // struct stat st;
         char buf[BUF_SIZE];
 
         snprintf(buf, BUF_SIZE, "%s/%s",
                 conf->rm_basedir,
                 conf->rm_uuid->str);
 
-        if (lstat(buf, &st) == -1) {
-            /* Missing conf dir => create */
-            rc = mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR |
-                            S_IRGRP | S_IWGRP | S_IXGRP);
-            if (rc != 0) {
-                ERROR("create conf directory, %s was failed\n", buf);
-                rc = PTS_INTERNAL_ERROR;
-                goto end;
-            }
-            // DEBUG("RM DIR  : %s\n", buf);
-        } else if ((st.st_mode & S_IFMT) != S_IFDIR) {
-            ERROR("RM directory, %s is not a directory %x %x\n", buf, (st.st_mode & S_IFMT), S_IFDIR);
+        rc = makeDir(buf);
+        if (rc != PTS_SUCCESS) {
+            ERROR("create conf directory, %s was failed\n", buf);
             rc = PTS_INTERNAL_ERROR;
             goto end;
         }
@@ -1479,7 +1447,6 @@ int makeRmSetDir(OPENPTS_CONFIG *conf) {
                 conf->rm_basedir,
                 conf->rm_uuid->str, i);
             conf->rm_filename[i] = smalloc(buf);
-            // DEBUG("RM File : %s\n", conf->rm_filename[i]);
         }
     }
     rc = PTS_SUCCESS;
@@ -1497,25 +1464,14 @@ int makeNewRmSetDir(OPENPTS_CONFIG *conf) {
     int i;
 
     if (conf->rm_basedir != NULL) {
-        struct stat st;
         char buf[BUF_SIZE];
 
         snprintf(buf, BUF_SIZE, "%s/%s",
                 conf->rm_basedir,
                 conf->newrm_uuid->str);
 
-        if (lstat(buf, &st) == -1) {
-            /* Missing conf dir => create */
-            rc = mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR |
-                            S_IRGRP | S_IWGRP | S_IXGRP);
-            if (rc != 0) {
-                ERROR("create conf directory, %s was failed\n", buf);
-                rc = PTS_INTERNAL_ERROR;
-                goto end;
-            }
-            // DEBUG("RM DIR  : %s\n", buf);
-        } else if ((st.st_mode & S_IFMT) != S_IFDIR) {
-            ERROR("RM directory, %s is not a directory %x %x\n", buf, (st.st_mode & S_IFMT), S_IFDIR);
+        rc = makeDir(buf);
+        if (rc != PTS_SUCCESS) {
             rc = PTS_INTERNAL_ERROR;
             goto end;
         }
@@ -1532,7 +1488,6 @@ int makeNewRmSetDir(OPENPTS_CONFIG *conf) {
                 conf->rm_basedir,
                 conf->newrm_uuid->str, i);
             conf->newrm_filename[i] = smalloc(buf);
-            // DEBUG("RM File : %s\n", conf->rm_filename[i]);
         }
     }
     rc = PTS_SUCCESS;
