@@ -26,7 +26,7 @@
  * \brief create pseudo IML of tboot, standalone tool
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2011-03-28
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-06 SM
  * refactoring
  *
  *
@@ -170,7 +170,6 @@ int parseTxtStatFile(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
 
     /* line by line */
     while (fgets(line, sizeof(line), fp) != NULL) {  // read line
-
         // TBOOT: v2 LCP policy data found
         if (!strncmp(line, "TBOOT: v2 LCP policy data found", 31)) {
             ctx->lcp_policy_version = 2;
@@ -226,10 +225,8 @@ int parseTxtStatFile(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
                 ctx->sinit_hash[j] = hex2byte(line, 1 + j * 3);
             }
             next_state = TXTSTAT_NA;
-            //if (verbose > 0) {
             DEBUG("sinit_hash\n");
             debugPrintHex("  sinit_hash : ", ctx->sinit_hash, 20, "\n");
-            //}
         }
         // TBOOT: 	 mle_hash:
         if (!strncmp(line, "TBOOT: 	 mle_hash:", 18)) {
@@ -423,7 +420,7 @@ void removecr(char *str) {
 // 000004c0  aa 3a c0 7f a7 46 db 18  2e ac 69 8f 8d 41 7f 5a  |.:...F....i..A.Z|
 // 0x4c0 = 1216
 int checkSinitAcm(BYTE *buf) {
-    BYTE UUID[16] = 
+    BYTE UUID[16] =
         {0xaa, 0x3a, 0xc0, 0x7f, 0xa7, 0x46, 0xdb, 0x18,
          0x2e, 0xac, 0x69, 0x8f, 0x8d, 0x41, 0x7f, 0x5a};
 
@@ -464,23 +461,31 @@ typedef struct {
 int sinit_acm_hash(char *filename, int size, BYTE *sha1_digest, BYTE *sha256_digest) {
     FILE *fp;
     char buf[2048];
-    char *acmbuf;
+    char *acmbuf = NULL;
     char *ptr;
     SINIT_ACM *acm;
     SHA_CTX sha_ctx;
     SHA256_CTX sha256_ctx;
     int len;
     int user_area;
+    int rc = PTS_SUCCESS;
 
     DEBUG("sinit_acm_hash() file = %s, size = %d\n", filename, size);
 
     acmbuf = malloc(size);
+    if (acmbuf == NULL) {
+        ERROR("no memory");
+        rc = PTS_FATAL;
+        goto error;
+    }
+    memset(acmbuf, 0, size);
 
     /* open */
     fp = fopen(filename, "rb");
     if (fp == NULL) {
         ERROR("File %s does not exist\n", filename);
-        return 0;
+        rc = PTS_FATAL;
+        goto error;
     }
 
     /* load */
@@ -521,8 +526,9 @@ int sinit_acm_hash(char *filename, int size, BYTE *sha1_digest, BYTE *sha256_dig
 
     debugPrintHex(" SHA256 Digest : ", sha256_digest, 32, "\n");
 
+  error:
     free(acmbuf);
-    return 0;
+    return rc;
 }
 
 
@@ -715,7 +721,7 @@ int emulateTboot(OPENPTS_TBOOT_CONTEXT *ctx) {
     resetPcr();
 
     // PCR 17
-    // Ref: Dev Guide 1.9.1 PCR 17 - p.14 
+    // Ref: Dev Guide 1.9.1 PCR 17 - p.14
     if (ctx->mle_version == 6) {
         // Extend(SHA-1(SinitMleData.SinitHash |
         //              SinitMleData.EdxSenterFlags))
@@ -766,15 +772,15 @@ int emulateTboot(OPENPTS_TBOOT_CONTEXT *ctx) {
         //              SinitMleData.MsegValid |
         //              SinitMleData.StmHash |
         //              SinitMleData.PolicyControl |
-        //              SinitMleData.LcpPolicyHash | 
+        //              SinitMleData.LcpPolicyHash |
         //              (OsSinitData.Capabilities, 0)))
         //
-        // SHA-1(SinitMleData.SinitHash | 
+        // SHA-1(SinitMleData.SinitHash |
         //       SHA-1( SinitMleData.BiosAcm.ID |
         //              SinitMleData.MsegValid |
         //              SinitMleData.StmHash |
         //              SinitMleData.PolicyControl |
-        //              SinitMleData.LcpPolicyHash | 
+        //              SinitMleData.LcpPolicyHash |
         //              (OsSinitData.Capabilities, 0)))
 
         // SHA256(sinit)
@@ -791,11 +797,11 @@ int emulateTboot(OPENPTS_TBOOT_CONTEXT *ctx) {
             debugPrintHex("  PCR[17]            : ", &pcr[17][0], 20, "\n");
         }
 
-        //extend(17, ctx->sinit_hash256_from_file);
-        //extend(17, ctx->sinit_hash_from_file);
-        //debugPrintHex("  mle v7 PCR17 ", &pcr[17][0], 20, "  (SINIT ACM)\n");
+        // extend(17, ctx->sinit_hash256_from_file);
+        // extend(17, ctx->sinit_hash_from_file);
+        // debugPrintHex("  mle v7 PCR17 ", &pcr[17][0], 20, "  (SINIT ACM)\n");
 
-        // Force 
+        // Force
         resetPcrWithSecret(17, ctx->sinit_hash);
         debugPrintHex("  SINIT hash   ", ctx->sinit_hash, 20, "\n");
         debugPrintHex("  mle v7 PCR17 ", &pcr[17][0], 20, "\n");
@@ -826,8 +832,8 @@ int emulateTboot(OPENPTS_TBOOT_CONTEXT *ctx) {
         extend(17, ctx->sinit_hash256_from_file);
         debugPrintHex("  mle v8 PCR17 ", &pcr[17][0], 20, "\n");
 
-        // Force 
-        //resetPcrWithSecret(17, ctx->sinit_hash);
+        // Force
+        // resetPcrWithSecret(17, ctx->sinit_hash);
         //
         SHA1_Init(&sha_ctx);
         SHA1_Update(&sha_ctx, ctx->bios_acm_id, 20);
@@ -852,8 +858,6 @@ int emulateTboot(OPENPTS_TBOOT_CONTEXT *ctx) {
     printHex("PCR-19", &pcr[19][0], 20, "\n");
 #endif
 
-
-
     extend(17, ctx->vl_pcr17);
     debugPrintHex("  extend  : ", ctx->vl_pcr17, 20, "\n");
     debugPrintHex("  PCR[17] : ", &pcr[17][0], 20, "\n");
@@ -906,6 +910,7 @@ int writeEvent(FILE *fp, TSS_PCR_EVENT *event) {
 
 /**
  *
+ * filename IML(binary) file
  */
 int generateEventlog(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
     int rc = PTS_SUCCESS;
@@ -928,8 +933,18 @@ int generateEventlog(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
 
     /* event  */
     event = malloc(sizeof(TSS_PCR_EVENT));
+    if (event == NULL) {
+        ERROR("no memory");
+        goto free;
+    }
     memset(event, 0, sizeof(TSS_PCR_EVENT));
+
+    /* PCR/digest */
     event->rgbPcrValue = malloc(20);
+    if (event->rgbPcrValue == NULL) {
+        ERROR("no memory");
+        goto free;
+    }
 
     /* */
     if (ctx->mle_version == 6) {
@@ -1057,6 +1072,10 @@ int generateEventlog(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
         eventdata = module->eventdata;
         event->ulEventLength = 20 + 20 + 4 + 4 + eventdata->command_size + eventdata->filename_size;
         event->rgbEvent = malloc(event->ulEventLength);
+        if (event->rgbEvent == NULL) {
+            ERROR("no memory");
+            goto free;
+        }
         ptr = event->rgbEvent;
         memcpy(ptr, eventdata->command_hash, 20);
         ptr += 20;
@@ -1073,6 +1092,7 @@ int generateEventlog(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
         // DEBUG("%s\n", eventdata->filename);
     } else {
         event->ulEventLength = 0;
+        event->rgbEvent = NULL;
         DEBUG("  module[0] eventdata = null, check the default value in the grub.conf\n");
     }
     memcpy(event->rgbPcrValue, ctx->vl_pcr18, 20);
@@ -1095,7 +1115,12 @@ int generateEventlog(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
         }
         eventdata = module->eventdata;
         event->ulEventLength = 20 + 20 + 4 + 4 + eventdata->command_size + eventdata->filename_size;
+        if (event->rgbEvent != NULL) free(event->rgbEvent);
         event->rgbEvent = malloc(event->ulEventLength);
+        if (event->rgbEvent == NULL) {
+            ERROR("no memory");
+            goto free;
+        }
         ptr = event->rgbEvent;
         memcpy(ptr, eventdata->command_hash, 20);
         ptr += 20;
@@ -1112,14 +1137,19 @@ int generateEventlog(OPENPTS_TBOOT_CONTEXT *ctx, char *filename) {
         // DEBUG("%s\n", eventdata->filename);
     } else {
         event->ulEventLength = 0;
+        event->rgbEvent = NULL;
         DEBUG("  module[0] eventdata = null, check the default value in the grub.conf\n");
     }
     memcpy(event->rgbPcrValue, ctx->vl_pcr19, 20);
     writeEvent(fp, event);
 
 
-    free(event->rgbPcrValue);
-    free(event);
+  free:
+    if (event != NULL) {
+        if (event->rgbPcrValue != NULL) free(event->rgbPcrValue);
+        if (event->rgbEvent != NULL) free(event->rgbEvent);
+        free(event);
+    }
 
     /* close */
     if (filename != NULL) {
