@@ -679,7 +679,7 @@ int main(int argc, char *argv[]) {
     /* Close TSS/TPM */
 
   close:
+    if (str != NULL) memset(str, 0, len);
     Tspi_Context_Close(hContext);
-    memset(str, 0, len);
     return result;
 }
