@@ -27,7 +27,7 @@
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-08-18
  * refactoring 2011-02-15 SM
- * cleanup 2011-04-26 SM
+ * cleanup 2011-07-20 SM
  *
  * Create Sign Key
  * Create AIK
@@ -88,7 +88,7 @@ int deleteTssKey(PTS_UUID *uuid, int ps_type) {
 int createAIK() {
     /* dummy */
     TODO("createAIK - TBD\n");
-    return TSS_FATAL;
+    return TSS_E_FAIL;
 }
 
 int getTssPubKey(
@@ -328,7 +328,7 @@ int setTpmStatus(TSS_FLAG flag, TSS_BOOL value, int tpm_password_mode) {
                 flag,  // TSS_TPMSTATUS_RESETLOCK,
                 value);
     if (result != TSS_SUCCESS) {
-        ERROR("Tspi_Policy_SetSecret failed rc=0x%x\n",
+        ERROR("Tspi_Policy_SetSecret failed rc=0x%x, TPM requires ownerauth, use tpm_resetdalock command\n",
                result);
         goto close;
     }
@@ -368,9 +368,6 @@ int printTssKeyList(int ps_type) {
     }
 
     /* List */
-    // buf = (BYTE *) & SRK_UUID;
-    // printhex("SRK uuid: ", buf, 16);
-
     result = Tspi_Context_GetRegisteredKeysByUUID(
                 hContext,
                 (UINT32) ps_type,  // TSS_PS_TYPE_SYSTEM,
@@ -383,16 +380,9 @@ int printTssKeyList(int ps_type) {
         goto close;
     }
 
-    // info = ppKeyHierarchy[0];
     printf("Key number   : %d\n", ulKeyHierarchySize);
     for (i = 0; i < (int)ulKeyHierarchySize; i++) {
         printf("Key %d\n", i);
-        // buf = (BYTE *) & info->versionInfo;
-        // printhex(" version     : ", buf, 4);
-        // buf = (BYTE *) & info->keyUUID;
-        // printhex(" uuid        : ", buf, 16);
-        // buf = (BYTE *) & info->parentKeyUUID;
-        // printhex(" parents uuid: ", buf, 16);
         info = info + 1;
     }
 
@@ -429,7 +419,6 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
     BYTE *keyBlob;
     int i;
     TSS_UUID tss_uuid;
-    // BYTE *buf;
     TSS_FLAG initFlag = TSS_KEY_SIZE_2048 | TSS_KEY_TYPE_SIGNING;
 
     /* Open TSS */
@@ -498,13 +487,10 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
         goto close;
     }
 
-
     /* UUID */
-    // uuit_t -> TSS_UUID
     memcpy(&tss_uuid, uuid, sizeof(TSS_UUID));
 
     /* Create New Key object */
-
     result = Tspi_Context_CreateObject(hContext,
                                        TSS_OBJECT_TYPE_RSAKEY,
                                        initFlag, &hKey);
@@ -514,7 +500,6 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
         goto close;
     }
 
-
     /* create Key */
     result = Tspi_Key_CreateKey(hKey, hSRK, 0);
     if (result != TSS_SUCCESS) {
@@ -524,7 +509,6 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
     }
 
     /* RegisterKey */
-
     if (ps_type == TSS_PS_TYPE_BLOB) {
         /* save as blob */
         FILE *fp;
@@ -586,8 +570,6 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
                     ERROR("Tspi_Context_RegisterKey failed rc=0x%x\n",
                      result);
                     ERROR("       TSS_E_KEY_ALREADY_REGISTERED\n");
-                    // buf = (BYTE *) & uuid;
-                    // printhex("       uuid=", buf, 16);
                 }
             } else {
                 ERROR("spi_Context_RegisterKey failed rc=0x%x\n",
@@ -602,7 +584,6 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
   close:
     /* Close TSS/TPM */
     Tspi_Context_Close(hContext);
-    // memset(str, 0, len);
 
     return result;
 }
@@ -640,10 +621,8 @@ int deleteTssKey(PTS_UUID *uuid, int ps_type) {
     }
 
     /* UUID */
-    // uuit_t -> TSS_UUID
     memcpy(&tss_uuid, uuid, sizeof(TSS_UUID));
 
-
     /* delete key */
     result =
         Tspi_Context_UnregisterKey(hContext,
@@ -717,7 +696,6 @@ int getTssPubKey(
     }
 
     /* UUID */
-    // uuit_t -> TSS_UUID
     memcpy(&tss_uuid, uuid, sizeof(TSS_UUID));
 
     /* load key */
@@ -773,7 +751,6 @@ int getTssPubKey(
 
     // TODO resetDaLock
 
-
     /* Load AIK or Sign key */
     if (ps_type == 0) {
         /* Blob file */
@@ -806,7 +783,7 @@ int getTssPubKey(
                     &hKey);
         if (result == 0x803) {
             ERROR("TPM is locked. use 'tpm_resetdalock' command to clear the lock\n");
-            ERROR("For the ptscd daemon, set the flag, 'tpm.resetdalock=on' in ptscd.conf ");
+            ERROR("For the ptsc, set the flag, 'tpm.resetdalock=on' in /etc/ptsc.conf, or use tpm_resetdalock command");
             goto close;
         } else if (result != TSS_SUCCESS) {
             ERROR("Tspi_Context_LoadKeyByUUID (Key) failed rc=0x%x\n", result);
@@ -852,21 +829,13 @@ int getTssPubKey(
         goto free;
     }
     /* copy to local */
-#if 0
-    if (conf->pubkey != NULL) {
-        // DEBUG("realloc conf->pubkey\n");  // TODO realloc happen
-        free(conf->pubkey);
-    }
-    conf->pubkey = malloc(conf->pubkey_length);
-    memcpy(conf->pubkey, buf, conf->pubkey_length);
-#else
     if (*pubkey != NULL) {
         // DEBUG("realloc conf->pubkey\n");  // TODO realloc happen
         free(*pubkey);
     }
     *pubkey = malloc(*pubkey_length);
     memcpy(*pubkey, buf, *pubkey_length);
-#endif
+
 
   free:
     Tspi_Context_FreeMemory(hContext, NULL);
@@ -999,7 +968,6 @@ int quoteTss(
     int pcrSelectCount = 0;
 
     /* UUID */
-    // uuit_t -> TSS_UUID
     memcpy(&tss_uuid, uuid, sizeof(TSS_UUID));
 
     /* Connect to TCSD */
@@ -1228,7 +1196,11 @@ int quoteTss(
     // total 48-bytes
     validationData->ulDataLength = validation_data.ulDataLength;
     validationData->rgbData = malloc(validation_data.ulDataLength);
-    // TODO check
+    if (validationData->rgbData == NULL) {
+        ERROR("no memory");
+        result = PTS_FATAL;
+        goto free;
+    }
     memcpy(
         validationData->rgbData,
         validation_data.rgbData,
@@ -1239,7 +1211,11 @@ int quoteTss(
     /* rgbData */
     validationData->ulDataLength = 20;
     validationData->rgbData = malloc(20);
-    // TODO check
+    if (validationData->rgbData == NULL) {
+        ERROR("no memory");
+        result = PTS_FATAL;
+        goto free;
+    }
     memcpy(
         validationData->rgbData,
         &validation_data.rgbData[8],
@@ -1250,7 +1226,11 @@ int quoteTss(
     /* rgbValidationData */
     validationData->ulValidationDataLength = validation_data.ulValidationDataLength;
     validationData->rgbValidationData = malloc(validation_data.ulValidationDataLength);
-    // TODO check
+    if (validationData->rgbValidationData == NULL) {
+        ERROR("no memory");
+        result = PTS_FATAL;
+        goto free;
+    }
     memcpy(
         validationData->rgbValidationData,
         validation_data.rgbValidationData,
@@ -1669,7 +1649,11 @@ int quote2Tss(
     // total 75-bytes???
     validationData->ulDataLength = validation_data.ulDataLength;
     validationData->rgbData = malloc(validation_data.ulDataLength);
-    // TODO check
+    if (validationData->rgbData == NULL) {
+        ERROR("no memory");
+        result = PTS_FATAL;
+        goto free;
+    }
     memcpy(
         validationData->rgbData,
         validation_data.rgbData,
@@ -1679,7 +1663,11 @@ int quote2Tss(
     /* rgbValidationData */
     validationData->ulValidationDataLength = validation_data.ulValidationDataLength;
     validationData->rgbValidationData = malloc(validation_data.ulValidationDataLength);
-    // TODO check
+    if (validationData->rgbValidationData == NULL) {
+        ERROR("no memory");
+        result = PTS_FATAL;
+        goto free;
+    }
     memcpy(
         validationData->rgbValidationData,
         validation_data.rgbValidationData,
@@ -2351,11 +2339,16 @@ int validatePcrCompositeV12(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData)
         rc = PTS_SUCCESS;
     } else {
         DEBUG("validatePcrCompositeV12() - bad digest\n");
+        if (verbose & DEBUG_FLAG) {
+            printHex("  calc    :", digest, 20, "\n");
+            printHex("  given   :", composit_hash, 20, "\n");
+        }
     }
 
     if (rc != PTS_SUCCESS) {
         /* why? */
         DEBUG("validatePcrCompositeV12() - validation fail, rc = %d\n", rc);
+        // 34 PTS_VERIFY_FAILED
     }
 
     /* free */
